Option Strict Off
Imports System.Math
Module MondrianMod1
    '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    '   Mondrian model                                          
    '
    ' Copyright 2007-2020 William S. Currie 
    '
    ' Licensed under the Apache License, Version 2.0 (the "License");
    ' you may Not use this file except In compliance With the License.
    ' You may obtain a copy Of the License at
    '
    ' http://www.apache.org/licenses/LICENSE-2.0
    '
    ' Unless required by applicable law Or agreed To In writing, software
    ' distributed under the License Is distributed On an "AS IS" BASIS,
    ' WITHOUT WARRANTIES Or CONDITIONS Of ANY KIND, either express Or implied.
    ' See the License For the specific language governing permissions And
    ' limitations under the License.
    '
    '   Mondrian is a model for probing nonlinear interactions in plant competition and community-ecosystem level
    ' processes and outcomes. Designed originally for use in collaborative projects to study plant 
    ' competition, plant invasions, and other nonequilibrium phenomena in marshes in Great Lakes lakeshores.
    '
    '   CAUTION:  this is a very complex individual-based model.  It would be difficult for someone who was 
    ' not involved in model development to make self-consistent changes because many parts of the model 
    ' depend on other parts in subtle but important ways.  
    '
    '   A detailed, comprehensive user guide is available for this model. See the user guide for 
    ' a wide range of information including parameter definitions, modeling of ecological processes,
    ' setting up a workflow for model runs, and interpreting output files.  The user guide also 
    ' contains an Appendix with conventions for coding, arrays, and naming of variables in the code. 
    '
    '   Citation: Currie WS, Goldberg DE, Martina J, Wildova R, Farrer E, Elgersma KJ (2014) 
    ' Emergence of nutrient-cycling feedbacks related to plant size and invasion success in a wetland 
    ' community-ecosystem model. Ecological Modelling 282:69-82
    '
    '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    'Declare GUI forms as objects
    Public myIntfcMain As New frmIntfcMain
    Public myInitial As New frmInitial
    Public myEndMsg As New frmEndMsg
    Public myProgress As New frmProgress
    Public myLRSProgress As New frmLRSProgress

    'Constants to be used in defining other variables and operations 
    Const maxyrs As Short = 100  'maximum number of years that the user could specify for length of each run

    'Plant individuals, grid system, and pools
    'These need to be public if the progress form is going to have access to draw them on screen during run
    'Note limit of individuals per species = maxindivs and limit of gents per species is maxgenets; both can 
    ' be changed here and all code that uses these should work.  
    Public maxindivs As Integer          'max number of individuals that can be created in a model run, by species
    Public maxgenets As Short            'max number of genets that can be created in a run, by species 
    Public iindmax(5) As Integer         'tracks highest indiv index number used so far, by species 
    Private igenmax(5) As Short          'tracks highest genet number used so far, by species
    Public SppInclude(5) As Boolean      'True if the species is included in this model run
    Public pil(5, maxindivs, 2) As Single
    Public pgr(5, maxindivs, 10) As Integer
    Public pga(5, maxindivs) As Single
    Public PIP(5, maxindivs, 7) As Single
    Private Phgt(5, maxindivs) As Single  'height (m) of each individual from plant top to plant base (i.e. stem-rhizome interface)
    Private Pdsubmerged(5, maxindivs) As Integer  'number of days each individual has been continuously submerged under water 
    'Note maximum size of grid is set here
    Public GCP(25, 25, 10) As Single
    Public GCPinit(10) As Single          'initial pool sizes of grid cell pools, in GCP array, at start of each run
    Public ncells As Short                'number of cells in entire grid (must be a square, e.g 9, 16, 25 ...)
    Public s As Single                    'length of side of one cell in the grid system (m) 
    Public GridixMax, GridiyMax As Short  'highest-used cell numbers in x and y dimensions of grid system
    Public GridArea As Single             'Area of the simulated study plot (m2)
    'Species and grid cell information that does not need to be public 
    Private SppName(5) As String          'Name of each species
    Private SppCsum(5) As Single          'C sum over all plants, by species: for printout
    Private SppAGCsum(5) As Single        'Aboveground C sum over all plants by species:  for printout 
    Private SppNsum(5) As Single          'N sum over all plants, by species: for printout
    Private SppRhiz(5) As Short           'number of living individual rhizomes by species
    Private SppStems(5) As Short          'number of living individual stems by species
    Private gckcalcs(25, 25, 2) As Single 'grid cell litter input C mass-weighted C mass * k
    Private kL(25, 25) As Single          'first-order decay constant of litter lumped within a grid cell
    Private RNP(2) As Single              'Regional Nutrient Pools; an array that holds values of nutrient pools 
    Private EcosystemC, EcosCinout, EcosystemN, EcosNinout, Cmassbal, Nmassbal As Single
    'More detailed info to print for plant species 1 only, for model testing
    Private Sp1AGCsum As Single   'Sp 1 aboveground C sum
    Private Sp1AGNsum As Single   'Sp 1 aboveground N sum
    Private Sp1BGCsum As Single   'Sp 1 belowground C sum
    Private Sp1BGNsum As Single   'Sp 1 belowgroung N sum
    Private Sp1AGnumber As Single 'Sp 1 number of stems with aboveground biomass
    Private Sp1BGnumber As Single 'Sp 1 number of stems with belowground biomass

    'Species-specific plant parameters
    Private initRamC(5) As Single          'spp-specific proportion of max C used to initialize ramets at start of model run
    Private initBGCNrat(5) As Single       'spp-specific growth target C/N ratio in belowground plant tissue
    Private initAGCNrat(5) As Single       'spp-specific growth target C/N ratio in aboveground plant tissue
    Private maxPlantAGC(5) As Single       'spp-specific max AboveGround plant indiv size, as C pool, g C
    Private maxPlantBGC(5) As Single       'spp-specific max BelowGround plant indiv size, as C pool, g C
    Private PlantType(5) As Short          'spp-specific plant type: 1 = clonal, annual plant with ramets and rhizomes
    Private resorbProp(5) As Single        'spp-specific proportion of foliar N resorbed during foliar senescence
    Private maxLongev(5) As Short          'spp-specific maximum longevity of plant individuals, in years
    Private PlantLitProp(5) As Single      'spp-specific parameter controlling amount or rate of aboveground litterfall 
    Private kLit(5) As Single              'spp-specific first-order decay constant for plant litter
    Private relGrate(5) As Single          'spp-specific relative growth rate, per day
    Private rhizRetCProp(5) As Single      'spp-specific proportion of active belowground C that parent rhizome retains
    Private rhizAllocCProp(5) As Single    'spp-specific proportion of max belowground C needed to make a daughter rhizome
    Private cidist(5) As Single            'spp-specific clonal internode distance, from parent to daughter ramet
    Private AGEstparmA(5) As Single        'spp-specific reln btwn litter thickness and bg C & N needed to establish new ag stem
    Private CallocBGprop(5) As Single      'spp-specific proportion of photosynthate allocated to belowground rhizome 
    Private MaxVRangle(5) As Single        'spp-specific max directional angle, from straight line, in making new rhizomes (nodes)
    Private PBGCNNmin(5) As Single         'spp-specific mimumum value of PropBGCNneed variable
    Private SubsidyPar(5) As Short         'spp-specific number of parents back in the chain that a daughter can get subsidy from
    Private FloodMortDays(5) As Short      'spp-specific number of days of stem being continuously submerged that causes mortality 
    Private AlloBioHgtA(5), AlloBioHgtB(5) As Single  'spp-specific parameters in allometric equations to calculate height from biomass 
    Private LgtExtCurveA(5), LgtExtCurveB(5), LgtExtCurveC(5) As Single  'spp-specific parameters in light extinction curve 
    Private LgtEHeight(5) As Single        'spp-specific light environment height:  proportion of indiv height at which light envt is calculated 
    Private BioFullShade(5) As Single      'spp-specific living plant biomass above which light extinction is forced to 97%
    Private TbranchProb(5) As Single       'spp-specific:  during clonal reproduction, probability of terminal branching (0 to 1)
    Private LbranchProb(5) As Single       'spp-specific:  during clonal reproduction, probability of lateral branching (0 to 1)    

    'Plant parameters that are not species-specific
    Public CIDistCostParm As Single       'C cost of internode distance (per m) as a proportion of max belowground size of individual 
    Public BGpersist As Short             'years (growing seasons) that BG rhizome can remain alive without making an aboveground stem
    Private LTflooddepth As Single        'Long-term flood depth (m).  This depth for 2 years kills all plants.  

    'Other biogeochemical parameters
    Private FlushPropMult As Single       'Multiplier for hydrologic flushing parameter read from scenario file, unitless 
    Private DetExpProp As Single          'Proportion of detrital biomass exported from each grid cell pool, 1/year
    Private kMSOM, kMUCK As Single        'First-order decay constants for MSOM and MUCK when aerobic, 1/year
    Private LitAnmod, MuckAnmod, MSOMAnmod As Single   'Modifiers to decomp rates for anaerobic conditions
    Private kBioturb As Single            'First-order const. used to set rate of transfer from MUCK to MSOM, representing bioturbation + particulate matter
    Private GCPhumProp As Single          'Controls transfer of litter to later-stage MSOM pool, as a proportion of decomp mass loss
    Private LitCritCNRat As Single        'Critical C:N ratio of litter:  below this, litter mineralizes N 
    Private HumifCNrat As Single          'C:N ratio of material passed from litter to MSOM, representing partial humification
    Private TeffLitOMparm As Single                 'Parameter that controls strength of effect of litter mass on OM temperatures
    Private bdAGL, bdBGL, bdMUCK, bdMSOM As Single  'bulk densities of litter and detritus pools
    Private cconcMUCK, cconcMSOM As Single          'organic C concentrations in detritus pools.  cconcMSOM is initialized by user but a dynamic variable. 
    Private cconcTissue As Single                   'C concentration per unit dw for plant tissue and fresh litter
    Private AGLvertdim(2), BGLvertdim(2) As Single  'top and bottom edges of each litter pool in the vertical dimension (m)
    Private MUCKvertdim(2), MSOMvertdim(2) As Single 'top and bottom edges of each detrital pool in the vertical dimension (m)
    Private NH4vertdim(2) As Single        'top and bottom edges of available NH4 pool in the vertical dimension (m)
    Private NO3vertdim(2) As Single        'top and bottom edges of available NO3 pool in the vertical dimension (m) 
    Private AGLThicknessSum As Single      'Sum used in calculating average litter layer thickness over entire space 
    Private AGLThicknessAvg As Single      'Average litter layer thickness over entire space
    Private MUCKThicknessSum As Single      'Sum used in calculating average muck thickness over entire space 
    Private MUCKThicknessAvg As Single      'Average muck thickness over entire space
    Private BGLspread As Single            'Multiplier that spreads belowground litter over a greater depth than it would be as pure litter 
    Private MSOMdepth As Single            'Hard-wired depth of MSOM pool (m)
    Private NitrifParm As Single           'Controls rate of nitrification where NH4 is available and conditions are aerobic
    Private DntrParm As Single             'Controls rate of denitrification where NO3 is available and conditions are anaerobic
    Private AZdepth As Single              'Active Zone depth (m):  In the MSOM, available NH4 and NO3 pools are assumed located within this depth
    Private HetRespBench As Single         'Benchmark for heterotrophic respiration used in denitrification submodel 
    Private CH4P0 As Single                'Proportion of heterotrophic resp C that is CH4-C production under ideal conditions for CH4 production 
    Private CH4oxIn As Single              'Proportion of CH4-C production that is oxidized in the wetland under inundated conditions 
    Private CH4oxAer As Single             'Proportion of CH4-C production that is oxidized in the wetland when the Muck is aerobic
    Private DntrN2OProp As Single          'Determines proportion of denitrification N flux that is N2O-N; the remainer is N2-N 
    Private N2Op1d As Single               'N2O-N proportion of denit. N flux when active zone has been inundated 1 day
    Private N2Op4d As Single               'N2O-N proportion of denit. N flux when active zone has been inundated 4 days
    Private N2Oplt As Single               'N2O-N proportion of denit. N flux when active zone has been inundated longer-term
    Private N2Opdepth As Single            'Depth (meters) at which a change in inundation activates the different values of N2O-N proportion 

    'Variables related to scenario file
    ' Baseyear and Runyears need to be public because graphics form uses them
    Public Baseyear As Short     'initialization year of model run
    Public Runyears As Short     'number of years to run
    Private NH4NinflowMult As Single   'Multiplier for daily flux of NH4-N inflow from hydr transport, rivers and groundwater from land, unitless
    Private NO3NinflowMult As Single   'Multiplier for daily flux of NO3-N inflow from hydr transport, rivers and groundwater from land, unitless
    Private NH4NinputPpt As Single     'Flux of NH4-N input form precipitation, g N/m2 y
    Private NO3NinputPpt As Single     'Flux of NO3-N input form precipitation, g N/m2 y
    Private TambAnnAvgStart As Single  'ambient temperature (C), annual average, at start of model run
    Private TambAnnAvgEnd As Single    'ambient temperature (C), annual average, at end of model run
    Private TambMaxDev As Single       'ambient temperature (C), if seasonal sinusoidal function, max deviation from ann average
    Private TdayAir, TdayWater As Single      'Daily average temperature of air and water (degrees C)  
    Private Waterlvl(maxyrs, 365) As Single   'water level (year, day) in meters: read in from scenario file
    Private WaterlvlDay As Single             'water level on current day, in meters 
    Private WaterlvlLast5days(5) As Single    'Water level info saved for last 5 days
    Private TrailAvgWlvl As Single         'Trailing average water level (m) calculated over the past few days
    Private NH4NinputLand(maxyrs, 365) As Single 'NH4-N input from land, read in from scenario file, g N/m2 day
    Private NO3NinputLand(maxyrs, 365) As Single 'NO3-N input from land, read in from scenario file, g N/m2 day
    Private FlushProp(maxyrs, 365) As Single  'Hydrologic flushing parameter, read in from scenario file, 1/day 
    Private GSstart As Short                  'Start of the growing season (day of year) 
    Private GSend As Short                    'End of the growing season (day of year)
    Private GSeas As Boolean                  'TRUE if current day is in growing season, otherwise FALSE 
    Private MowTmt(maxyrs, 365) As Integer    'Mowing as a management treatment; 0=no, 1=yes.  From Scenario input file. 
    Private MowHeight(maxyrs, 365) As Single  'Mowing height (meters), by year and day, for mangement treatment.  From Scenario input file.

    'Management to control invasives:  management parameters and variables 
    Private BioCutStartYr As Short    'year in which biocut (mowing) begins
    Private BioCutEndYr As Short      'year in which biocut ends 
    Private BioCutDay(5) As Short     'up to 5 days of year to do biocuts 
    Private BioCutHeight As Single    'height of biocut treatments
    Private BioCut As Boolean         'If Biocut treatment occurs on the current day, this = TRUE.  Otherwise FALSE.
    Private BioCutRemoval As Short    '0 = biocut and left on site; 1 = biocut and removal off site of cut biomass 
    Private HbcideStartYr As Short    'year in which herbicide treatment begins
    Private HbcideEndYr As Short      'year in which herbicide treatment ends
    Private HbcideDay(5) As Short     'up to 5 days of year to do herbicide treatments
    Private HbcidePen As Single       'depth (m) from top of canopy (m) that herbicide penetrates
    Private Hbcide As Boolean         'If herbicide treatment occurs on the current day, this = TRUE.  Otherwise FALSE. 
    Private BurnTrStartYr As Short    'year in which burn management treatments start
    Private BurnTrEndYr As Short      'year in which burn management treatments end
    Private BurnTrDay(5) As Short     'up to 5 days of year to do burn management treatments 
    Private BurnTrLitSev As Single    'proportion of litter layer consumed during burn, as a metric of burn severity 
    Private BurnTrNVol As Single      'of C volatilized in burn, proportion of associated N that is volatilized
    Private BurnTrLiveEff As Single   'Efficiency of burning live biomass.     
    Private BurnTr As Boolean         'If burn treatment occurs on the current day, this = TRUE.  Otherwise FALSE.  
    Private BurnTrHeight As Single    'Height above which litter layer burns; calculated internally 

    'Model run parameters.  Those that are public are accessed by user on forms, or needed on graphics forms 
    Public nStochasticRuns As Short    'number of multiple identical, stochastic runs
    Public LargeRunSet As Boolean  'True if large set of model runs; false if single run or small set of identical, stochastic runs
    Public ninsubset As Short  'number in subset, for systematic subsets of large sets of model runs
    Private sppcomb As Short   'Combination of species used in a subset of a large set of model runs
    Public Printvars As Short   'index for format for text file printout of model results
    Public PrintSuppress As Boolean  'when true, suppresses printing of output text files except the master printout file
    Public Graphvars As Short   'index for type of graph to draw and set of variables to graph
    Public GraphBackColor As Short 'index for background color of graphs, for projecting vs printing
    Public outfile11, outfile15, outfile16, outfile17 As String    'path and filenames of text output files
    Public parafile As String   'path and filename of text input file for site parameters and initial conditions
    Public seedfile As String   'path and filename of text input file for info about how to seed in plants in cohorts 
    Public scenariofile As String 'path and filename of text input file for scenario input file
    Public batchfile As String  'path and filename of text input file that organizes batch set of runs in large subset
    Public outputfolder As String 'folder path for output files  
    Private ParaValue(600) As Single  'array for storing parameter values read in from para input file, and altered during batch runs
    Private LRParaChangeIndex(16) As Short  'index numbers of parameters to be varied in large sets of model runs 
    Private LRParaChangeName(16) As String  'names of parameters to be varied in large sets of model runs  
    Private LRParaChangeVals(250, 16) As Single  'values of parameters to be varied in large sets of model runs 
    Private nLRParaChangVars As Short  'number of parameters being changed in batch text file in large sets of model runs
    Private OutputComments(16) As String   'lines of comments to put at top of results output files
    Public SppInitNum(5) As Short  'user choice: # of plant indivs per initialization cohort (spp-specific)
    Public SppInitYr(5) As Short   'user choice: year to begin introducing plant indivs; 1st initialization cohort (spp-specific)
    Public SppInitCohorts(5) As Short 'user choice: # of times to introduce plant indivs in initialization cohorts (spp-specific)
    Public SppInitInterval(5) As Short 'user choice: interval, in years, between successive initialization cohorts of plant indivs (spp-specific)
    Public SppBckgndNum(5) As Single 'user choice: background number of genets to seed in continually throughout model run

    'Bookkeeping variables needed as intermediary results or for results output
    Private PIPCsum, PIPNsum As Single
    Private GCPAGlitCsum, GCPAGlitNsum, GCPBGlitCsum, GCPBGlitNsum, GCPINsum, GCPMUCKCsum, GCPMUCKNsum, GCPMSOMCsum, GCPMSOMNsum As Single
    Private LitNminGCPDay As Single 'Daily total of N min from litter pools in grid cells
    Private MSOMNminGCPDay, MUCKNminGCPDay As Single 'Daily totals of N min from MSOM pool & MUCK pool in grid cells
    Private NitrifDay, N2DntrDay, N2ODntrDay As Single  'Daily totals of nitrification and denitrification in grid cells, g N/m2 y
    Private CtoMSOMDay, CtoMUCKDay As Single    'C flux transferred to MSOM & MUCK, totalled per day over all space
    Private CO2CRespDay, CH4CRespDay As Single  'CO2-C and CH4-C fluxes from heterotrophic respiration, totalled per day over all space
    Private ErrCount As Integer         'Error-checking counter: serious error
    Private ErrLast As Short          'Error-checking: value tracks the type of last error recorded
    Private firstErr13 As Boolean     'Error checking flag:  true if first time Error 13 occurred 
    Private LFASpp(5, 25) As Integer  'Logical Flow Analysis by Species array (see notes)
    Private PropBGCNneedAnnTot(5) As Single 'For tracking stats of PropBGCNneed variable
    Private PropBGCNneedAnnNum(5) As Integer  'For tracking stats of PropBGCNneed variable
    Private RGRLgtModSum(5) As Single  'For tracking stats of RGRLgtMod index
    Private RGRLgtModNum(5) As Integer   'For tracking stats of RGRLgtMod index
    Private LTfloodtally As Integer      'For tally of days water level has been deeper than long-term threshold

    'N fluxes
    Private NtransYr As Single   'annual cumulative, net amount of N transfer btwn grid cells and RNP, g N/m2 y 
    Private NH4NhydexpYr, NO3NhydexpYr As Single  'annual hydrologic export of NH4-N and NO3-N from RNP pools, g N/m2 y
    Private NH4NUptakeYr, NO3NUptakeYr As Single  'annual summed plant uptake of NH4-N and NO3-N, g N/m2 y
    Private NH4NinfluxYr, NO3NinfluxYr As Single  'annual summed influxes of NH4-N and NO3-N, g N/m2 y
    Private NinfluxLandYr As Single 'annual summed influx of N only from land, g N/m2 y
    Private NdetexpYr As Single     'annual loss of N via export of detritus, g N/m2 y
    Private NAGltrfallYr As Single  'annual AG flux of N in litterfall, summed across all spp, g N/m2 y
    Private NBGltrfallYr As Single  'annual BG flux of N in litterfall, summed across all spp, g N/m2 y
    Private NAGltrfallDay As Single 'litter N flux summed over all plants (g N/m2 day)
    Private NBGLtrfallDay As Single 'litter N flux summed over all plants (g N/m2 day)
    Private LitNminYr As Single   'annual flux of N mineralization from litter, g N/m2 y
    Private MSOMNminYr As Single  'annual flux of N mineralization from MSOM, g N/m2 y
    Private MUCKNminYr As Single  'annual flux of N mineralization from MUCK, g N/m2 y
    Private NitrifYr As Single    'annual flux of nitrification, g N/m2 y
    Private N2DntrYr, N2ODntrYr As Single     'annual fluxes of N2 and N2O in denitrification, g N/m2 y

    'C fluxes
    Private CNPPYr As Single       'annual flux of C in NPP summed over all species
    Private CNPPDaySpp(5) As Single    'C flux, daily NPP summed by species
    Private CNPPYrSpp(5) As Single 'annual flux of C in NPP (above + below ground) by species
    Private CANPPYrSpp(5) As Single  'annual flux of C in above-ground NPP 
    Private CNEPYr As Single       'annual flux of C in NEP (+ is into ecosystem)
    Private CNEEYr As Single       'annual net ecosystem exchange of CO2-C (+ is into ecosystem) 
    Private CO2CRespYr, CH4CRespYr As Single   'annual fluxes of CO2-C and CH4-C in heterotrophic respiration
    Private CAGgrowthYr As Single  'annual flux of C in aboveground plant growth
    Private CBGallocYr As Single   'annual flux of C in allocation to belowground plant parts
    Private CAGltrfallYr As Single 'annual AG flux of C in litterfall
    Private CBGltrfallYr As Single 'annual BG flux of C in litterfall
    Private CAGltrfallDay As Single 'litter C flux summed over all plants (g C/m2 day)
    Private CBGLtrfallDay As Single 'litter C flux summed over all plants (g C/m2 day)
    Private CdetExpYr As Single    'annual flux of C lost via detrital export from grid cell pools
    Private CtoMSOMYr As Single    'annual flux of C transferred to MSOM from AG and BG litter
    Private CtoMUCKYr As Single    'annual flux of C transferred to MUCK from AG and BG litter

    'Model Structure Options; switches that can be turned ON or OFF.  
    ' Originally scoped public so forms could access these
    Public LitSuppressVR As Boolean  'Model Structure Option: Litter suppression of vegetative regen
    Public LitSpatDistr As Short 'Model Structure Option:  Choice for spatial distribution of litter
    Public TeffOM As Boolean   'Model Structure Option: temperature effect on decomp rate of OM pools
    Public TeffLitOM As Boolean 'Model structure Option: temperature effect of litter on T of OM pools
    Public LgtComp As Boolean   'Model Structure Option: light competition on = TRUE, off = FALSE
    Public FloodMortality As Boolean  'Mortality due to flooding:  on = TRUE, off = FALSE
    Public ClonalBranching As Boolean 'During reproduction can clones branch:  on = TRUE, off = FALSE
    Public PTNSN As Boolean   'Pass-through non-stem nodes:  on = TRUE, off = FALSE 
    Private BrVisFreq As Short  'Frequency (years) at which branching visualization info is printed out 
    Private MGSPrintDay As Short 'Mid Growing Season Print Day:  Day of year to print certain info 

    Public Sub MondrianMain()
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        ' This is the Main Program for all of the ecology & biogeochemistry code:
        ' Control passes here once user has set some initial options and model run begins.
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        ' Initializes model execution, whether a single run or large set of runs
        SetRunsInit()

        ' iinsubset loops allows a subset of model runs within a large set of model runs. 
        ' For a single model run, ninsubset = 1 and this loop goes through only once.  
        For iinsubset As Short = 1 To ninsubset

            ' Within iinsubset loop, parameters can be changed as outlined in batch text file  
            If LargeRunSet Then Call ParaRevise(iinsubset)

            ParaAssign()

            ' Whether large sets of model runs, or single model runs, small sets of multiple runs
            ' can be done for stochastic variation; these stochastic run reps are the irun loop   
            For irun As Short = 1 To nStochasticRuns

                'Initialize variables needed at the start of a single, chronological, model run
                RunInit(irun, iinsubset)

                For iyr As Short = 1 To Runyears

                    YearInit(iyr)

                    DistrInit(iyr)

                    For iday As Short = 1 To 365

                        DayInit(iyr, iday)

                        NutrientInputs(iyr, iday)

                        NutrientTrans()

                        Litterfall(iyr, iday)

                        DetritalDyn(iyr, iday)

                        'Plant growth and N uptake take place over growing period set by user 
                        If GSeas Then
                            'Calc N demand and belowground N uptake
                            BGNUptake()
                            'Establish new aboveground stems.  Only call this every 5th day, to speed model operation. 
                            If (iday Mod 5) = 0 Then
                                AGStemEstablish(iday)
                            End If
                            'Growth of aboveground plant tissue
                            AGPlantGrowth(iday)
                            'Allow rhizome nodes to try to establish new daughter rhizome nodes.  
                            ' It is key that this only happens on selected days; this sets the limit for the number
                            ' of new vegetative repro indiv that can be created each year for each genet;
                            ' it also embodies a built-in delay for veg repro of each new individual in a chain 
                            If ((iday - GSstart) Mod 8) = 0 Then
                                BGRhizRepro()
                            End If

                            If FloodMortality Then FloodingMortality(iyr, iday)

                        End If

                        'Avoid possibility of litterfall and plant growth on same day
                        If iday = GSend + 1 Then Senescence()

                        If FloodMortality Then FloodingMortality2(iyr, iday)

                        NutrientExport(iyr, iday)

                        DayEndCalcs()

                        If iday = 365 Then

                            AgeIncrement()

                            YearEndCalcs()

                        End If

                        Sums(iday)

                        Printout(iinsubset, irun, iyr, iday)

                    Next iday

                Next iyr

            Next irun    'small number of identical, stochastic model runs

        Next iinsubset   'subset within large sets of model runs

        FileClose(11)
        'FileClose(12)
        FileClose(13)
        ' FileClose(14)
        FileClose(15)
        FileClose(16)
        FileClose(17)

        'close the progress bar and real-time graphics form
        If LargeRunSet Then
            myLRSProgress.Close()
        Else
            myProgress.Close()
        End If


        '  The form below tells the user the run has completed successfully and to close and exit the program.  
        myEndMsg.Show()

    End Sub

    Private Sub DistrInit(ByVal iyr As Short)
        '***********************************************************************************
        '  Called near the start of each year. 
        ' Figures out when new plants (as new genets) need to be created and introduced, 
        ' not as reproduction of existing individuals, but as initial distributions, or as genets
        ' seeding in, according to options set by user.  
        '  Calls subs to create the plants in spatial distributions. 
        '  Having this code here in YearInit, rather than RunInit2, allows it to initialize plants
        ' in multiple cohorts over time, plus continuous background seeding.
        '  Allowing the cohorts initial introduction to vary by species allows a native community
        ' to become established before an invader species is introduced.  
        '***********************************************************************************
        Dim ispp, icohort, igen, numindivs As Short

        '  Initial distribution of plants is done in multiple cohorts at intervals set by the user. 
        ' Species-specific.
        For ispp = 1 To 5
            If SppInitNum(ispp) > 0 Then
                For icohort = 1 To SppInitCohorts(ispp)
                    'the test below will call sub DistrRandom in the appropriate years: 
                    ' in the start year for the species and then at the correct intervals and the 
                    ' correct number of times, for each species 
                    If (SppInitYr(ispp) + (icohort - 1) * SppInitInterval(ispp)) = iyr Then
                        'Calculate the first new genet number to use
                        igen = igenmax(ispp) + 1
                        'Distribute all new individuals randomly over the space 
                        DistrRandom(ispp, SppInitNum(ispp), igen)
                    End If
                Next
            End If   'end of initial distribution of individuals in cohorts

            '  Background "seeding in" of new individuals (as genets) is done here.
            ' This begins *after* initial distributions, specified above, are completed, 
            ' and continues throughout the run.  It is done this way so that a new species 
            ' can be introduced later in the run, e.g a potential invader in years 15 and 20, 
            ' and the 'background' seeding of the invader species won't occur prior to last cohort 
            ' of the attempted invasion.  
            '  Once it begins, background seeding in occurs every year. 
            '  A user parameter sets the number of individuals to introduce each year as background. 
            '  If the user-set background seeding-in number is 1 or greater, it is assumed to be 
            ' an integer.  
            '  If the user-set background seeding-in number is < 1, it is assumed to be a decimal 
            ' between 0 and 1, representing a probability of having 1 propagule seed in each year. 
            If SppBckgndNum(ispp) > 0 Then
                If iyr > SppInitYr(ispp) + SppInitInterval(ispp) * (SppInitCohorts(ispp) - 1) Then
                    'Calculate the first new genet number to use
                    igen = igenmax(ispp) + 1
                    If SppBckgndNum(ispp) < 1 Then
                        'The number of new individuals to seed in is a decimal between 0 and 1, 
                        ' treated as a probability.  Use a uniform random number generator. 
                        Randomize()
                        If SppBckgndNum(ispp) > Rnd() Then
                            DistrRandom(ispp, 1, igen)
                        End If
                    Else
                        'The number of new inidividuals to seed in is 1 or greater. The number is rounded 
                        ' to an integer.
                        'The species, number of new individuals, and first new genet number to use 
                        'are passed to DistrRandom to execute the seeding-in.  
                        numindivs = CInt(SppBckgndNum(ispp))
                        DistrRandom(ispp, numindivs, igen)
                    End If
                End If
            End If   'end of background seeding in of individuals
        Next   'next species

    End Sub

    Private Sub DistrRandom(ByVal isp As Short, ByVal inumb As Short, ByVal igen As Short)
        '*************************************************************************************************
        '  This Sub randomly distributes and initializes new plant individuals, not from reproduction
        ' within the space, but from arrivals from outside the space as propagules.
        ' 
        '  This is called by DistrInit in any year that new individuals arrive, either in cohorts
        ' at the start of the run to populate the space with plants, or as invader propagules, 
        ' or as background propagule arrivals in subsequent years through the run
        '*************************************************************************************************
        Dim inew As Short
        Dim iindnew As Integer
        Dim RndNorm As Single          'Random number normally distributed, mean zero, sd +-1
        Dim RamSizeProp As Single      'Proportion of max size, used to initialize individual sizes
        Dim newC1 As Single, newC2 As Single, newN1 As Single, newN2 As Single
        Dim openspot As Boolean        'True if there is room in storage arrays to create a new individual

        Randomize()

        'Check to see if creating the new genets would go beyond the allowed maxgenets.  If so, 
        'give a message to the user, log an error, and do not create the new genets.
        If igen + inumb > maxgenets Then
            MsgBox("Mondrian tried to create more than " & maxgenets & " genets in species " & isp & _
                   ". The genets were not created and an error was logged.")
            ErrCount += 1
            ErrLast = 18
        Else
            For inew = 1 To inumb
                'Call Sub to get the index identifier number for new individual in this species 
                iindnew = 0 : openspot = False
                Call Findiind(isp, openspot, iindnew)

                If openspot = True Then
                    '******************* Location of individuals ************************************
                    'Locate each individual randomly within the space, uniformly distributed
                    'Rnd function returns random number uniformly distributed between 0 and 1
                    pil(isp, iindnew, 1) = s * GridixMax * Rnd()
                    pil(isp, iindnew, 2) = s * GridiyMax * Rnd()

                    '******************* C and N of individuals *************************************
                    '  Initialize each individual plant stem with C and N using spp-specific parameters
                    '  Randomize the initial sizes so that all starting individuals won't be exactly equal 

                    '  Sub NormDist returns a normally-distributed random number, mean zero, sd +-1
                    RndNorm = NormDist()

                    'RamSizeProp is the proportion of max size that will be used to initialize this 
                    ' individual.
                    'Initialize plant stem sizes in a normal distribution with mean equal to user-input
                    ' initial proportion of max size, and stand dev equal to 20% of that mean
                    RamSizeProp = (1 + 0.25 * RndNorm) * initRamC(isp)       'g C
                    'But disallow initial stem size to be greater than max stem size for the species, or less than zero
                    If RamSizeProp > 1 Then RamSizeProp = 1
                    If RamSizeProp < 0 Then RamSizeProp = 0
                    newC1 = RamSizeProp * maxPlantAGC(isp)
                    PIP(isp, iindnew, 1) = newC1                         'g C
                    newC2 = RamSizeProp * maxPlantBGC(isp)
                    PIP(isp, iindnew, 3) = newC2                         'g C
                    PIP(isp, iindnew, 5) = 0                             'g C
                    EcosCinout += newC1 + newC2

                    'For initial aboveground N in plant tissue, subtract proportion resorbed at end of growing season 
                    newN1 = (1 - resorbProp(isp)) * RamSizeProp * maxPlantAGC(isp) / initAGCNrat(isp)     'g N
                    PIP(isp, iindnew, 2) = newN1                                                             'g N
                    newN2 = RamSizeProp * maxPlantBGC(isp) / initBGCNrat(isp)                             'g N
                    PIP(isp, iindnew, 4) = newN2                                                             'g N
                    EcosNinout += newN1 + newN2

                    '******************* Genet and ramet information for individuals ****************
                    'At start of model run, each ramet is ramet #1 in its own genet
                    ' pgr array stores genet and ramet info: see array design notes
                    ' Because this sub is called in multiple years, one of the parms that it is passed is 
                    ' igen, the first new genet number to use
                    pgr(isp, iindnew, 1) = igen + inew - 1   'number of the genet
                    ' igenmax(isp) records the highest genet number index used so far, by species. 
                    igenmax(isp) = pgr(isp, iindnew, 1)
                    pgr(isp, iindnew, 2) = 0     '0 = this indiv did not wrap around the torus 
                    pgr(isp, iindnew, 3) = 0     '0 = this ramet has not reproduced vegetatively yet
                    'When initializing, give each ramet a random age between zero and its maximum age
                    ' This stops initial or any cohort from being all synchronized in age
                    ' Use a uniform distrubition returned by Rnd()
                    pgr(isp, iindnew, 4) = CInt(Int(Rnd() * maxLongev(isp)))   'age in years; 0 = this ramet is in its first year
                    pgr(isp, iindnew, 5) = 0     '0 = rhizome has not gone a growing season without making a stem, yet
                    ' This rhizome node has no parents; set all parent identifiers to zero
                    pgr(isp, iindnew, 6) = 0 : pgr(isp, iindnew, 7) = 0 : pgr(isp, iindnew, 8) = 0 : pgr(isp, iindnew, 9) = 0 : pgr(isp, iindnew, 10) = 0

                    'At start of run, give random angles for compass direction stored for individual angle
                    ' Random angle is in radians, 0 to 2pi
                    Randomize()
                    pga(isp, iindnew) = 6.28 * Rnd()

                End If  'end test of if openspot = true

            Next  'inew
        End If  'end test of whether adding new genets would go beyond maxgenets allowed 

    End Sub

    Private Function NormDist() As Single
        '*******************************************************************************************
        '  Convert uniform-dist Rnd to normal-dist GasDev
        '  GasDev is normally distributed with mean zero and sd +- 1.
        '*******************************************************************************************
        Dim r, v1, v2, GasDev As Single    'parameters needed to create random normal distribution

        Randomize()
        r = 1
        Do Until r < 1
            v1 = 2 * Rnd() - 1
            v2 = 2 * Rnd() - 1
            r = v1 ^ 2 + v2 ^ 2
        Loop
        GasDev = v1 * Sqrt(-2 * Log(r) / r)
        Return GasDev

    End Function

    Private Sub DayEndCalcs()
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
        '  Bookkeeping calculations performed at the end of the daily loop
        '  Called once each day from Main
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        CAGltrfallYr = CAGltrfallYr + CAGltrfallDay             'g C/m2 y
        CBGltrfallYr = CBGltrfallYr + CBGLtrfallDay             'g C/m2 y
        NAGltrfallYr = NAGltrfallYr + NAGltrfallDay             'g N/m2 y
        NBGltrfallYr = NBGltrfallYr + NBGLtrfallDay             'g N/m2 y

    End Sub

    Private Sub YearEndCalcs()
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  Perform year-end bookkeeping calculations
        '  Called once each year, after all ecology & biogeochemistry code, after
        ' DayEndCalcs the last time, outside of the iday loop.  Prior to Printout.
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim ispp, ix, iy As Short
        Dim iind As Integer
        Dim tempCsum, tempNsum As Single

        '****** Calculate whole ecosystem C and N, for mass balance checks. ***********
        EcosystemC = 0 : EcosystemN = 0
        'Add organic C pools in each plant individual
        tempCsum = 0 : tempNsum = 0
        For ispp = 1 To 5
            ' Note this code block will only execute for spp included in this run, where iindmax(ispp) > 0
            For iind = 1 To iindmax(ispp)
                tempCsum = tempCsum + PIP(ispp, iind, 1) + PIP(ispp, iind, 3) + PIP(ispp, iind, 5) + PIP(ispp, iind, 6)    'g C
                tempNsum = tempNsum + PIP(ispp, iind, 2) + PIP(ispp, iind, 4) + PIP(ispp, iind, 7)                         'g N  
            Next
        Next
        EcosystemC = EcosystemC + tempCsum                       'g C
        EcosystemN = EcosystemN + tempNsum                       'g N

        'Add C and N pools in each grid cell
        For ix = 1 To GridixMax
            For iy = 1 To GridiyMax
                EcosystemC = EcosystemC + GCP(ix, iy, 1) + GCP(ix, iy, 5) + GCP(ix, iy, 7) + GCP(ix, iy, 9)     'g C
                EcosystemN = EcosystemN + GCP(ix, iy, 2) + GCP(ix, iy, 3) + GCP(ix, iy, 4) + GCP(ix, iy, 6) + GCP(ix, iy, 8) + GCP(ix, iy, 10)       'g N
            Next
        Next

        'Add N in RNP pools
        EcosystemN = EcosystemN + RNP(1) + RNP(2)                        'g N

        '********************** Error checking ****************************************
        ' Calculate Cmassbal and Nmassbal, the mass-balance test variables to print and plot.
        'If C mass balance is being preserved, Cmassbal should be a flat line, unchanging, on a graph 
        'over time. EcosCinout is a cumulative record of all system inputs and losses of C through 
        'the model run.
        Cmassbal = EcosystemC - EcosCinout                       'g C
        Nmassbal = EcosystemN - EcosNinout                       'g N

        ' Check to see that iindmax(ispp) arrray is working correctly.  Test to see if there
        'are any nonzero values of aboveground C, or belowground C, or plant individual locations, 
        'at any index greater than iindmax(ispp) in arrays that hold all information by individuals.
        For ispp = 1 To 5
            'Note this code block will execute for all species, whether or not they are included in this run 
            For iind = (iindmax(ispp) + 1) To maxindivs
                If PIP(ispp, iind, 1) > 0 Or PIP(ispp, iind, 3) > 0 Or pil(ispp, iind, 1) > 0 Or pil(ispp, iind, 2) > 0 Then
                    ErrCount += 1
                    ErrLast = 17
                End If
            Next
        Next

        '********************** Other year-end calculations ***************************
        'NEP is NPP minus Heterotrophic respiration.
        ' If C is fixed in photosynthesis and then exported as detritus, this does not diminish
        ' NEP; the detritus that is exported is still included in NEP.  
        CNEPYr = CNPPYr - CO2CRespYr - CH4CRespYr                             'g C/m2 y
        'NEE is net ecosystem exchange of CO2-C 
        ' This is NEP corrected for loss of C in detrital export
        ' No need to correct for CH4-C emission because that was already omitted from NEP calculation 
        CNEEYr = CNEPYr - CdetExpYr

    End Sub

    Private Sub AgeIncrement()
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  In this sub, age of each individual is incremented 1 year
        ' 
        '  Called near the end of each year by Main. 
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim ispp As Short
        Dim iind As Integer

        'Increment the age of all living ramets by 1 year
        For ispp = 1 To 5
            ' Note this code block will only execute for spp included in this run, where iindmax(ispp) > 0
            For iind = 1 To iindmax(ispp)
                '**************** Increment age in years *****************************
                'use living BG biomass to test whether there is a living individual
                If PIP(ispp, iind, 3) > 0 Then
                    'increment age of ramet (plant individual) by 1 year
                    pgr(ispp, iind, 4) = pgr(ispp, iind, 4) + 1
                End If
            Next
        Next

    End Sub

    Private Sub AGMortality(ByVal jspp As Short, ByVal jind As Integer)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  This sub is called when the aboveground plant part (stem) dies in certain situations  
        '
        '  Called by FloodingMortality
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        'Move aboveground C and aboveground N from living pools to standing dead pools.
        'Do not allow N to be resorbed into rhizome first.
        'The fact that there is now zero C in aboveground stem C pool is the indicator, now, that 
        '  there is no living stem there.  The standing dead litter will later be subject to litterfall. 
        PIP(jspp, jind, 6) += PIP(jspp, jind, 1)                    'g C
        PIP(jspp, jind, 1) = 0                                      'g C
        PIP(jspp, jind, 7) += PIP(jspp, jind, 2)                    'g N
        PIP(jspp, jind, 2) = 0                                      'g N
    End Sub

    Private Sub BGMortality(ByVal jspp As Short, ByVal jind As Integer)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '   For clonal plants with long lived (> annual) belowground plant parts, 
        '  this sub is called when the belowground plant part dies.  
        '   All belowground C and N are passed to below-ground litter pools. 
        '   All information in all of the arrays that store plant information about the
        '  now-dead individual are zeroed out. 
        '
        '  Called by Litterfall once per year before the start of the growing season
        '  Called by AGStemEstablish at end of growing season 
        '  Called by FloodingMortality if long-term flooding kills rhizomes 
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim CBGltrflux As Single     'Flux of C in belowground litter, when indivs die
        Dim NBGltrflux As Single     'Flux of N in belowground litter, when indivs die
        Dim ixGridLoc, iyGridLoc As Short        'Grid cell location of individual plant
        Dim iknum As Short            'Loop counter index

        'Figure out which grid cell the individual plant is in
        'First grid cell is numbered (1,1); see notes
        ixGridLoc = Int(pil(jspp, jind, 1) / s) + 1
        iyGridLoc = Int(pil(jspp, jind, 2) / s) + 1

        'Flux of C and N in belowground litter are equal to all plant indiv belowground C and N
        CBGltrflux = PIP(jspp, jind, 3) + PIP(jspp, jind, 5)     'g C
        NBGltrflux = PIP(jspp, jind, 4)                          'g N

        'Add C and N litter fluxes to grid cell below-ground litter pools
        'For belowground plant parts, all litter enters grid cell the individual is in 
        GCP(ixGridLoc, iyGridLoc, 7) = GCP(ixGridLoc, iyGridLoc, 7) + CBGltrflux       'g C
        GCP(ixGridLoc, iyGridLoc, 8) = GCP(ixGridLoc, iyGridLoc, 8) + NBGltrflux       'g C

        'set belowground biomass pools equal to zero
        PIP(jspp, jind, 3) = 0
        PIP(jspp, jind, 4) = 0
        PIP(jspp, jind, 5) = 0

        'error checking:  test to see if aboveground biomass is nonzero; if so, this is an error
        'because this sub should only be called if there is no aboveground plant biomass
        If PIP(jspp, jind, 1) > 0 Then
            ErrCount = ErrCount + 1
            ErrLast = 14
        End If
        If PIP(jspp, jind, 2) > 0 Then
            If PIP(jspp, jind, 2) < 0.000001 Then
                'Assume this is a round-off error from previous calculations
                PIP(jspp, jind, 2) = 0
            Else
                ErrCount = ErrCount + 1
                ErrLast = 14
            End If
        End If

        'set plant location information to zero
        pil(jspp, jind, 1) = 0 : pil(jspp, jind, 2) = 0
        'Zero out the clonal reproduction and pedigree information for that plant individual
        For iknum = 1 To 10
            pgr(jspp, jind, iknum) = 0
        Next

        '********************* Bookkeeping *****************************************************
        ' Increment daily litterfall fluxes, convert units to per m2
        CBGLtrfallDay = CBGLtrfallDay + CBGltrflux / GridArea                     'g C/m2 day
        NBGLtrfallDay = NBGLtrfallDay + NBGltrflux / GridArea                     'g N/m2 day

    End Sub

    Private Sub SetRunsInit()
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  This sub is called once at the start of Mondrian model execution, whether a single
        ' model run or a large set of runs.
        '  This is the place to do misc. tasks that need to be done at the start of the 
        ' execution of Mondrian Main, and/or that use variables at the scope of this module.  
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        firstErr13 = False

        FileOpen(11, outfile11, OpenMode.Output)

        If LargeRunSet Then
            PrintSuppress = True  'Suppress printing of extra, detailed output files for large sets of runs
        Else
            ' Open files for printing of detailed output 
            FileOpen(15, outfile15, OpenMode.Output)
            FileOpen(16, outfile16, OpenMode.Output)
            FileOpen(17, outfile17, OpenMode.Output)
        End If

    End Sub
    Private Sub RunInit(ByVal irun, ByVal iinsubset)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  This sub is called from within MondrianMain at the start of each individual run
        ' in the set of model runs. 
        '  Various start-of-run tasks are executed.
        '  This includes setting run-length bookkeeping variables and error counters to zero.
        '  NOTE:  We do not initialize array elements to zero because that is their default value
        ' upon array declaration.  However, now with mutliple runs it may be needed to
        ' re-declare them or re-initialize to zero using array methods. 
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        Dim ix, iy As Short

        If LargeRunSet = False Then
            'For a single model run, initialize and show the dynamic graphics and progress bar 
            myProgress.ProgressBar1.Minimum = 0
            myProgress.ProgressBar1.Maximum = Runyears
            myProgress.Show()
        Else
            'For a batch set of model runs, initilize the progress bar on the first run of the set 
            If irun = 1 And iinsubset = 1 Then
                myLRSProgress.ProgressBar1.Minimum = 0
                myLRSProgress.ProgressBar1.Maximum = ninsubset * nStochasticRuns
                myLRSProgress.ProgressBar1.Step = 1
                myLRSProgress.Show()
            End If
            'In a batch set of model runs, update the progress bar showing what proportion of model 
            ' runs have been completed 
            'myLRSProgress.ProgressBar1.Value = iinsubset
            myLRSProgress.ProgressBar1.PerformStep()
            myLRSProgress.lblUpdate2.Text = "Mondrian is currently on run number " & iinsubset & " out of " & ninsubset
            myLRSProgress.Refresh()

        End If

        '************ Re-initialize storage arrays to zero ********************************************
        'To ensure that in each run of the set, all arrays are re-set to zero, they are Redimensioned here
        'Arrays that contain information that is already zeroed out at start of each day or year are not included
        'These arrays store info about plant individuals, grid system, and pools
        ReDim pil(5, maxindivs, 2)
        ReDim pgr(5, maxindivs, 10)
        ReDim pga(5, maxindivs)
        ReDim PIP(5, maxindivs, 7)
        ReDim Phgt(5, maxindivs)
        ReDim Pdsubmerged(5, maxindivs)
        ReDim GCP(25, 25, 10)
        ReDim iindmax(5)
        ReDim igenmax(5)
        ReDim AGLvertdim(2)
        ReDim BGLvertdim(2)
        ReDim MUCKvertdim(2)
        ReDim MSOMvertdim(2)

        '************ Re-initialize single run length bookeeping variables **************************** 
        '  These variables are for mass-balance checks: 
        ' set to zero once at start of run; accumulate throughout run
        EcosCinout = 0 : EcosNinout = 0
        '  Initialize error counters
        ErrCount = 0 : ErrLast = 0

        '************ Other initializations at start of each run **************************************
        'Initialize tally of days for long-term flooding to zero
        LTfloodtally = 0
        'Initialize hardwired parameter value 
        MSOMdepth = -0.3
        'Ensure that active zone depth is not lower than MSOM depth 
        If AZdepth < MSOMdepth Then AZdepth = MSOMdepth

        'Initialize grid cell pools at start of each run
        'Convert initial grid cell pools here from g C/m2 and g N/m2 to g C and g N using cell size
        For ix = 1 To GridixMax
            For iy = 1 To GridiyMax
                GCP(ix, iy, 1) = GCPinit(1) * s ^ 2    'g C
                GCP(ix, iy, 2) = GCPinit(2) * s ^ 2    'g N
                GCP(ix, iy, 3) = GCPinit(3) * s ^ 2    'g N
                GCP(ix, iy, 4) = GCPinit(4) * s ^ 2    'g N 
                GCP(ix, iy, 5) = GCPinit(5) * s ^ 2    'g C
                GCP(ix, iy, 6) = GCPinit(6) * s ^ 2    'g N
                GCP(ix, iy, 7) = GCPinit(7) * s ^ 2    'g C
                GCP(ix, iy, 8) = GCPinit(8) * s ^ 2    'g N
                GCP(ix, iy, 9) = GCPinit(9) * s ^ 2    'g C
                GCP(ix, iy, 10) = GCPinit(10) * s ^ 2    'g N
            Next
        Next

        'Initialize current-year weighted litter k value to that of species 1 at start of run
        For ix = 1 To GridixMax
            For iy = 1 To GridiyMax
                kL(ix, iy) = kLit(1)
            Next
        Next

    End Sub

    Public Sub ReadInputs()
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  Called at the start of a single model run or set of model runs.  
        ' Opens and reads text input files.  
        ' Also writes the parameters from Para text file to an output file
        ' for examination and comparison to see if all parameters were read properly.
        '
        '  Calls sub ParaAssign to translate read-in parameters into model parameters
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim ie, iloc, iyr, iday As Short
        Dim ParaIndex(600) As Short
        Dim ParaName(600) As String
        Dim paracheck As String = ""
        Dim batchcheck As String = ""
        Dim PrintInputFlag As Boolean
        Dim ispp As Short
        Dim readinclude(5) As Short
        Dim sppnumber As Short
        Dim iinsval As Short

        '**************** Open files needed for input and print-back of parameters **************
        'File 20 is the parameter input file
        FileOpen(20, parafile, OpenMode.Input)

        'File 12 is the parameter-check output file
        PrintInputFlag = True 'True = print input var. to output file to check; False = no print
        If PrintInputFlag Then paracheck = outputfolder & "\MondrianParaOut.txt"
        FileOpen(21, paracheck, OpenMode.Output)

        'File 22 is the seedfile input file
        FileOpen(22, seedfile, OpenMode.Input)

        '**************** Read parameters from Para text file **********************************
        'Read the 2-line comment header and put it directly into header for output files
        Input(20, OutputComments(6))
        Input(20, OutputComments(7))

        ' Read parameter index number, name, and value on each line. 
        ' Values read in are stored in three arrays. 
        ' An array ParaIndex() stores the index number of each parameter,
        ' a string array ParaName() stores each variable name while
        ' the array ParaValue() stores the actual value for the corresponding variable.

        For ie = 1 To 600   'ie is for "input element" 
            Input(20, ParaIndex(ie))
            Input(20, ParaName(ie))
            Input(20, ParaValue(ie))
            If PrintInputFlag Then 'prints list of variables to output file to check read-in
                PrintLine(21, ie, ParaIndex(ie), " ", ParaName(ie), " ", ParaValue(ie))
            End If
        Next

        '******************* Read seeding-in information from Seed text file ******************
        '   Reads information about how to seed in species in cohorts, and stores 
        ' that information in arrays.  
        '   Read the 2-line comment header and put it directly into header for output files
        Input(22, OutputComments(3))
        Input(22, OutputComments(4))

        Input(22, sppcomb)  'species combination:  1st line. Used only for printing on output file  
        For ispp = 1 To 5
            Input(22, sppnumber)  'no need to save this; listed in input file for convenience 
            Input(22, SppName(ispp))
            Input(22, readinclude(ispp))
            'Convert info whether species is included, to boolean
            If readinclude(ispp) = 0 Then
                SppInclude(ispp) = False
            Else
                SppInclude(ispp) = True
            End If
            'Read number of individuals to seed per cohort, but set to zero if this species is not included
            Input(22, SppInitNum(ispp)) : If (SppInclude(ispp) = False) Then SppInitNum(ispp) = 0
            Input(22, SppInitYr(ispp))
            Input(22, SppInitCohorts(ispp))
            Input(22, SppInitInterval(ispp))
            Input(22, SppBckgndNum(ispp))
        Next

        '****************** Read in scenario of environmental drivers from text file **********
        'File 25 is the scenario input file
        FileOpen(25, scenariofile, OpenMode.Input)

        Input(25, OutputComments(12))
        Input(25, OutputComments(13))

        For iyr = 1 To maxyrs
            For iday = 1 To 365
                Input(25, iyr)
                Input(25, iday)
                Input(25, Waterlvl(iyr, iday))        'meters
                Input(25, NH4NinputLand(iyr, iday))   'g N / m2 day
                Input(25, NO3NinputLand(iyr, iday))   'g N / m2 day
                Input(25, FlushProp(iyr, iday))       '1/day 
            Next
        Next


        '****************** Read in multiple-run plan in Batch text file **********************
        ' Only open and read this file if it is a large set of model runs 
        If LargeRunSet Then
            FileOpen(23, batchfile, OpenMode.Input)
            'Read the 2-line comment header and save it to print on output files
            Input(23, OutputComments(9))
            Input(23, OutputComments(10))

            'Use an output file to print batch info right back out for checking
            If PrintInputFlag Then batchcheck = outputfolder & "\MondrianBatchOut.txt"
            FileOpen(24, batchcheck, OpenMode.Output)

            ' Read 1st line of data:  ninsubset number, then parameter index values
            ' Note that ninsubset is not declared in this Sub because it is scoped Public 
            Input(23, ninsubset)
            For iloc = 1 To 15
                Input(23, LRParaChangeIndex(iloc))
                'Record max number of parameters being changed, to use as loop counter
                If LRParaChangeIndex(iloc) = 0 Then nLRParaChangVars = iloc - 1
            Next

            'Print first line of data back out
            If PrintInputFlag Then
                Print(24, iinsval)
                Print(24, " ")
                For iloc = 1 To 15
                    Print(24, LRParaChangeIndex(iloc))
                    Print(24, " ")
                Next
                PrintLine(24)
            End If

            'Read 2nd line of data:  dummy iinsubset number, then parameter names
            Input(23, iinsval)
            For iloc = 1 To 15
                Input(23, LRParaChangeName(iloc))
            Next

            'Print 2nd line of data back out
            If PrintInputFlag Then
                Print(24, iinsval)
                Print(24, " ")
                For iloc = 1 To 15
                    Print(24, LRParaChangeName(iloc))
                    Print(24, " ")
                Next
                PrintLine(24)
            End If

            'Read 3rd and subsequent lines of data: iinsubset number, then parameter values
            ' ie is for "input element".  These are rows in the batch file, each row 
            ' representing the parameter changes for a single model run in the batch subset 
            For ie = 1 To ninsubset
                Input(23, iinsval)
                For iloc = 1 To 15
                    '  In 2-d array LRParaChangeVals(i, j),
                    ' i = iinsubset (1 to 250) 
                    ' j = parameter that will be changed (1 to 15) 
                    Input(23, LRParaChangeVals(ie, iloc))
                Next 'iloc
            Next  'ie 

            'Print 3rd and subsequent lines of data back out for checking 
            If PrintInputFlag Then
                For ie = 1 To ninsubset
                    Print(24, ie)
                    Print(24, " ")
                    For iloc = 1 To 15
                        Print(24, LRParaChangeVals(ie, iloc))
                        Print(24, " ")
                    Next
                    PrintLine(24, " ")
                Next   'ie 
            End If

            FileClose(23)
            FileClose(24)
        Else  'LargeRunSet = false
            ninsubset = 1  'ninsubset = 1 means this is a single model run 
        End If

        FileClose(20)
        FileClose(21)
        FileClose(22)

    End Sub
    Private Sub ParaRevise(ByVal iinsubset As Short)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  Called by Mondrian Main at the start of each iinsubset loop, if this is a large
        ' set of model runs.  
        '  Revise parameter values from Para text file as indicated in Batch text file.
        '  Here, simply replace the values in the ParaValue array. 
        '  ParaAssign will be called next, to translate these into model variables. 
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim iloc, jloc As Short

        ' Go through all parameter index values in Para text file, and test whether they
        ' are in the set of parameters being changed in the Batch file  
        For iloc = 1 To 600
            For jloc = 1 To nLRParaChangVars
                ' If the current parameter index (iloc) from the Para text file and 
                ' in the ParaValue array is one of the ones being changed, then ... 
                If LRParaChangeIndex(jloc) = iloc Then
                    ' Replace the new value, read in from the Batch text file, 
                    ' for this value of iinsubset, in the ParaValue array 
                    ' at the current parameter index (iloc) 
                    ParaValue(iloc) = LRParaChangeVals(iinsubset, jloc)
                End If
            Next
        Next

    End Sub
    Private Sub ParaAssign()
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  This sub translates each value in the ParaValue() array to the actual parameter
        ' used in the model. Also calculates site specific variables dependent upon
        ' these input values.
        '
        '  For single model runs (or small sets of runs that vary only stochastically), this
        ' is called only once, by ReadInputs.  
        '  For large sets of model runs, the batch input file specifies changes in some 
        ' parameters, which are made directly in the ParaValue array during each iinsubset.  
        ' Then this sub is called again for each iinsubset, to re-translate the ParaValue array 
        ' and re-assign the parameter values as needed in the model.  
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        'Grid, cell size, and number of runs in set, max number of indivs and genets
        ncells = ParaValue(1)
        s = ParaValue(2)
        nStochasticRuns = ParaValue(3)
        maxindivs = ParaValue(4)
        maxgenets = ParaValue(5)

        'Now that the user has specified the side length of each cell and the number of cells, 
        'calculate the dimensions and area. 
        GridixMax = CInt(Sqrt(ncells))
        GridiyMax = GridixMax
        GridArea = (GridixMax * s) ^ 2        'units m2

        'Species-specific parameters
        PlantLitProp(1) = ParaValue(11)
        PlantLitProp(2) = ParaValue(12)
        PlantLitProp(3) = ParaValue(13)
        PlantLitProp(4) = ParaValue(14)
        PlantLitProp(5) = ParaValue(15)

        kLit(1) = ParaValue(21)
        kLit(2) = ParaValue(22)
        kLit(3) = ParaValue(23)
        kLit(4) = ParaValue(24)
        kLit(5) = ParaValue(25)

        relGrate(1) = ParaValue(31)       'relative growth rate of spp 1, as a factor per day
        relGrate(2) = ParaValue(32)       'relative growth rate of spp 2, as a factor per day
        relGrate(3) = ParaValue(33)       'relative growth rate of spp 3, as a factor per day
        relGrate(4) = ParaValue(34)       'relative growth rate of spp 4, as a factor per day
        relGrate(5) = ParaValue(35)       'relative growth rate of spp 5, as a factor per day

        PlantType(1) = CShort(ParaValue(41))
        PlantType(2) = CShort(ParaValue(42))
        PlantType(3) = CShort(ParaValue(43))
        PlantType(4) = CShort(ParaValue(44))
        PlantType(5) = CShort(ParaValue(45))

        initRamC(1) = ParaValue(51)
        initRamC(2) = ParaValue(52)
        initRamC(3) = ParaValue(53)
        initRamC(4) = ParaValue(54)
        initRamC(5) = ParaValue(55)

        initBGCNrat(1) = ParaValue(61)
        initBGCNrat(2) = ParaValue(62)
        initBGCNrat(3) = ParaValue(63)
        initBGCNrat(4) = ParaValue(64)
        initBGCNrat(5) = ParaValue(65)

        initAGCNrat(1) = ParaValue(71)
        initAGCNrat(2) = ParaValue(72)
        initAGCNrat(3) = ParaValue(73)
        initAGCNrat(4) = ParaValue(74)
        initAGCNrat(5) = ParaValue(75)

        maxPlantBGC(1) = ParaValue(81)
        maxPlantBGC(2) = ParaValue(82)
        maxPlantBGC(3) = ParaValue(83)
        maxPlantBGC(4) = ParaValue(84)
        maxPlantBGC(5) = ParaValue(85)

        maxPlantAGC(1) = ParaValue(91)
        maxPlantAGC(2) = ParaValue(92)
        maxPlantAGC(3) = ParaValue(93)
        maxPlantAGC(4) = ParaValue(94)
        maxPlantAGC(5) = ParaValue(95)

        resorbProp(1) = ParaValue(101)
        resorbProp(2) = ParaValue(102)
        resorbProp(3) = ParaValue(103)
        resorbProp(4) = ParaValue(104)
        resorbProp(5) = ParaValue(105)

        rhizRetCProp(1) = ParaValue(111)
        rhizRetCProp(2) = ParaValue(112)
        rhizRetCProp(3) = ParaValue(113)
        rhizRetCProp(4) = ParaValue(114)
        rhizRetCProp(5) = ParaValue(115)

        rhizAllocCProp(1) = ParaValue(121)
        rhizAllocCProp(2) = ParaValue(122)
        rhizAllocCProp(3) = ParaValue(123)
        rhizAllocCProp(4) = ParaValue(124)
        rhizAllocCProp(5) = ParaValue(125)

        maxLongev(1) = CShort(ParaValue(131))
        maxLongev(2) = CShort(ParaValue(132))
        maxLongev(3) = CShort(ParaValue(133))
        maxLongev(4) = CShort(ParaValue(134))
        maxLongev(5) = CShort(ParaValue(135))

        AGEstparmA(1) = ParaValue(141)
        AGEstparmA(2) = ParaValue(142)
        AGEstparmA(3) = ParaValue(143)
        AGEstparmA(4) = ParaValue(144)
        AGEstparmA(5) = ParaValue(145)

        CallocBGprop(1) = ParaValue(151)
        CallocBGprop(2) = ParaValue(152)
        CallocBGprop(3) = ParaValue(153)
        CallocBGprop(4) = ParaValue(154)
        CallocBGprop(5) = ParaValue(155)

        'For MaxVRangle, as they are read in, convert from degrees to radians
        MaxVRangle(1) = (6.28 / 360) * ParaValue(161)
        MaxVRangle(2) = (6.28 / 360) * ParaValue(162)
        MaxVRangle(3) = (6.28 / 360) * ParaValue(163)
        MaxVRangle(4) = (6.28 / 360) * ParaValue(164)
        MaxVRangle(5) = (6.28 / 360) * ParaValue(165)

        cidist(1) = ParaValue(171)
        cidist(2) = ParaValue(172)
        cidist(3) = ParaValue(173)
        cidist(4) = ParaValue(174)
        cidist(5) = ParaValue(175)

        PBGCNNmin(1) = ParaValue(181)
        PBGCNNmin(2) = ParaValue(182)
        PBGCNNmin(3) = ParaValue(183)
        PBGCNNmin(4) = ParaValue(184)
        PBGCNNmin(5) = ParaValue(185)

        SubsidyPar(1) = CShort(ParaValue(191))
        SubsidyPar(2) = CShort(ParaValue(192))
        SubsidyPar(3) = CShort(ParaValue(193))
        SubsidyPar(4) = CShort(ParaValue(194))
        SubsidyPar(5) = CShort(ParaValue(195))

        AlloBioHgtA(1) = ParaValue(201)
        AlloBioHgtA(2) = ParaValue(202)
        AlloBioHgtA(3) = ParaValue(203)
        AlloBioHgtA(4) = ParaValue(204)
        AlloBioHgtA(5) = ParaValue(205)

        AlloBioHgtB(1) = ParaValue(211)
        AlloBioHgtB(2) = ParaValue(212)
        AlloBioHgtB(3) = ParaValue(213)
        AlloBioHgtB(4) = ParaValue(214)
        AlloBioHgtB(5) = ParaValue(215)

        LgtExtCurveA(1) = ParaValue(221)
        LgtExtCurveA(2) = ParaValue(222)
        LgtExtCurveA(3) = ParaValue(223)
        LgtExtCurveA(4) = ParaValue(224)
        LgtExtCurveA(5) = ParaValue(225)

        LgtExtCurveB(1) = ParaValue(231)
        LgtExtCurveB(2) = ParaValue(232)
        LgtExtCurveB(3) = ParaValue(233)
        LgtExtCurveB(4) = ParaValue(234)
        LgtExtCurveB(5) = ParaValue(235)

        LgtExtCurveC(1) = ParaValue(241)
        LgtExtCurveC(2) = ParaValue(242)
        LgtExtCurveC(3) = ParaValue(243)
        LgtExtCurveC(4) = ParaValue(244)
        LgtExtCurveC(5) = ParaValue(245)

        LgtEHeight(1) = ParaValue(251)
        LgtEHeight(2) = ParaValue(252)
        LgtEHeight(3) = ParaValue(253)
        LgtEHeight(4) = ParaValue(254)
        LgtEHeight(5) = ParaValue(255)

        BioFullShade(1) = ParaValue(261)
        BioFullShade(2) = ParaValue(262)
        BioFullShade(3) = ParaValue(263)
        BioFullShade(4) = ParaValue(264)
        BioFullShade(5) = ParaValue(265)

        FloodMortDays(1) = ParaValue(271)
        FloodMortDays(2) = ParaValue(272)
        FloodMortDays(3) = ParaValue(273)
        FloodMortDays(4) = ParaValue(274)
        FloodMortDays(5) = ParaValue(275)

        TbranchProb(1) = ParaValue(281)
        TbranchProb(2) = ParaValue(282)
        TbranchProb(3) = ParaValue(283)
        TbranchProb(4) = ParaValue(284)
        TbranchProb(5) = ParaValue(285)

        LbranchProb(1) = ParaValue(291)
        LbranchProb(2) = ParaValue(292)
        LbranchProb(3) = ParaValue(293)
        LbranchProb(4) = ParaValue(294)
        LbranchProb(5) = ParaValue(295)

        'Plant parameters that are not species-specific
        CIDistCostParm = ParaValue(401)
        BGpersist = CShort(ParaValue(402))
        LTflooddepth = ParaValue(405)

        'Initial values of grid cell pools
        GCPinit(1) = ParaValue(421)
        GCPinit(2) = ParaValue(422)
        GCPinit(3) = ParaValue(423)
        GCPinit(4) = ParaValue(424)
        GCPinit(5) = ParaValue(425)
        GCPinit(6) = ParaValue(426)
        GCPinit(7) = ParaValue(427)
        GCPinit(8) = ParaValue(428)
        GCPinit(9) = ParaValue(429)
        GCPinit(10) = ParaValue(430)

        'Regional nutrient pool
        'Convert from g N/m2 to g N using whole grid area
        RNP(1) = ParaValue(431) * GridArea            'g N
        RNP(2) = ParaValue(432) * GridArea            'g N

        'Ecosystem and biogeochemistry parameters
        FlushPropMult = ParaValue(451)
        DetExpProp = ParaValue(452)
        kMSOM = ParaValue(453)
        GCPhumProp = ParaValue(454)
        LitCritCNRat = ParaValue(455)
        HumifCNrat = ParaValue(456)
        TeffLitOMparm = ParaValue(458)
        kMUCK = ParaValue(459)
        kBioturb = ParaValue(460)
        bdAGL = ParaValue(461)
        bdBGL = ParaValue(462)
        bdMUCK = ParaValue(463)
        bdMSOM = ParaValue(464)
        cconcMUCK = ParaValue(465)
        cconcMSOM = ParaValue(466)
        cconcTissue = ParaValue(467)
        BGLspread = ParaValue(468)
        LitAnmod = ParaValue(469)
        MuckAnmod = ParaValue(470)
        MSOMAnmod = ParaValue(471)
        NitrifParm = ParaValue(472)
        'Check that this parameter is not less than zero or greater than 1
        If NitrifParm < 0 Then NitrifParm = 0
        If NitrifParm > 1 Then NitrifParm = 1
        AZdepth = ParaValue(473)
        DntrParm = ParaValue(474)
        'Check that this parameter is not less than zero or greater than 1
        If DntrParm < 0 Then DntrParm = 0
        If DntrParm > 1 Then DntrParm = 1
        HetRespBench = ParaValue(475)
        CH4P0 = ParaValue(477)
        'Check that this parameter is not less than zero or greater than 1
        If CH4P0 < 0 Then CH4P0 = 0
        If CH4P0 > 1 Then CH4P0 = 1
        CH4oxIn = ParaValue(478)
        'Check that this parameter is not less than zero or greater than 1
        If CH4oxIn < 0 Then CH4oxIn = 0
        If CH4oxIn > 1 Then CH4oxIn = 1
        CH4oxAer = ParaValue(479)
        'Check that this parameter is not less than zero or greater than 1
        If CH4oxAer < 0 Then CH4oxAer = 0
        If CH4oxAer > 1 Then CH4oxAer = 1
        N2Op1d = ParaValue(480)
        'Check that this parameter is not less than zero or greater than 1
        If N2Op1d < 0 Then N2Op1d = 0
        If N2Op1d > 1 Then N2Op1d = 1
        N2Op4d = ParaValue(481)
        'Check that this parameter is not less than zero or greater than 1
        If N2Op4d < 0 Then N2Op4d = 0
        If N2Op4d > 1 Then N2Op4d = 1
        N2Oplt = ParaValue(482)
        'Check that this parameter is not less than zero or greater than 1
        If N2Oplt < 0 Then N2Oplt = 0
        If N2Oplt > 1 Then N2Oplt = 1
        N2Opdepth = ParaValue(483)
        'Check that this parameter is not less than AZdepth
        If N2Opdepth < AZdepth Then N2Opdepth = AZdepth

        'Scenario parameters:  Perhaps eventually move these to a separate Scenario input file and allow some to vary by year
        Baseyear = ParaValue(506)
        Runyears = ParaValue(507)
        TambAnnAvgStart = ParaValue(508)
        TambAnnAvgEnd = ParaValue(509)
        TambMaxDev = ParaValue(510)
        NH4NinputPpt = ParaValue(512)
        NO3NinputPpt = ParaValue(513)
        GSstart = ParaValue(514)
        GSend = ParaValue(515)
        NH4NinflowMult = ParaValue(516)
        NO3NinflowMult = ParaValue(517)

        'Management parameters
        BioCutStartYr = ParaValue(526)
        BioCutEndYr = ParaValue(527)
        BioCutDay(1) = ParaValue(528)
        BioCutDay(2) = ParaValue(529)
        BioCutDay(3) = ParaValue(530)
        BioCutDay(4) = ParaValue(531)
        BioCutDay(5) = ParaValue(532)
        BioCutHeight = ParaValue(533)
        BioCutRemoval = ParaValue(534)

        HbcideStartYr = ParaValue(536)
        HbcideEndYr = ParaValue(537)
        HbcideDay(1) = ParaValue(538)
        HbcideDay(2) = ParaValue(539)
        HbcideDay(3) = ParaValue(540)
        HbcideDay(4) = ParaValue(541)
        HbcideDay(5) = ParaValue(542)
        HbcidePen = ParaValue(543)

        BurnTrStartYr = ParaValue(546)
        BurnTrEndYr = ParaValue(547)
        BurnTrDay(1) = ParaValue(548)
        BurnTrDay(2) = ParaValue(549)
        BurnTrDay(3) = ParaValue(550)
        BurnTrDay(4) = ParaValue(551)
        BurnTrDay(5) = ParaValue(552)
        BurnTrLitSev = ParaValue(553)
        'Do not allow a value greater than 0.99 
        If BurnTrLitSev > 0.99 Then BurnTrLitSev = 0.99
        BurnTrNVol = ParaValue(554)
        BurnTrLiveEff = ParaValue(555)

        'Model structure options
        LitSuppressVR = False : If CShort(ParaValue(576)) = 1 Then LitSuppressVR = True
        LitSpatDistr = CShort(ParaValue(577))
        TeffOM = False : If CShort(ParaValue(579)) = 1 Then TeffOM = True
        TeffLitOM = False : If CShort(ParaValue(580)) = 1 Then TeffLitOM = True
        LgtComp = False : If CShort(ParaValue(583)) = 1 Then LgtComp = True
        FloodMortality = False : If CShort(ParaValue(584)) = 1 Then FloodMortality = True
        ClonalBranching = False : If CShort(ParaValue(585)) = 1 Then ClonalBranching = True
        PTNSN = False : If CShort(ParaValue(586)) = 1 Then PTNSN = True
        BrVisFreq = ParaValue(587)
        MGSPrintDay = ParaValue(588)

    End Sub
    Private Sub YearInit(ByVal iyr As Short)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  Called at the start of each year. 
        ' Initializes numerous annual bookkeeping variables to zero.
        ' Initializes logical flow analysis counters, error counters, etc. to zero. 
        ' Sets up calculations of weighted litter k values. 
        ' Re-sets other counters to zero as needed at start of year. 
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim ispp, ilfa, ix, iy As Short
        Dim iind As Integer

        'Update the progress bar for a single model run 
        'Refresh the progress bar for both cases:  large run set, or single model run 
        If LargeRunSet Then
            myLRSProgress.Refresh()
        Else
            myProgress.ProgressBar1.Value = iyr
            myProgress.lblYear.Text = "Year = " & iyr
            myProgress.Refresh()
        End If

        'At start of each year, zero out annual bookkeeping fluxes of N and C
        NtransYr = 0 : NH4NhydexpYr = 0 : NO3NhydexpYr = 0 : NH4NUptakeYr = 0 : NO3NUptakeYr = 0 : NH4NinfluxYr = 0 : NO3NinfluxYr = 0 : NinfluxLandYr = 0
        NdetexpYr = 0 : NAGltrfallYr = 0 : NBGltrfallYr = 0 : LitNminYr = 0 : MSOMNminYr = 0 : MUCKNminYr = 0 : NitrifYr = 0 : N2DntrYr = 0 : N2ODntrYr = 0

        CdetExpYr = 0 : CNPPYr = 0 : CNEPYr = 0 : CNEEYr = 0 : CO2CRespYr = 0 : CH4CRespYr = 0 : CAGgrowthYr = 0
        CBGallocYr = 0 : CAGltrfallYr = 0 : CBGltrfallYr = 0 : CtoMSOMYr = 0 : CtoMUCKYr = 0
        For ispp = 1 To 5
            CNPPYrSpp(ispp) = 0
            CANPPYrSpp(ispp) = 0
        Next

        ' Re-set all counters in the Logical Flow Analysis by Species array to zero at start of each year
        For ispp = 1 To 5
            For ilfa = 1 To 25
                LFASpp(ispp, ilfa) = 0
            Next
        Next

        'Re-set error checking counters to zero
        ErrCount = 0
        ErrLast = 0

        'Initialize additional annual bookkeeping variables
        For ispp = 1 To 5
            PropBGCNneedAnnTot(ispp) = 0
            PropBGCNneedAnnNum(ispp) = 0
        Next

        'Initialize information for calculating weighted litter k values by grid cell at start of year
        For ix = 1 To GridixMax
            For iy = 1 To GridiyMax
                gckcalcs(ix, iy, 1) = 0 : gckcalcs(ix, iy, 2) = 0
            Next
        Next

        'Re-set counters of days each aboveground stem is continuously submerged to zero at start of each year
        For ispp = 1 To 5
            For iind = 1 To iindmax(ispp)
                Pdsubmerged(ispp, iind) = 0
            Next
        Next
    End Sub
    Private Sub DayInit(ByVal iyr As Short, ByVal iday As Short)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  Initialize quantities that are accumulated daily and that are scoped higher than
        ' the subroutine level, for example if needed for daily printout of results. 
        '
        '  Calculate daily air and water temperature 
        '
        '  Figure out whether certain management treatments need to occur on this day or not,
        ' based on info from parameter file. 
        '
        '  Called by Main, at start of each day. 
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim ispp As Short              'index used to initialize array
        Dim TambAnnAvg As Single       'Ambient annual average temperature for this year

        'Bookkeeping variable accumulated daily for results printout
        For ispp = 1 To 5
            CNPPDaySpp(ispp) = 0
        Next

        'Other daily bookkeeping variables
        CAGltrfallDay = 0 : CBGLtrfallDay = 0
        NAGltrfallDay = 0 : NBGLtrfallDay = 0
        MSOMNminGCPDay = 0 : MUCKNminGCPDay = 0
        LitNminGCPDay = 0 : CtoMSOMDay = 0
        CtoMUCKDay = 0 : CO2CRespDay = 0 : CH4CRespDay = 0
        NitrifDay = 0 : N2DntrDay = 0 : N2ODntrDay = 0

        'To calculate daily air and water temperature, first calculate this year's avg ambient temperature 
        TambAnnAvg = TambAnnAvgStart + (TambAnnAvgEnd - TambAnnAvgStart) * iyr / Runyears
        'Calculate daily air temperature
        'Use seasonal, sinusoidal function, min at Jan 20, max at July 20 (iday 202) 
        TdayAir = TambAnnAvg + TambMaxDev * Cos((iday - 202) * 0.01721)
        'Caluclate daily water temperature based on air temperature;  
        ' relationship based On data from Great Lakes wetland sites, From LHM team at MSU
        If TdayAir > 0 Then
            TdayWater = 3.4 + 0.785 * TdayAir
        Else
            TdayWater = 3.4
        End If

        'Store the value water level on current day in a new variable.
        'This simply speeds the model by not needing to read this from the Waterlvl array repeatedly
        WaterlvlDay = Waterlvl(iyr, iday)   'meters 

        'Call WaterlvlCalc to calculate new trailing average in water level on current day
        WaterLvlCalc(iyr, iday)

        'Set boolean variable for whether this day is in the growing season 
        If (iday >= GSstart) And (iday <= GSend) Then
            GSeas = True
        Else
            GSeas = False
        End If

        'Management to control invasives: Figure out whether biocut (e.g. mowing) occurs on this day or not 
        BioCut = False
        If iyr >= BioCutStartYr And iyr <= BioCutEndYr Then
            For ibiocutday = 1 To 5
                If BioCutDay(ibiocutday) = iday Then
                    BioCut = True
                End If
            Next
        End If
        If BioCutStartYr = 0 Then BioCut = False 'double-check to make sure biocut is turned on 

        'Management to control invasives: Figure out whether Herbicide treatment occurs on this day or not 
        Hbcide = False
        If iyr >= HbcideStartYr And iyr <= HbcideEndYr Then
            For ihday = 1 To 5
                If HbcideDay(ihday) = iday Then
                    Hbcide = True
                End If
            Next
        End If
        If HbcideStartYr = 0 Then Hbcide = False 'double-check to make sure herbicide is turned on 

        'Management to control invasives: Figure out whether burn treatment occurs on this day or not 
        BurnTr = False
        If iyr >= BurnTrStartYr And iyr <= BurnTrEndYr Then
            For ibday = 1 To 5
                If BurnTrDay(ibday) = iday Then
                    BurnTr = True
                End If
            Next
        End If
        If BurnTrStartYr = 0 Then BurnTr = False 'double-check to make sure burning is turned on 

    End Sub
    Private Sub NutrientInputs(ByVal iyr As Short, ByVal iday As Short)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  This sub is called once per day, from Main.
        '  Nutrients input to the system as a whole, from outside, occur here. 
        '  All nutrient inputs come into the RNP pool (Regional Nutrient Pool).
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim NH4NInSumDay As Single     'NN4-N input per day, due to N inputs
        Dim NO3NInSumDay As Single     'NO3-N input per day, due to N inputs
        Dim NH4NinLandDay As Single    'NH4-N input per day, due to NH4-N inputs from land, streams, and groundwater 
        Dim NO3NinLandDay As Single    'NO3-N input per day, due to NO3-N inputs from land, streams, and groundwater 
        Dim NH4NinPptDay As Single     'NH4-N input per day, due just to NH4-N inputs from precipitation
        Dim NO3NinPptDay As Single     'NO3-N input per day, due just to NO3-N inputs from precipitation

        'For N input from land, including streams, rivers, and groundwater, value can vary daily, 
        ' set in Scenario input file. Here multiply each by individual multipliers which allows the user
        ' to easily do a set of runs where these are multiplied by a set of increasing factors.  
        'Convert units, (g N/m2 day) to (g N/day)
        NH4NinLandDay = NH4NinputLand(iyr, iday) * NH4NinflowMult * GridArea    'g NH4-N/day summed over entire modeled area 
        NO3NinLandDay = NO3NinputLand(iyr, iday) * NO3NinflowMult * GridArea    'g NO3-N/day summed over entire modeled area

        'At present N inputs from precip are annual input fluxes 
        ' divided equally over the year
        'Convert units, (g N/m2 y) to (g N/day)
        NH4NinPptDay = NH4NinputPpt * GridArea / 365      'g N/day
        NO3NinPptDay = NO3NinputPpt * GridArea / 365      'g N/day

        RNP(1) = RNP(1) + NH4NinLandDay + NH4NinPptDay          'g N
        RNP(2) = RNP(2) + NO3NinLandDay + NO3NinPptDay          'g N

        'Annual bookkeeping variables for summed N influxes; fluxes converted back to per m2 units
        ' Keep track of two total influxes; all influxes, and that from land only
        ' so that N retention can be calculated two ways, as a percentage of each
        NH4NInSumDay = NH4NinLandDay + NH4NinPptDay   'g N/day
        NO3NInSumDay = NO3NinLandDay + NO3NinPptDay   'g N/day
        NH4NinfluxYr += NH4NInSumDay / GridArea                                         'g N/m2 y
        NO3NinfluxYr += NO3NInSumDay / GridArea                                         'g N/m2 y
        NinfluxLandYr += (NH4NinLandDay + NO3NinLandDay) / GridArea               'g N/m2 y

        'EcosNinout for mass balance check
        EcosNinout = EcosNinout + NH4NInSumDay + NO3NInSumDay             'g N

    End Sub
    Private Sub NutrientTrans()
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  This sub is called once per day, from Main
        '  Here, nutrients are transferred between the Regional Nutrient Pool and 
        ' the Grid Cell Pools. 
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim ix, iy As Short
        Dim RNPstart(2) As Double                  'avail N in RNP pool at start of this routine, normalized g N/m2
        Dim GCPNH4Nstart As Double                 'avail NH4-N in each grid cell pool at start, normalized g N/m2 
        Dim GCPNO3Nstart As Double                 'avail NO3-N in each grid cell pool at start, normalized g N/m2
        Dim NH4Ndiff, NO3Ndiff As Double           'differennce in avail N btwn grid cell and RNP pool, in g N/m2
        Dim NH4Ntransfer, NO3Ntransfer As Double   'amount of N, in absolute g, to transfer btwn grid cell pool and RNP pool
        Dim NtransDay As Double                    'daily cumulative, net amt of N transfer btwn grid cells and RNP pool

        'Store how much avail N was in RNP pool at start of this routine
        RNPstart(1) = RNP(1) / GridArea
        RNPstart(2) = RNP(2) / GridArea

        '***Go through all the grid cells, and let N flow between RNP and each grid cell individually,
        'driven by differences in N concentration per unit area.
        '  
        NtransDay = 0
        For ix = 1 To GridixMax
            For iy = 1 To GridiyMax
                'Calc starting grid cell N pools, normalized per m2
                GCPNH4Nstart = GCP(ix, iy, 3) / (s ^ 2)      'g N/m2
                GCPNO3Nstart = GCP(ix, iy, 4) / (s ^ 2)      'g N/m2
                'Calc difference in available N concentration per area
                NH4Ndiff = GCPNH4Nstart - RNPstart(1)           'g N/m2
                NO3Ndiff = GCPNO3Nstart - RNPstart(2)
                '  Calc each nutrient transfer for this grid cell as the amount that will equilibrate the concentrations
                'per m2 btwn this grid cell and the appropriate RNP pool at start of this timestep.  
                '  The code here does both positive and negative N transfers; either direction, depending on concns
                'at the start of the timestep. 
                '  Positive value of N transfer is defined as being FROM grid cell TO the RNP pool.  
                '  Positive N transfer occurs when initial grid cell N conc was greater than initial RNP conc (normalized)
                '  Convert back to absolute g units for the N transfer. 
                NH4Ntransfer = (NH4Ndiff * s ^ 2) / 2                 'g N
                GCP(ix, iy, 3) = GCP(ix, iy, 3) - NH4Ntransfer        'g N
                RNP(1) = RNP(1) + NH4Ntransfer
                NO3Ntransfer = (NO3Ndiff * s ^ 2) / 2                 'g N
                GCP(ix, iy, 4) = GCP(ix, iy, 4) - NO3Ntransfer        'g N
                RNP(2) = RNP(2) + NO3Ntransfer

                'NtransDay is a cumulative bookkeeping variable, net daily flux of N transfer summed over both NH4 and NO3
                ' across the entire grid.  (Below, normalized to per unit area when summed annually.)  
                NtransDay = NtransDay + NH4Ntransfer + NO3Ntransfer                'g N/day

                'Zero out any small round-off errors that might have caused an N pool to go below zero
                'At same time, record an error if an N pool went below zero by more than a round-off error
                If RNP(1) < 0 Then
                    If RNP(1) < -0.00002 Then
                        ErrCount = ErrCount + 1
                        ErrLast = 1
                    End If
                    RNP(1) = 0
                End If
                If RNP(2) < 0 Then
                    If RNP(2) < -0.00002 Then
                        ErrCount = ErrCount + 1
                        ErrLast = 1
                    End If
                    RNP(2) = 0
                End If
                If GCP(ix, iy, 3) < 0 Then
                    If GCP(ix, iy, 3) < -0.00002 Then
                        ErrCount = ErrCount + 1
                        ErrLast = 3
                    End If
                    GCP(ix, iy, 3) = 0
                End If
                If GCP(ix, iy, 4) < 0 Then
                    If GCP(ix, iy, 4) < -0.00002 Then
                        ErrCount = ErrCount + 1
                        ErrLast = 3
                    End If
                    GCP(ix, iy, 4) = 0
                End If
            Next
        Next

        'Error check: see if overall N transfer matches change in RNP pools, within round-off
        If Abs((RNP(1) + RNP(2) - (RNPstart(1) + RNPstart(2)) * GridArea) - NtransDay) > 0.0005 Then
            ErrCount = ErrCount + 1
            ErrLast = 2
        End If

        'Update flux NtransYr, a net annual N transfer flux
        NtransYr = NtransYr + NtransDay / GridArea              'g N/m2 y

    End Sub
    Private Sub NutrientExport(ByVal iyr As Short, ByVal iday As Short)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  This sub is called once per day, from Main.
        '  Nutrients exported from the system as a whole, to outside, occur here. 
        '  All nutrient exports come from the RNP pool (Regional Nutrient Pool).
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim NH4NexportRNP, NO3NexportRNP As Single

        NH4NexportRNP = RNP(1) * FlushProp(iyr, iday) * FlushPropMult         'g N/day
        RNP(1) = RNP(1) - NH4NexportRNP                                       'g N
        NO3NexportRNP = RNP(2) * FlushProp(iyr, iday) * FlushPropMult         'g N / day
        RNP(2) = RNP(2) - NO3NexportRNP

        'Bookkeeping: keep track of N exported annually
        NH4NhydexpYr = NH4NhydexpYr + NH4NexportRNP / GridArea    'g N/m2 y
        NO3NhydexpYr = NO3NhydexpYr + NO3NexportRNP / GridArea
        'EcosNinout is for mass balance check
        EcosNinout = EcosNinout - NH4NexportRNP - NO3NexportRNP

    End Sub
    Private Sub BGNUptake()
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  This sub is called once per day during the growing season, from Main.
        '  Belowground plant N demand is calculated for each plant stem, N is competed for
        'and distributed among all plant stems in each cell, and taken up into belowground tissue. 
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        Dim ix, iy As Short                          'x, y indices in spatial grid cells 
        Dim ispp As Short                            'species index
        Dim iind As Integer                          'plant individual index  
        Dim NH4NUptake, NO3NUptake As Single         'NH4-N and NO3-N uptake flux for one individual plant, per timestep (g N/day)
        Dim NH4NUptakeDay, NO3NUptakeDay As Single   'NH4-N and NO3-N uptake flux for the area as a whole, per unit area (g N/m2 day)
        Dim RhizBGC, RhizBGN As Single               'Belowground C and N pool of indiv plant Rhiz (g C or g N)
        Dim RhizBGNdemand As Single                  'N uptake demand by individual plant belowground part (g N/day)
        Dim BGNdemand(5, maxindivs) As Single        'N uptake demand by species and Rhiz individual (g N/day)
        Dim CellNdemand As Single                    'N uptake demand totalled within a single grid cell, (g N/day)
        Dim CellNavail As Single                     'N available in single grid cell (g N)
        Dim CellNH4Nprop As Single                   'Proportion of available N in grid cell that is NH4-N 
        Dim NDScaleBackProp As Single                'N Demand Scale-Back Proportion, within a single grid cell (N mass ratio)

        NH4NUptakeDay = 0 : NO3NUptakeDay = 0
        '****** Belowground competition and uptake of N within each grid cell. Proceed cell by cell. 
        'N is taken up only into belowground plant component. 
        For ix = 1 To GridixMax
            For iy = 1 To GridiyMax
                CellNdemand = 0

                'Go through all plant individuals
                For ispp = 1 To 5
                    ' Note this code block will only execute for spp included in this run, where iindmax(ispp) > 0
                    For iind = 1 To iindmax(ispp)
                        'Read belowground C and N pools for this Rhiz into temporary variable for clarity
                        ' Here for belowground C we use only the rhizome C subject to growth once established;
                        ' not including the structural internode belowground C (PIP(ispp, iind, 5))
                        RhizBGC = PIP(ispp, iind, 3)              'g C
                        RhizBGN = PIP(ispp, iind, 4)              'g N
                        'Test to see if this is a nonzero plant individual, and only proceed if it is
                        'Use belowground C > 0 to indicate a nonzero plant individual
                        If RhizBGC > 0 Then
                            'Test to see if the indiv plant Rhiz is in the current grid cell.  
                            If (Int(pil(ispp, iind, 1) / s) + 1) = ix And (Int(pil(ispp, iind, 2) / s) + 1) = iy Then
                                'If indiv Rhiz is in this cell, calculate N demand of the belowground part
                                'Demand calc is based on max belowground C size, current belowground C size, 
                                '  and target C/N ratio for new tissue growth for the species. 
                                'Note that since we will add N via uptake, but not add C at the moment,
                                '  the C/N ratio is allowed to float, concentrating N for later aboveground growth. 
                                'Note that this does not try to make up for any existing N deficit that might be in the
                                '  tissue; possible future improvement
                                RhizBGNdemand = RhizBGC * relGrate(ispp) / (initBGCNrat(ispp))          'g N/day
                                'If Ndemand calculated above would make belowground plant part larger than its max size
                                ' divided by target C/N ratio, re-calculate to avoid that overgrowth
                                If (RhizBGNdemand + RhizBGN) > (maxPlantBGC(ispp) / initBGCNrat(ispp)) Then
                                    RhizBGNdemand = (maxPlantBGC(ispp) / initBGCNrat(ispp)) - RhizBGN      'g N/day
                                End If
                                CellNdemand = CellNdemand + RhizBGNdemand              'g N/day
                                BGNdemand(ispp, iind) = RhizBGNdemand                  'g N/day
                            End If   'Test of whether individual plant Rhiz is in the current grid cell 
                        End If   'Test of whether this is a nonzero plant individual
                    Next   'iind
                Next    'ispp

                'Now that N demand within the cell has been totalled for all plant individuals and species within the cell,
                'Apportion the available N in the cell among all Rhizs in the cell, in proportion to their demand
                'N uptake will be calculated as N demand scaled back in equal proportion for all individuals in the cell.
                'Only allow 90% of the available N in the cell to be taken up in total.
                '  At the same time, apportion the N uptake of each individual between NH4-N and NO3-N based on the ratio
                'of the two in this grid cell prior to any N uptake.  
                CellNavail = GCP(ix, iy, 3) + GCP(ix, iy, 4)
                CellNH4Nprop = GCP(ix, iy, 3) / (GCP(ix, iy, 3) + GCP(ix, iy, 4))
                If CellNdemand > 0.9 * CellNavail Then
                    NDScaleBackProp = 0.9 * CellNavail / CellNdemand
                Else
                    NDScaleBackProp = 1
                End If

                'Go through all plant individuals again, executing plant N uptake into belowground part for each individual
                For ispp = 1 To 5
                    ' Note this code block will only execute for spp included in this run, where iindmax(ispp) > 0
                    For iind = 1 To iindmax(ispp)
                        RhizBGC = PIP(ispp, iind, 3)              'g C
                        RhizBGN = PIP(ispp, iind, 4)              'g N
                        'Test to see if this is a nonzero plant individual, and only proceed if it is
                        'Use belowground C > 0 to indicate a nonzero plant individual
                        If RhizBGC > 0 Then
                            'Test to see if the indiv plant Rhiz is in the current grid cell. If so, take up N.   
                            If (Int(pil(ispp, iind, 1) / s) + 1) = ix And (Int(pil(ispp, iind, 2) / s) + 1) = iy Then
                                '****************** NH4-N uptake ****************************************************
                                'Calculate its NH4-N uptake based on its demand, the scale-back factor for this cell,
                                'and the proportion of available N that is NH4-N in this grid cell 
                                NH4NUptake = NDScaleBackProp * CellNH4Nprop * BGNdemand(ispp, iind)            'g N/day
                                'Test to ensure that Grid Cell NH4-N pool is not negative; zero-out any small round-off error
                                If GCP(ix, iy, 3) < 0 Then
                                    If GCP(ix, iy, 3) < -0.00001 Then
                                        ErrCount = ErrCount + 1
                                        ErrLast = 3
                                    End If
                                    GCP(ix, iy, 3) = 0
                                End If
                                'Transfer the NH4-N uptake: this amount of N from the grid cell NH4-N pool to the individual plant belowground part
                                ' Include error-avoiding code that disallows Grid Cell NH4-N pool to be drawn below zero
                                If NH4NUptake > GCP(ix, iy, 3) Then NH4NUptake = 0.9 * GCP(ix, iy, 3)
                                PIP(ispp, iind, 4) = PIP(ispp, iind, 4) + NH4NUptake
                                GCP(ix, iy, 3) = GCP(ix, iy, 3) - NH4NUptake                  'g N
                                'Double-check to make sure that GCP N pool was not drawn below zero
                                If GCP(ix, iy, 3) < 0 Then
                                    ErrCount = ErrCount + 1
                                    ErrLast = 3
                                End If

                                '****************** NO3-N uptake ****************************************************
                                'Calculate its NO3-N uptake based on its demand, the scale-back factor for this cell,
                                'and the proportion of available N that is NO3-N in this grid cell 
                                NO3NUptake = NDScaleBackProp * (1 - CellNH4Nprop) * BGNdemand(ispp, iind)            'g N/day
                                'Test to ensure that Grid Cell NO3-N pool is not negative; zero-out any small round-off error
                                If GCP(ix, iy, 4) < 0 Then
                                    If GCP(ix, iy, 4) < -0.00001 Then
                                        ErrCount = ErrCount + 1
                                        ErrLast = 3
                                    End If
                                    GCP(ix, iy, 4) = 0
                                End If
                                'Transfer the NO3-N uptake: this amount of N from the grid cell NO3-N pool to the individual plant belowground part
                                ' Include error-avoiding code that disallows Grid Cell NO3-N pool to be drawn below zero
                                If NO3NUptake > GCP(ix, iy, 4) Then NO3NUptake = 0.9 * GCP(ix, iy, 4)
                                PIP(ispp, iind, 4) = PIP(ispp, iind, 4) + NO3NUptake
                                GCP(ix, iy, 4) = GCP(ix, iy, 4) - NO3NUptake                  'g N
                                'Double-check to make sure that GCP NO3-N pool was not drawn below zero
                                If GCP(ix, iy, 4) < 0 Then
                                    ErrCount = ErrCount + 1
                                    ErrLast = 3
                                End If

                                'Record whether the N demand was scaled back, in the LFASpp array
                                If NDScaleBackProp < 1 Then LFASpp(ispp, 1) += 1

                                'Add individual N uptake to daily totals, convert units to per m2
                                NH4NUptakeDay = NH4NUptakeDay + NH4NUptake / GridArea               'g N/m2 day
                                NO3NUptakeDay = NO3NUptakeDay + NO3NUptake / GridArea               'g N/m2 day

                            End If   'Test of whether individual plant Rhiz is in the current grid cell 
                        End If  'Test of whether this is a nonzero plant individual
                    Next    'ispp
                Next   'iind
            Next    'iy
        Next    'ix

        'Bookkeeping: Increment annual N uptake with daily N uptake 
        NH4NUptakeYr = NH4NUptakeYr + NH4NUptakeDay              'g N/m2 y
        NO3NUptakeYr = NO3NUptakeYr + NO3NUptakeDay              'g N/m2 y

    End Sub
    Private Sub AGStemEstablish(ByVal iday As Short)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  Here, aboveground stem parts are established, either in clonal reproduction after the
        ' establishment of a daughter rhizome node, or at the start of a new growing season 
        ' for a perennial rhizome that survived over the winter but lost its herbaceous stem to  
        ' litterfall.  It needs to re-establish an herbaceous stem each year.    
        '
        '  Above existing belowground tissue for an individual, establish a new  
        ' aboveground stems if there is not one already there and if the belowground
        ' plant pools have enough C and N.  The C and N for new stem establishment has to be 
        ' passed from the belowground plant tissue of the individual.   
        '
        '  The new aboveground stem has to be able to push up through the litter layer to 
        ' become established (if this model structural option of litter suppression is 'on').
        ' This is proportionally more difficult to do as the litter layer is more massive
        ' and the species max stem size is smaller. 
        '
        '  If the belowground part of an individual has no aboveground tissue and is not able 
        ' to establish one at present, then it can ask for parental subsidies of C and N 
        ' (controlled by a model structural option that determines how many parents back
        ' the subsidies can come from.)  Then it has to wait until this subroutine is called 
        ' again to see if it will now be able to make a new stem.  
        '
        ' If it gets to the end of the growing season without establishing an aboveground
        ' stem, then the belowground part may die and become litter, or it may
        ' live to try again next year.  This is controlled through a user-set parameter. 
        '
        ' This Sub is called throughout the growing season, from Main.
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        Dim ispp As Short                       'species index
        Dim iind As Integer                     'individual plant index number
        Dim iindp(5) As Integer                 'individual index number of each parent back in the chain.
        ' iindp(1) = 1st-back parent, iindp(2) = 2nd-back parent, etc. 
        Dim ixGridLoc, iyGridLoc As Short
        Dim PropBGCNneed As Single              'proportion of max bg plant C and N needed to establish ag stem 
        Dim AGEstCtest, AGEstNtest As Boolean   'Tests whether there is enough C and N to establish AG stem

        For ispp = 1 To 5
            ' Note this code block will only execute for spp included in this run, where iindmax(ispp) > 0
            For iind = 1 To iindmax(ispp)
                'Test to see if there is a nonzero belowground individual in this slot
                'AND if there is zero C in the aboveground plant part for that individual
                'AND no standing dead litter in the aboveground plant part for that individual
                If PIP(ispp, iind, 3) > 0 And PIP(ispp, iind, 1) = 0 And PIP(ispp, iind, 6) = 0 Then
                    'Figure out which grid cell the individual plant is in
                    'First grid cell is numbered (1,1); see notes
                    ixGridLoc = Int(pil(ispp, iind, 1) / s) + 1
                    iyGridLoc = Int(pil(ispp, iind, 2) / s) + 1

                    'Re-store this individual rhizome node's parental pedigree information in simpler form
                    'for use in asking parents for C and N subsidies (below)
                    iindp(1) = pgr(ispp, iind, 6)
                    iindp(2) = pgr(ispp, iind, 7)
                    iindp(3) = pgr(ispp, iind, 8)
                    iindp(4) = pgr(ispp, iind, 9)
                    iindp(5) = pgr(ispp, iind, 10)

                    '************************ Litter suppression of stem establishment ***************************
                    'Calculate the proportion of max aboveground plant C and N that are needed to establish
                    ' an aboveground plant stem in this cell.  Call it PropBGCNneed. 
                    'Model Structure Option:  If user has selected LitSuppressVR, then 
                    ' the calculation of PropBGCNneed is based on the amount of above-ground litter C present
                    ' in this cell; it takes more C and N in the new stem to push up through more litter.
                    'The relationship between overlying litter C and PropBGCNneed is controlled
                    ' by the parameter AGEstParmA.   
                    'Grid cell above-ground litter C pool is divided by grid cell area to make AGEstParmA based on g litter C/m2
                    'Model Structure Option:  If user has chosen LitSuppressVR = False, then 
                    ' PropBGCNneed is a constant proportion regardless of litter layer mass. 
                    If LitSuppressVR Then
                        PropBGCNneed = GCP(ixGridLoc, iyGridLoc, 1) * AGEstparmA(ispp) / (s ^ 2)   'unitless proportion

                        'The minimum PropBGCNneed is a parameter in user input file, the maximum is 90%.  
                        If PropBGCNneed < PBGCNNmin(ispp) Then
                            PropBGCNneed = PBGCNNmin(ispp)
                        ElseIf PropBGCNneed > 0.9 Then
                            PropBGCNneed = 0.9
                        End If
                    Else
                        PropBGCNneed = PBGCNNmin(ispp)
                    End If

                    'Bookkeeping:  keep track of PropBGCNneed stats to calculate average later
                    PropBGCNneedAnnTot(ispp) += PropBGCNneed
                    PropBGCNneedAnnNum(ispp) += 1

                    'Test to see if BG rhizome has enough C and N to establish a new aboveground stem.  
                    'AGestCtest and AGEstNtest are both returned as boolean; if both true, test is passed 
                    Call AGStemEstabTest(ispp, iind, PropBGCNneed, AGEstCtest, AGEstNtest)

                    If AGEstCtest And AGEstNtest Then
                        'Create a new aboveground stem
                        Call AGStemCreate(ispp, iind, PropBGCNneed)

                    Else
                        'If control gets to here, that means there is a belowground part, with no aboveground stem
                        ' above it, and not enough belowground C and N to establish a new aboveground stem.

                        'Check user option:  are ramets allowed to get subsidy from 1 parent back?
                        If SubsidyPar(ispp) >= 1 Then
                            '******************** Parental ramet subsidy of C and N, 1 parent back ********************
                            ' Test whether first-back parent is alive 
                            If pgr(ispp, iind, 6) > 0 And PIP(ispp, iindp(1), 3) > 0 Then
                                'Test whether 1st-back parent rhizome has an aboveground stem OR whether non-stem pass-through is ON    
                                If PIP(ispp, iindp(1), 1) > 0 Or PTNSN = True Then
                                    '  Hooray!  We can now ask this parent for some C and N subsidy. 

                                    Call ParentSubsidy(ispp, iind, iindp(1))

                                    '******************** Re-do test to see if new stem can be created **********************
                                    'Test to see if BG rhizome has enough C and N to establish a new aboveground stem.  
                                    'AGestCtest and AGEstNtest are both returned as boolean; if both true, test is passed 
                                    Call AGStemEstabTest(ispp, iind, PropBGCNneed, AGEstCtest, AGEstNtest)

                                    If AGEstCtest And AGEstNtest Then
                                        'If tests were passed, create a new aboveground stem
                                        Call AGStemCreate(ispp, iind, PropBGCNneed)

                                        'If the tests were not passed, then check user option:  are ramets allowed to get subsidy from 2 parents back? 
                                    ElseIf SubsidyPar(ispp) >= 2 Then
                                        '******************** Parental ramet subsidy of C and N, 2 parents back *************
                                        ' Test whether 2nd-back parent rhizome is alive
                                        If pgr(ispp, iind, 7) > 0 And PIP(ispp, iindp(2), 3) > 0 Then
                                            'Test whether 2nd-back parent rhizome has an aboveground stem OR whether non-stem pass-through is ON  
                                            If PIP(ispp, iindp(2), 1) > 0 Or PTNSN = True Then
                                                'We can now ask the 2nd-back parent for some C and N subsidy

                                                Call ParentSubsidy(ispp, iind, iindp(2))

                                                '******************* Re-do test to see if new stem can be created ********************************************
                                                'Test to see if BG rhizome has enough C and N to establish a new aboveground stem. 
                                                'AGEstCtest And AGEstNtest are both returned as boolean; if both true, test is passed 
                                                Call AGStemEstabTest(ispp, iind, PropBGCNneed, AGEstCtest, AGEstNtest)

                                                If AGEstCtest And AGEstNtest Then
                                                    'If tests were passed, then create a new aboveground stem
                                                    Call AGStemCreate(ispp, iind, PropBGCNneed)

                                                    'If the tests were not passed, then check user option:  are ramets allowed to get subsidy from 3 parents back?
                                                ElseIf SubsidyPar(ispp) >= 3 Then
                                                    '***************** Parental ramet subsidy of C and N, 3rd parent back **************************************
                                                    ' Test whether the 3rd-back parent rhizome is alive 
                                                    If pgr(ispp, iind, 8) > 0 And PIP(ispp, iindp(3), 3) > 0 Then
                                                        'Test whether 3rd-back parent rhizome has an aboveground stem OR whether non-stem pass-through is ON 
                                                        If PIP(ispp, iindp(3), 1) > 0 Or PTNSN = True Then
                                                            'We can now ask the 3rd-back parent for some C and N subsidy. 

                                                            Call ParentSubsidy(ispp, iind, iindp(3))

                                                            '*********** Re-do test to see if new stem can be created ****************************************
                                                            'Test to see if BG rhizome has enough C and N to establish a new aboveground stem. 
                                                            'AGEstCtest And AGEstNtest are both returned as boolean; if both true, test is passed 
                                                            Call AGStemEstabTest(ispp, iind, PropBGCNneed, AGEstCtest, AGEstNtest)

                                                            If AGEstCtest And AGEstNtest Then
                                                                'If tests were passed, create a new aboveground stem
                                                                Call AGStemCreate(ispp, iind, PropBGCNneed)

                                                                'If the tests were not passed, then check user option: are ramets allowed to get subsidy from 4 parents back?
                                                            ElseIf SubsidyPar(ispp) >= 4 Then
                                                                '******** Parental ramet subsidy of C and N, 4th parent back ***********************************
                                                                ' Test whether the 4th-back parent rhizome is alive 
                                                                If pgr(ispp, iind, 9) > 0 And PIP(ispp, iindp(4), 3) > 0 Then
                                                                    'Test whether 4th-back parent rhizome has an aboveground stem OR whether non-stem pass-through is ON 
                                                                    If PIP(ispp, iindp(4), 1) > 0 Or PTNSN = True Then
                                                                        'Hooray!  We can now ask the 4th-back parent for some C and N subsidy. 

                                                                        Call ParentSubsidy(ispp, iind, iindp(4))

                                                                        '******************** Re-do test to see if new stem can be created **************************
                                                                        'Test to see if BG rhizome has enough C and N to establish a new aboveground stem.  
                                                                        'AGestCtest and AGEstNtest are both returned as boolean; if both true, test is passed 
                                                                        Call AGStemEstabTest(ispp, iind, PropBGCNneed, AGEstCtest, AGEstNtest)

                                                                        If AGEstCtest And AGEstNtest Then
                                                                            'If tests were passed, then create a new aboveground stem
                                                                            Call AGStemCreate(ispp, iind, PropBGCNneed)

                                                                            'If the tests were not passed, then check user option:  are ramets allowed to get subsidy from 5 parents back?
                                                                        ElseIf SubsidyPar(ispp) >= 5 Then
                                                                            '******************** Parental ramet subsidy of C and N, 5th parent back ********************
                                                                            ' Test whether the 5th-back parent rhizome is alive 
                                                                            If pgr(ispp, iind, 10) > 0 And PIP(ispp, iindp(5), 3) > 0 Then
                                                                                'Test whether 5th-back parent rhizome has an aboveground stem OR whether non-stem pass-through is ON 
                                                                                If PIP(ispp, iindp(5), 1) > 0 Or PTNSN = True Then
                                                                                    'Hooray!  We can now ask the 5th-back parent for some C and N subsidy. 

                                                                                    Call ParentSubsidy(ispp, iind, iindp(5))

                                                                                    '******************** Re-do test to see if new stem can be created **************************
                                                                                    'Test to see if BG rhizome has enough C and N to establish a new aboveground stem.  
                                                                                    'AGestCtest and AGEstNtest are both returned as boolean; if both true, test is passed 
                                                                                    Call AGStemEstabTest(ispp, iind, PropBGCNneed, AGEstCtest, AGEstNtest)

                                                                                    If AGEstCtest And AGEstNtest Then
                                                                                        'If tests were passed, then create a new aboveground stem
                                                                                        Call AGStemCreate(ispp, iind, PropBGCNneed)

                                                                                    End If  'end of test whether C and N tests were passed after subsidy from 5th-back parent

                                                                                    '*** Closing all the conditionals for possible subsidy from 5th-back parent
                                                                                End If  'end of test whether 5th-back parent rhizome is alive and has an aboveground stem
                                                                            End If  'end of test whether 5th parent back has a nonzero identifier number
                                                                        End If 'test of whether subsidy is allowed from 5th-back parent

                                                                        '*** Closing all the conditionals for possible subsidy from 4th-back parent
                                                                    End If  'end of test whether 4th-back parent rhizome is alive and has an aboveground stem
                                                                End If  'end of test whether 4th parent back has a nonzero identifier number 
                                                            End If 'test of whether subsidy is allowed from 4th-back parent

                                                            '*** Closing all the conditionals for possible subsidy from 3rd-back parent
                                                        End If  'end of test whether 3rd-back parent rhizome is alive and has an aboveground stem
                                                    End If  'end of test whether 3rd parent back has a nonzero identifier number
                                                End If 'test of whether subsidy is allowed from 3rd-back parent

                                                '*** Closing all the conditionals for possible subsidy from 2nd-back parent
                                            End If  'end of test whether 2nd-back parent rhizome is alive and has an aboveground stem
                                        End If  'end of test whether 2nd-back parent has a nonzero identifier number
                                    End If  'end of test whether subsidy is allowed from 2nd-back parent

                                    '*** Closing all the conditionals for possible subsidy from 1st-back parent
                                End If  'end of test whether 1st-back parent rhizome is alive and has an aboveground stem
                            End If  'end of test whether first-back parent has a nonzero identifier number
                        End If  'end of test whether subsidy is allowed from 1st-back parent

                        '*** Closing conditionals for creation of a new stem without any parental subsidy
                    End If  'Test of whether there is enough belowground C and N to establish an aboveground stem tissue on its own

                    '****************** Death of unsuccessful rhizome at end of growing season ********************
                    'Test to see if the aboveground C is still zero.  If it is, and if it is nearly the last day of the 
                    ' growing season, the rhizome may die.  Depends on value of user-set parameter.   
                    If PIP(ispp, iind, 1) = 0 And iday >= (GSend - 1) Then
                        'Increment the number of growing seasons it has gone without producing a stem.
                        'Note that this is cumulative; it is not re-set to zero if the rhizome makes a stem later.  
                        pgr(ispp, iind, 5) += 1
                        'If it has gone all the growing seasons it is allowed to go without a stem, it dies
                        If pgr(ispp, iind, 5) = BGpersist Then BGMortality(ispp, iind)
                    End If   'Test of whether it is the end of the growing season

                End If  'Test of whether there is a belowground part, without a aboveground stem above it

                '*** loop back for next individual 
            Next  'iind
        Next  'ispp

    End Sub

    Private Sub AGStemEstabTest(ByVal ispp As Short, ByVal iind As Integer, ByVal PropBGCNneed As Single, ByRef AGEstCTest As Boolean, ByRef AGEstNtest As Boolean)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  This sub tests to see if there is enough C and N in a belowground rhizome 
        ' to create a new aboveground plant stem.  
        '  Called by AGStemEstablish. 
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        ' Note that belowground rhizome must also retain 10% of its max C and N to pass these tests
        AGEstCTest = False : AGEstNtest = False
        If PIP(ispp, iind, 3) >= PropBGCNneed * maxPlantAGC(ispp) + 0.1 * maxPlantBGC(ispp) Then
            AGEstCTest = True
        End If
        If PIP(ispp, iind, 4) >= PropBGCNneed * maxPlantAGC(ispp) / initAGCNrat(ispp) + 0.1 * maxPlantBGC(ispp) / initBGCNrat(ispp) Then
            AGEstNtest = True
        End If

        'LFASpp is for logical flow analysis by species
        If AGEstCTest = True And AGEstNtest = False Then
            LFASpp(ispp, 2) += 1
        ElseIf AGEstCTest = False And AGEstNtest = True Then
            LFASpp(ispp, 3) += 1
        End If
    End Sub
    Private Sub AGStemCreate(ByVal ispp As Short, ByVal iind As Integer, ByVal PropBGCNneed As Single)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  This sub creates a new aboveground stem. 
        '  Called by AGStemEstablish. 
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim CtranstoAG As Single       'C flux to translocate to new aboveground stem pool, units g C/day
        Dim NtranstoAG As Single       'N flux to translocate to new aboveground stem pool, units g N/day

        'This belowground individual now creates a new aboveground stem tissue above it. 
        CtranstoAG = PropBGCNneed * maxPlantAGC(ispp)                  'g C / day

        'Now transfer the C from belowground part to aboveground part. 
        'this creation of nonzero C in the aboveground part effectively establishes the 
        'existence of an aboveground part for this plant individual
        PIP(ispp, iind, 1) = CtranstoAG                             'g C
        PIP(ispp, iind, 3) = PIP(ispp, iind, 3) - CtranstoAG        'g C

        'Now transfer the N associated with the C transfer calculated above.  
        'Use the growth target C/N ratio for new aboveground tissue for the species. 
        NtranstoAG = CtranstoAG / initAGCNrat(ispp)                 'g N / day
        'Error check:  if there is nonzero N in the aboveground stem N pool before transfer, record an error
        If PIP(ispp, iind, 2) > 0 Then
            If PIP(ispp, iind, 2) < 0.000001 Then
                'If control gets here, assume this is a round-off error in previous calcs 
                PIP(ispp, iind, 2) = 0
            Else
                ErrCount = ErrCount + 1
                ErrLast = 5
            End If
        End If

        PIP(ispp, iind, 2) = NtranstoAG
        PIP(ispp, iind, 4) = PIP(ispp, iind, 4) - NtranstoAG        'g N

        'LFA analysis:  counter 4 counts stem creation; note that this does not record 
        'whether or not a parental subsidy was needed.  Future change could allow different LFA counters
        'to distinguish whether or not a parental subsidy was needed.  
        LFASpp(ispp, 4) += 1

    End Sub
    Private Sub ParentSubsidy(ByVal ispp As Short, ByVal iind As Integer, ByVal iindp As Integer)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  This sub tests to see if a parent individual, whether a 1st-back, 2nd-back, 3rd-back
        ' parent, etc., can provide a C or N subsidy to the current individual (iind). 
        '  The sub that calls this one (AGStemEstablish), knows whether the parent is a 1st-back, 
        ' 2nd-back, 3rd-back, etc; and has established that the generalized parent exists and is 
        ' allowed to be asked for a subsidy.  But in this sub all we know is that it is a parent (iindp)  
        ' at some point back in the chain and it is being asked to provide a subsidy.  
        '  If the parent is able to provide a C or N subsidy, it is executed here.  
        '  Csubsidy, Nsubsidy are the fluxes to transfer from parent rhizome (iindp) to current indiv (iind).
        '  Note these are simply calculated as the maximum amount of C and N that the parent
        ' rhizome could give, based on its current pools and the amounts it needs to keep.  
        '
        '  Called by AGStemEstablish. 
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim Csubsidy As Single         'C flux subsidy from parent rhizome to daughter rhizome, g C/day
        Dim Nsubsidy As Single         'N flux subsidy from parent rhizome to daughter rhizome, g N/day

        Csubsidy = PIP(ispp, iindp, 3) - rhizRetCProp(ispp) * maxPlantBGC(ispp)     'g C/day
        Nsubsidy = PIP(ispp, iindp, 4) - rhizRetCProp(ispp) * maxPlantBGC(ispp) / initBGCNrat(ispp)   'g N/day

        'If Csubsidy and Nsubsidy are > 0 then go ahead and make the transfer
        'Note that C and N subsidies from parent rhizome are decoupled from each other
        If Csubsidy > 0 Then
            PIP(ispp, iindp, 3) = PIP(ispp, iindp, 3) - Csubsidy      'g C
            PIP(ispp, iind, 3) = PIP(ispp, iind, 3) + Csubsidy        'g C
        End If  'end of test whether Csubsidy > 0
        If Nsubsidy > 0 Then
            PIP(ispp, iindp, 4) = PIP(ispp, iindp, 4) - Nsubsidy      'g N
            PIP(ispp, iind, 4) = PIP(ispp, iind, 4) + Nsubsidy        'g N
        End If  'end of tests whether Nsubsidy > 0

    End Sub
    Private Sub AGPlantGrowth(ByVal iday As Short)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  Here, existing aboveground tissues undergo growth.  
        '  This sub is called each day of the growing season, from Main. 
        '
        '  Growth is N limited, with N coming from belowground tissue. 
        '  As N comes up from belowground to aboveground tissue, aboveground growth occurs,
        ' from photosynthesis.  Here, the aboveground part gains C from the atmosphere, not from
        ' the belowground part.  Concomitant with the aboveground tissue C growth is an 
        ' additional(flux)of C from photosynthesis that is translated to the belowground
        ' part of the stem. 
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim ispp, ibin As Short
        Dim iind As Integer
        Dim ix, iy As Short
        Dim RamAGNdemand As Double    'N demand related to growth for aboveground part of indiv Ramet
        Dim RamAGC As Double          'C in aboveground living plant part (including stems) for an individual Ramet
        Dim RamSDC As Double          'C in aboveground standing dead litter for an individual ramet 
        Dim RamAGN As Double          'N in aboveground plant part pool for an individual Ramet
        Dim RamBGC As Double          'C in belowground plant part pool for an individual Ramet
        Dim RamBGN As Double          'N in belowground plant part pool for an individual Ramet
        Dim NtranstoAG As Double       'N flux to translocate to new aboveground stem pool, units g N/day
        Dim CAGgrowth As Single        'C flux in aboveground plant growth of one individual stem, g C/day
        Dim CtranstoBG As Single       'C transfer of photosynthate from above to below ground part of Ramet, g C/day
        Dim CnetPs As Single           'C flux fixed in photosynthesis in indiv plant stem, g C/day
        Dim CNPPDay As Single          'C flux, daily total NPP across all species and individuals
        Dim CAGgrowthDay As Single     'C flux, daily total CAGgrowth across all species and individuals
        Dim CBGallocDay As Single      'C flux, daily total CtranstoBG across all species and individuals
        Dim CCanopyLayer(GridixMax, GridiyMax, 5, 41) As Single  'C (g C / m2) in each 10 cm layer in each grid cell, species-specific, for calculating shading
        Dim CCanopySumCell(5, 41) As Single  'C (g C/m2 summed vertically for each layer and all above
        Dim BioCanopySumCell As Single  'CCanopySumCell converted to biomass (dry wt) within one cell 
        Dim PercentFullSun(GridixMax, GridiyMax, 41)  '% full sunlight in each vertical layer in each grid cell
        Dim PercentSunReduction, PercentSunRemaining As Single  'For calculation of light extinction within a grid cell 
        Dim h As Short                 'integer number of 0.10 meter segments high for particular individual based on height
        Dim Lgtbin As Single           'vertical layer bin to calculate the light environment experienced by an individual in its grid cell
        Dim RGRLgtMod As Single        'RGR light (shading) modification:  index from 0 to 1 for an individual plant

        'Partition aboveground living and standing dead C into vertical layers separately within each grid cell  
        For ispp = 1 To 5
            For iind = 1 To iindmax(ispp)
                RamAGC = PIP(ispp, iind, 1)
                RamSDC = PIP(ispp, iind, 6)

                'Error checking: test to see if above ground ramet has both living C and standing dead C; should never happen
                If RamAGC > 0 And RamSDC > 0 Then
                    ErrCount = ErrCount + 1
                    ErrLast = 35
                End If
                'Test to see if there is either living or standing dead tissue for this individual;
                ' if there is, calculate its height 
                If RamAGC + RamSDC > 0 Then Call AGHeightCalc(ispp, iind)

                'figure out which grid cell (ix, iy) each indiv is in using pil array
                ix = (Int(pil(ispp, iind, 1) / s) + 1)
                iy = (Int(pil(ispp, iind, 2) / s) + 1)

                '  Use AGC + SDC and height to partition plant C in vertical layers and add to layers for the correct grid cell and species 
                ' C vertical layers array is split into 0.1 m (10 cm) layers, index refers to bottom of each vertical layer
                ' i.e., vertical index '5' holds C in layer 0.50 to 0.60 m. 
                '  Note that because SDC is included, standing dead litter shades other plants just like live plant tissue does.  
                '  Zero value of C canopy vertical layers is defined as the top of the MUCK layer, i.e. the plant
                ' stem-rhizome interface 

                'Think of vertical layers for shading as 40 bins, 0.10 m high each.  
                '  For this plant individual, given its height in meters, how many bins high is it?
                '  Note how vertical bins are defined:  0 to 0.1 m is bin 1; 0.1 to 0.2 m is bin 2, etc.   
                h = Int(10 * Phgt(ispp, iind)) + 1
                For ibin = 1 To h
                    CCanopyLayer(ix, iy, ispp, ibin) += (RamAGC + RamSDC) / (h * s ^ 2)                   'g C / m2
                Next
            Next
        Next

        'loop through all grid cells to calculate summed light extinction, by grid cell and vertical layer, across all species 
        For ix = 1 To GridixMax
            For iy = 1 To GridiyMax
                '   calc % full sun in layers (save this), summed over shading from all species
                ReDim CCanopySumCell(5, 41)   'C (g C/m2 summed vertically for each layer and all above
                For ispp = 1 To 5
                    CCanopySumCell(ispp, 40) = CCanopyLayer(ix, iy, ispp, 40)
                    For ibin = 39 To 1 Step -1
                        CCanopySumCell(ispp, ibin) = CCanopyLayer(ix, iy, ispp, ibin) + CCanopySumCell(ispp, (ibin + 1))
                    Next
                Next

                'Now I have, within this particular grid cell, for each of the species, an array that holds sums of C in 
                ' each vertical layer and in all vertical layers above 
                'Now use this to calculate light extinction by species within this grid cell 
                ' And add light extinction across all species, and save that for this grid cell 
                For ibin = 1 To 40
                    PercentFullSun(ix, iy, ibin) = 100
                    For ispp = 1 To 5
                        'Convert (g C / m2) to (g dry wt / m2)
                        BioCanopySumCell = CCanopySumCell(ispp, ibin) / cconcTissue              'g dry wt / m2

                        'Only calculate shading if there is > 0 biomass for this species in this vertical bin.  
                        'This fixes a glitch in which the value of LgtExtCurveC causes a problem in the calculation when biomass is zero 
                        If BioCanopySumCell = 0 Then
                            PercentSunReduction = 0
                        Else
                            'Since light extinction is a polynomial curve fit, we need to protect against the light extinction 
                            ' curve going back up if biomass is really high.  Thus force it to 0.5% sun remaining, at biomass values above threshold supplied by user 
                            If BioCanopySumCell > BioFullShade(ispp) Then
                                PercentSunReduction = 99.95
                            Else
                                'Use polynomial curve fit for light extinction, % light remaining = A*x2 + B*x + C (where x is sum of biomass by spp in grid cell) 
                                PercentSunRemaining = LgtExtCurveA(ispp) * BioCanopySumCell ^ 2 + LgtExtCurveB(ispp) * BioCanopySumCell + LgtExtCurveC(ispp)
                                If PercentSunRemaining > 100 Then PercentSunRemaining = 100
                                If PercentSunRemaining < 0 Then PercentSunRemaining = 0
                                PercentSunReduction = 100 - PercentSunRemaining
                            End If
                            'Check that percent sunlight reduction is between 0 and 100%
                            If PercentSunReduction < 0 Or PercentSunReduction > 100 Then
                                ErrLast = 33
                                ErrCount = ErrCount + 1
                            End If

                        End If 'Test whether BioCanopySumCell > 0 
                        'Implement shading due to this species, in this layer, in a way that allows iterative shading
                        ' from all species in this layer, one species at a time.  
                        ' Do not let percent sun remaining in this layer go below 3% when aggregated across all species.
                        ' The reason for this 3% limit is because 0% sunlight would stop any individuals from reproducing, 
                        '   plus field light measurements seldom go below 3%.  
                        PercentFullSun(ix, iy, ibin) += -PercentSunReduction
                        If PercentFullSun(ix, iy, ibin) < 0.05 Then PercentFullSun(ix, iy, ibin) = 0.05

                    Next  'ispp
                Next  'ibin
            Next  'grid cell y
        Next  'grid cell x

        'Calculate aboveground growth and belowground C allocation for each individual 
        CNPPDay = 0 : CAGgrowthDay = 0 : CBGallocDay = 0
        For ispp = 1 To 5
            If iday = MGSPrintDay Then
                RGRLgtModSum(ispp) = 0 : RGRLgtModNum(ispp) = 0
            End If
            ' Note this code block will only execute for spp included in this run, where iindmax(ispp) > 0
            For iind = 1 To iindmax(ispp)
                RamAGC = PIP(ispp, iind, 1)
                'Test to see if there is existing aboveground plant part for this Ramet
                If RamAGC > 0 Then
                    RamAGN = PIP(ispp, iind, 2)
                    'For belowground C we use only the active portion subject to growth
                    RamBGC = PIP(ispp, iind, 3)
                    RamBGN = PIP(ispp, iind, 4)

                    '********** Calculate light competition effect on relative growth rate 
                    'figure out which grid cell (ix, iy) this indiv is in using pil array
                    ix = (Int(pil(ispp, iind, 1) / s) + 1)
                    iy = (Int(pil(ispp, iind, 2) / s) + 1)

                    Lgtbin = Int(10 * LgtEHeight(ispp) * Phgt(ispp, iind)) + 1

                    'RGRLgtMod is short for relative growth rate light modifier.  A value of 1 means no reduction 
                    ' in relative growth rate; values below 1 indicate reduction in growth of current indiv from shading. 
                    ' If light competition is turned on (LgtComp = TRUE), then RGRLgtMod is calculated using shading.
                    ' Otherwise RGRLgtMod is set equal to 1 to turn off light competition.  
                    If LgtComp Then
                        RGRLgtMod = 1.13 * PercentFullSun(ix, iy, Lgtbin) / (13 + PercentFullSun(ix, iy, Lgtbin))
                    Else
                        RGRLgtMod = 1
                    End If

                    'check that RGRLgtMod is between 0 and 1
                    If RGRLgtMod < 0 Or RGRLgtMod > 1 Then
                        ErrLast = 32
                        ErrCount = ErrCount + 1
                    End If

                    'One day of the year, save average RGRLgtMod by species and print it out 
                    If iday = MGSPrintDay Then
                        RGRLgtModSum(ispp) += RGRLgtMod
                        RGRLgtModNum(ispp) += 1
                    End If

                    '********** Calculate N demand of aboveground plant part ************
                    'Calculate N demand based on relative growth rate and light (shading) reduction modifier
                    RamAGNdemand = RGRLgtMod * RamAGC * relGrate(ispp) / (initAGCNrat(ispp))          'g N/day
                    'If Ndemand calculated above would make belowground plant part larger than its max size
                    ' divided by target C/N ratio, re-calculate to avoid that overgrowth
                    If (RamAGNdemand + RamAGN) > (maxPlantAGC(ispp) / initAGCNrat(ispp)) Then
                        RamAGNdemand = (maxPlantAGC(ispp) / initAGCNrat(ispp)) - RamAGN      'g N/day
                    End If

                    'If aboveground N demand would draw the belowground N down too much, as compared to the 
                    ' maximum belowground plant size, then scale it back.
                    If PIP(ispp, iind, 4) - RamAGNdemand < 0.1 * maxPlantBGC(ispp) / initBGCNrat(ispp) Then
                        RamAGNdemand = PIP(ispp, iind, 4) - 0.1 * maxPlantBGC(ispp) / initBGCNrat(ispp)
                        LFASpp(ispp, 5) += 1
                    End If

                    'Check to ensure that RamAGNdemand can never go below zero
                    If RamAGNdemand < 0 Then RamAGNdemand = 0

                    '********** Transfer N from belowground plant part to aboveground *********
                    NtranstoAG = RamAGNdemand                    'g N /day
                    PIP(ispp, iind, 2) = PIP(ispp, iind, 2) + NtranstoAG
                    PIP(ispp, iind, 4) = PIP(ispp, iind, 4) - NtranstoAG
                    'Zero out any round off error that caused belowground N to go below zero
                    'At same time, if belowground N pool went below zero by more than a small round-off error, 
                    ' record an error
                    If PIP(ispp, iind, 4) < 0 Then
                        If PIP(ispp, iind, 4) < -0.00001 Then
                            ErrCount = ErrCount + 1
                            ErrLast = 6
                        End If
                        PIP(ispp, iind, 4) = 0
                    End If

                    '********** Aboveground growth resulting from N transfer aboveground
                    'C growth in aboveground tissue is based on N transfer up from belowground tissue 
                    ' and target C:N stoichiometry for new aboveground growth
                    'It is assumed there is no light, water, or other nutrient limitation
                    CAGgrowth = NtranstoAG * initAGCNrat(ispp)               'g C/day
                    'There is no C pool to debit here, since it is assumed to come from the atmosphere
                    ' in photosynthesis. 
                    PIP(ispp, iind, 1) = PIP(ispp, iind, 1) + CAGgrowth      'g C

                    '********** C allocation to belowground plant part ************************
                    'An extra amount of photosynthesis is assumed to occur to produce photosynthate C
                    '  to allocate belowground.  
                    'Proportion of belowground C allocation is controlled by a spp-specific parameter CallocBGprop. 
                    CtranstoBG = (CallocBGprop(ispp) / (1 - CallocBGprop(ispp))) * CAGgrowth
                    'If this C transfer belowground would make the bg tissue larger than its max allowable
                    ' size, scale it back
                    If CtranstoBG + RamBGC > maxPlantBGC(ispp) Then
                        CtranstoBG = maxPlantBGC(ispp) - RamBGC
                    End If

                    'Transfer the C allocation belowground.  
                    ' There is nothing to debit here, since it is assumed to come from the atmosphere
                    '  in photosynthesis. 
                    ' The C allocation goes to the active, belowground portion subject to growth. 
                    PIP(ispp, iind, 3) = PIP(ispp, iind, 3) + CtranstoBG     'g C

                    '************ Bookkeeping *************************************************
                    'This is a quick way to model photosynthesis based just on N limitation. 
                    '  More mechanistic would be to calculate it based on light, temperature, leaf area, 
                    '  then allocate the resulting C fixation above and below ground (and include resp). 
                    CnetPs = CAGgrowth + CtranstoBG

                    'Increment daily total C fluxes, convert units to per m2
                    CNPPDaySpp(ispp) = CNPPDaySpp(ispp) + CnetPs / GridArea   'g C/m2 d
                    CNPPDay = CNPPDay + CnetPs / GridArea                     'g C/m2 d
                    CAGgrowthDay = CAGgrowthDay + CAGgrowth / GridArea        'g C/m2 d
                    'Increment species-specific aboveground NPP annual sum 
                    CANPPYrSpp(ispp) = CANPPYrSpp(ispp) + CAGgrowth / GridArea 'g C/m2 yr
                    CBGallocDay = CBGallocDay + CtranstoBG / GridArea         'g C/m2 d

                End If  'Test of whether there is existing aboveground plant part for this Ram
            Next
        Next

        'Increment species-specific above + belowground NPP annual sum
        For ispp = 1 To 5
            CNPPYrSpp(ispp) = CNPPYrSpp(ispp) + CNPPDaySpp(ispp)
        Next
        ' NPP and aboveground and belowground growth, all species aggregated together 
        CNPPYr = CNPPYr + CNPPDay                                          'g C/m2
        CAGgrowthYr = CAGgrowthYr + CAGgrowthDay                           'g C/m2
        CBGallocYr = CBGallocYr + CBGallocDay                              'g C/m2

        'EcosCinout for mass balance check
        EcosCinout = EcosCinout + CNPPDay * GridArea                       'g C

    End Sub
    Private Sub AGHeightCalc(ByVal ispp As Integer, ByVal iind As Integer)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '   Calculate the height of an individual aboveground stem based on allometric equation
        ' based on biomass 
        '
        ' Called by:  AGPlantGrowth, Litterfall 
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim RamAGC As Double          'C in aboveground living plant part (including stems) for an individual Ramet
        Dim RamSDC As Single          'C in aboveground standing dead litter for an individual ramet 

        RamAGC = PIP(ispp, iind, 1)
        RamSDC = PIP(ispp, iind, 6)

        ' Calculate plant indiv height and store in plant indiv array
        ' This includes both aboveground living and standing dead tissue, but it will be one or the other for an indiv 
        Phgt(ispp, iind) = AlloBioHgtA(ispp) * (((RamAGC + RamSDC) / cconcTissue) ^ AlloBioHgtB(ispp))     'm
        ' Max height is hardwired to 4.0 m
        If Phgt(ispp, iind) > 4.0 Then Phgt(ispp, iind) = 4.0

    End Sub

    Private Sub MgmtBioCut(ByVal ispp As Integer, ByVal iind As Integer, ByRef BioCutLitAGC As Single, ByRef BioCutLitSDC As Single, ByRef BioCutLitAGN As Single, ByRef BioCutLitSDN As Single)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        'Exectute management treatment BioCut (e.g. mowing) to control invasives. 
        '
        ' Called by:  Litterfall 
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim IndivHeight As Single  'Height (meters) of individual ramet 

        'Read individual plant height from array where this is stored 
        IndivHeight = Phgt(ispp, iind)

        If BioCutHeight < IndivHeight Then
            'Calculate new litter C and N from cutting this individual 
            ' Note that this calculation assumes equal distribution of plant biomass with height 
            BioCutLitAGC = PIP(ispp, iind, 1) * (1 - BioCutHeight / IndivHeight)    'g C
            BioCutLitSDC = PIP(ispp, iind, 6) * (1 - BioCutHeight / IndivHeight)    'g C
            If PIP(ispp, iind, 1) > 0 Then
                BioCutLitAGN = BioCutLitAGC * PIP(ispp, iind, 2) / PIP(ispp, iind, 1)   'g N
            Else
                BioCutLitAGN = 0
            End If

            If PIP(ispp, iind, 6) > 0 Then
                BioCutLitSDN = BioCutLitSDC * PIP(ispp, iind, 7) / PIP(ispp, iind, 6)   'g N
            Else
                BioCutLitSDN = 0
            End If
        Else  'Cutting height was higher than the individual; thus cut nothing from this indiv 
            BioCutLitAGC = 0
            BioCutLitSDC = 0
            BioCutLitAGN = 0
            BioCutLitSDN = 0
        End If

    End Sub
    Private Sub BGRhizRepro()
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  This Sub manages the testing to see if new belowground rhizome nodes can be created, 
        ' the first step in creating new plant individuals via clonal reproduction. 
        '  Tests are conducted to see if conditions are met to reproduce a new rhizome node
        ' for each node that has not reproduced yet (i.e. leading edges), but also, if the 
        ' user has turned clonal branching ON, then tests are repeated to see if conditions
        ' are met for new rhizome node creation as terminal branches and lateral branches.
        '
        '  Called by Main, not every day during the growing season but only on selected days. 
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim ispp As Short
        Dim iind, iindnew As Integer
        Dim RamAGC As Single          'C in aboveground plant part pool for an individual Ramet
        Dim RamAGN As Single          'N in aboveground plant part pool for an individual Ramet
        Dim RamBGC As Single          'C in belowground plant part pool for an individual Ramet
        Dim RamBGN As Single          'N in belowground plant part pool for an individual Ramet
        Dim CCostInternode As Single  'C cost in making new structural internode connection
        Dim CtranstoNew As Single     'C flux transfer to new individual being established, g C/day
        Dim NtranstoNew As Single     'N flux transfer to new individual being established, g N/day
        Dim openspot As Boolean       'True if there is room in storage arrays to create a new individual 

        'go through all species and individuals
        For ispp = 1 To 5
            If PlantType(ispp) = 1 Then
                ' Note this code block will only execute for spp included in this run, where iindmax(ispp) > 0
                For iind = 1 To iindmax(ispp)
                    RamAGC = PIP(ispp, iind, 1)                  'g C

                    'Test to see if there is an aboveground stem. 
                    If RamAGC > 0 Then
                        RamAGN = PIP(ispp, iind, 2)              'g N
                        'For belowground C use the active C subject to growth
                        RamBGC = PIP(ispp, iind, 3)              'g C
                        RamBGN = PIP(ispp, iind, 4)              'g N

                        'Test to see if there is enough C and N in the belowground part to create a daughter node.
                        'CtranstoNew and NtranstoNew are the C and N needed to establish the active part of a new
                        ' rhizome subject to future growth.  CCostInternode is the structural C cost of creating 
                        ' the internode, which is a function of the distance between parent and daughter, and which
                        ' will become tracked as part of the parent structural belowground C pool.
                        CtranstoNew = rhizAllocCProp(ispp) * maxPlantBGC(ispp)                         'g C 
                        NtranstoNew = rhizAllocCProp(ispp) * maxPlantBGC(ispp) / initBGCNrat(ispp)     'g N
                        'Internode structural cost is a per unit length calculation for length of internode distance
                        'Calculation is based on CIDistCostParm, spp-independent, as a proportion; multiplied by max BGC size 
                        CCostInternode = CIDistCostParm * maxPlantBGC(ispp) * cidist(ispp)             'g C

                        'To pass the tests, the current belowground part needs to have enough C and N to transfer to
                        ' the daughter and enough C to construct the internode, while retaining some C and N in
                        ' proportion to its species maximum size
                        If (RamBGC - CtranstoNew - CCostInternode > maxPlantBGC(ispp) * rhizRetCProp(ispp)) And
                         (RamBGN - NtranstoNew > (maxPlantBGC(ispp) / initBGCNrat(ispp)) * rhizRetCProp(ispp)) Then

                            'Test to see if this ramet has reproduced vegetatively yet. If not, that means it is a terminal node.   
                            If pgr(ispp, iind, 3) = 0 Then

                                '**************** Create new daughter rhizome node at existing terminal node ******************
                                '  All tests have been satisfied to create a new belowground individual, and we know that the present
                                'rhizome node is a terminal node.  
                                '  Call Sub to get the index identifier number for a new individual in this species   
                                iindnew = 0 : openspot = False
                                Call Findiind(ispp, openspot, iindnew)

                                ' Create new belowground individual rhizome node
                                If openspot = True Then Call BGRhizEstab(ispp, iind, iindnew, CtranstoNew, NtranstoNew, CCostInternode)

                                ' Increment the appropriate LFA counter
                                LFASpp(ispp, 9) += 1

                                '**************** Create new daughter rhizome node on a terminal branch ***********************
                                '  With a user-set probability, allow the rhizome node that was a terminal node, that just created
                                'a daughter, to immediately see if it can create a second daughter on a terminal branch.   
                                Randomize()
                                If ClonalBranching And (Rnd() < TbranchProb(ispp)) Then
                                    '  To create a terminal branch, this node has to pass the tests for C and N availability a second time.
                                    ' This requires recalculating RamBGC and RamBGN.  
                                    ' Note that CtranstoNew, NtranstoNew, and CCostInternode do not need to be recalculated. 
                                    RamBGC = PIP(ispp, iind, 3)              'g C
                                    RamBGN = PIP(ispp, iind, 4)              'g N
                                    If (RamBGC - CtranstoNew - CCostInternode > maxPlantBGC(ispp) * rhizRetCProp(ispp)) And
                                       (RamBGN - NtranstoNew > (maxPlantBGC(ispp) / initBGCNrat(ispp)) * rhizRetCProp(ispp)) Then

                                        '  Call Sub to get index identifier number for a new individual in this species   
                                        iindnew = 0 : openspot = False
                                        Call Findiind(ispp, openspot, iindnew)

                                        ' Create new belowground individual rhizome node on a new terminal branch
                                        If openspot = True Then Call BGRhizEstab(ispp, iind, iindnew, CtranstoNew, NtranstoNew, CCostInternode)

                                        ' Increment the appropriate LFA counter
                                        LFASpp(ispp, 10) += 1

                                    End If  'Test of whether the C and N tests were passed to establish a terminal branch 

                                End If  'Test of whether clonal branching is turned ON and probability allows terminal branching 

                            Else
                                '******************** Create new daughter rhizome node on a lateral branch **********************
                                '  If control gets to here, then the current node had already reproduced vegetatively prior to this 
                                ' subroutine call, so it must be a non-terminal node. 
                                '  With a user-set probability, allow this non-terminal node to see if it has enough C and N 
                                ' to create a new daughter rhizome node on a new lateral branch.
                                '  Note that this will allow multiple lateral branches from the same rhizome node, at different times
                                If ClonalBranching And (Rnd() < LbranchProb(ispp)) Then
                                    ' Note that CtranstoNew, NtranstoNew, and CCostInternode do not need to be recalculated. 
                                    RamBGC = PIP(ispp, iind, 3)              'g C
                                    RamBGN = PIP(ispp, iind, 4)              'g N
                                    If (RamBGC - CtranstoNew - CCostInternode > maxPlantBGC(ispp) * rhizRetCProp(ispp)) And
                                       (RamBGN - NtranstoNew > (maxPlantBGC(ispp) / initBGCNrat(ispp)) * rhizRetCProp(ispp)) Then

                                        '  Call Sub to get index identifier number for new individual in this species  
                                        iindnew = 0 : openspot = False
                                        Call Findiind(ispp, openspot, iindnew)

                                        ' Create new belowground individual rhizome node on a new lateral branch
                                        If openspot = True Then Call BGRhizEstab(ispp, iind, iindnew, CtranstoNew, NtranstoNew, CCostInternode)

                                        ' Increment the appropriate LFA counter
                                        LFASpp(ispp, 11) += 1

                                    End If  'Test of whether the C and N tests were passed 

                                End If  'Test of whether clonal branching is turned ON and probability allows lateral branching
                            End If  'end of test whether pgr(ispp, iind, 3) = 0
                        End If  'end of code testing whether CtranstoNew > 0 and NtranstoNew > 0 for attempt at making first terminal new rhizome node
                    End If  'end of test if RamAGC > 0

                Next  'iind loop

            Else  'Plant type is not = 1
                'If this is encountered, something is wrong:  code for other plant types not yet included
                ErrCount = ErrCount + 1
                ErrLast = 12
            End If

        Next  'ispp loop

    End Sub

    Private Sub BGRhizEstab(ByVal ispp As Short,
                            ByVal iind As Integer,
                            ByVal iindnew As Integer,
                            ByVal CtranstoNew As Single,
                            ByVal NtranstoNew As Single,
                            ByVal CCostInternode As Single)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        ' Here, a new daughter rhizome node is established / created.  
        ' 
        ' Called by:  BGRhizRepro, each time the tests are passed to create a new rhizome node. 
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim crAnglesign As Short        'positive or negative change in angle
        Dim crAnglechange As Single     'change in directional angle for new rhizome establishment
        Dim crAngle As Single           'angle from parent, for creating location for new daughter ramet 
        Dim crXdist, crYdist As Single  'x and y distance from parent of new daughter ramet
        Dim newXloc As Single, newYloc As Single          'new x and y locations for daughter ramet
        Dim Xovershoot As Single, Yovershoot As Single    'overshoot in x or y location if ramet tries to go out of space

        ' Transfer C and N from parent individual's belowground tissue
        PIP(ispp, iind, 3) = PIP(ispp, iind, 3) - CtranstoNew - CCostInternode
        PIP(ispp, iindnew, 3) = CtranstoNew
        PIP(ispp, iind, 4) = PIP(ispp, iind, 4) - NtranstoNew
        PIP(ispp, iindnew, 4) = NtranstoNew
        'Create internode structural C, tracked as a part of the parent structural C pool
        PIP(ispp, iind, 5) += CCostInternode
        'Test to ensure that belowground active C pool did not go below zero
        If PIP(ispp, iind, 3) < 0 Then
            ErrCount += 1
            ErrLast = 15
        End If

        '************** Spatial location for the new individual ***********************
        'Make it a certain distance from the parent; the distance is a spp-specific trait
        ' Calculate new angle based on existing angle and spp-specific max angle deviation
        ' First decide change in angle will be positive or negative
        crAnglesign = 1  '1 = positive
        Randomize()
        If Rnd() < 0.5 Then crAnglesign = -1
        'Calculate angle change and then new angle, based on old angle
        crAnglechange = crAnglesign * MaxVRangle(ispp) * Rnd()    'radians
        crAngle = pga(ispp, iind) + crAnglechange   'radians
        'Now store the angle information for the new individual 
        pga(ispp, iindnew) = crAngle                'radians
        'Calculate x and y components of distance from parent using angle
        crXdist = cidist(ispp) * Cos(crAngle)
        crYdist = cidist(ispp) * Sin(crAngle)
        'Calculate X and Y location of daughter. 
        newXloc = pil(ispp, iind, 1) + crXdist
        newYloc = pil(ispp, iind, 2) + crYdist


        'If either newXloc < 0, or Xovershoot (calculated below) > 0, then the new location tried to go out of the space. 
        ' (Same for Y dimension.)  If so, then 'wrap the map'; make it come in from the opposite side. 
        ' Also record the fact that this individual wrapped around the torus.  
        ' No need to change the angle that is heading within a genet -- this can stay the same.  
        pgr(ispp, iindnew, 2) = 0   'This location in this array stores info about whether this new indiv wrapped around the torus 
        Xovershoot = newXloc - GridixMax * s
        If newXloc < 0 Then
            newXloc = (GridixMax * s) + newXloc  'Note that newXloc is negative, so this keeps it in the space
            pgr(ispp, iindnew, 2) = 1
        ElseIf Xovershoot > 0 Then
            newXloc = Xovershoot
            pgr(ispp, iindnew, 2) = 1
        End If
        Yovershoot = newYloc - GridiyMax * s
        If newYloc < 0 Then
            newYloc = (GridiyMax * s) + newYloc  'Note that newYloc is negative, so this keeps it in the space
            pgr(ispp, iindnew, 2) = 1
        ElseIf Yovershoot > 0 Then
            newYloc = Yovershoot
            pgr(ispp, iindnew, 2) = 1
        End If

        'Establish new location
        pil(ispp, iindnew, 1) = newXloc
        pil(ispp, iindnew, 2) = newYloc

        '************ Assign clonal reproduction attributes for new individual ***********
        '  iind is the indiv index number of the rhizome node that is now reproducing 
        '  iindnew is the indiv index number of the new daughter rhizome node
        '  Keep same genet number as parent
        pgr(ispp, iindnew, 1) = pgr(ispp, iind, 1)   'number of the genet
        ' Now record that the parent genet has now reproduced vegetatively, but the daughter has not yet
        pgr(ispp, iindnew, 3) = 0   '0 = new ramet has not reproduced vegetatively yet
        pgr(ispp, iind, 3) = 1      '1 = parent ramet has now reproduced vegetatively
        ' Age information for the new ramet
        pgr(ispp, iind, 4) = 0      '0 = in its first year
        pgr(ispp, iind, 5) = 0      '0 = rhizome has not gone a growing season without making a stem, yet
        ' Transfer parent pedigree history information from reproducing rhizome node to new daughter
        pgr(ispp, iindnew, 6) = iind
        pgr(ispp, iindnew, 7) = pgr(ispp, iind, 6)
        pgr(ispp, iindnew, 8) = pgr(ispp, iind, 7)
        pgr(ispp, iindnew, 9) = pgr(ispp, iind, 8)
        pgr(ispp, iindnew, 10) = pgr(ispp, iind, 9)

    End Sub

    Private Sub Findiind(ByVal ispp As Short, ByRef openspot As Boolean, ByRef iindnew As Integer)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        ' Determine the unique, individual, identifer index number for a new individual
        '  to be created in this species.          
        ' This sub is called from BGRhizRepro and Sub DistrRandom; possibly multiple times for one genet:
        '  Each time a new individual rhizome node is to be created.   
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        '  iindmax(ispp) records the highest value of the index used, so far, for new individual               
        ' in all arrays that index individuals.  Species-specific.  iindmax can then be used
        ' as max value in loops through indivs for sums, counts, and sorting, to speed model runs.  
        '  iindmax(ispp) starts at zero for each run, and increments every
        ' time a new individual is created. 

        'Test whether the model has already maxed out the number of individuals allowed 
        If iindmax(ispp) = maxindivs Then
            openspot = False
            ErrCount = ErrCount + 1
            ErrLast = 13
            'If first time this has occurred, send a message to the user
            If firstErr13 = False Then
                MsgBox("The model tried to create a new individual rhizome for species " & ispp &
                       " but there was already " & maxindivs & " individuals; no space in storage arrays for another.")
                firstErr13 = True
            End If
        Else
            'The model has not maxed out the number of individuals.  
            ' Increment iindmax by 1 for the current species, to use as the identifier index for the
            ' new individual to be created, and also increment the value of iindmax counter by 1
            openspot = True
            iindnew = iindmax(ispp) + 1
            iindmax(ispp) += 1
        End If

    End Sub

    Private Sub Litterfall(ByVal iyr As Short, ByVal iday As Short)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  Here, above-ground litter inputs are calculated and added to above-ground litter 
        ' pools. Below-ground litter is not executed in this subroutine; it only occurs when
        ' a rhizome dies, in Sub BGMortality.  BGMortality is called from the present 
        ' subroutine if, after overwintering, a rhizome node has reached its max longevity. 
        '
        '  This sub is called from Main, once each day. 
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim CShootLit As Single    'Litter C flux from an individual plant (g C/day)
        Dim NShootLit As Single    'Litter N flux from an individual plant (g N/day)
        Dim ShootCNrat As Single   'C/N ratio in aboverground tissue of an individual plant
        Dim BioCutLitAGC As Single   'Flux of litter C from live indiv plant produced during mgmt Biocut 
        Dim BioCutLitSDC As Single   'Flux of litter C from standing dead indiv plant produced during mgmt Biocut 
        Dim BioCutLitAGN As Single   'Flux of litter N from live indiv plant produced during mgmt Biocut
        Dim BioCutLitSDN As Single   'Flux of litter N from standing dead indiv plant produced during mgmt Biocut
        Dim MaxCanopyHgt As Single   'Maximum height of current plant canopy in entire model (m) 
        Dim CVolBurn As Single       'C flux volatilized from individual burned during management burn 
        Dim NVolBurn As Single       'N flux volatilized from individual burned during management burn 
        Dim NMinBurn As Single       'N flux mineralized from individual burned during management burn 
        Dim Cscorched As Single      'C flux schorched from indiv during burn 
        Dim Nscorched As Single      'N flux schorched from indiv during burn 
        Dim PropCBurn As Single      'Proportion of living stem burned in fire management burn
        Dim PropCVol As Single       'Of living biomass burned, proportion of C that is volatilized
        Dim PropCScorch As Single    'Of living biomass burned, proportion of C that is not volatilized 
        Dim IndivHgt As Single       'Height of an individual plant aboveground
        Dim IndivStemC As Single     'C in living stem of an individual at the start of the timestep
        Dim IndivStemN As Single     'N in living stem of an individual at the start of the timestep 
        Dim ispp As Short          'index representing plant species
        Dim iind As Integer          'index representing plant individual within a species

        'Figure out the maximum height of all plants in the model space.  Need this for herbicide treatment.  
        If Hbcide Then
            MaxCanopyHgt = 0
            For ispp = 1 To 5
                For iind = 1 To iindmax(ispp)
                    If Phgt(ispp, iind) > MaxCanopyHgt Then
                        MaxCanopyHgt = Phgt(ispp, iind)
                    End If
                Next
            Next
        End If   'Hbcide 

        'Calculate burn height, if today is a day with a management treatment burn  
        ' Using average litter layer thickness over entire space, and user-defined proportion of litter severity, 
        ' calculate burn height as a height in the litter layer (i.e. proportion of aboveground downed litter layer). 
        ' Check that this is above the water depth, and correct if necessary if litter layer is partially inundated.  
        ' This burn height will be applied to downed litter, standing dead litter, and live aboveground biomass.  
        If BurnTr Then
            BurnTrHeight = AGLThicknessAvg * (1 - BurnTrLitSev)     ' meters
            'Check that this is above water level and correct it if necessary; taking MUCK thickness into account  
            If Waterlvl(iyr, iday) > (BurnTrHeight + MUCKThicknessAvg) Then
                BurnTrHeight = Waterlvl(iyr, iday) - MUCKThicknessAvg
            End If
        End If   'BurnTr

        'Loop through arrays for plant species and plant individuals
        For ispp = 1 To 5
            If PlantType(ispp) = 1 Then 'This is a wetland herbaceous clonal plant

                '******************* Move standing dead litter to ground litter in spring ********************************
                'During senescence in the fall, all living aboveground (stem, including foliar) tissue was 
                ' senesced and became standing dead litter.  
                'Now the standing dead litter falls to downed litter pool in grid cells over a 45-day period in the spring. 
                'Litter falls gradually, but at the end of that period, all remaining standing dead tissue enters litter. 
                ' C and nutrients in the rhizome part of each stem remain.  There is no root litter or litter from 
                ' the belowground rhizome at all until it dies.  
                If (iday >= 60) And (iday < GSstart) Then
                    ' Note this code block will only execute for spp included in this run, where iindmax(ispp) > 0
                    For iind = 1 To iindmax(ispp)
                        'Check to see if there is any aboveground standing dead litter C in this stem
                        If PIP(ispp, iind, 6) > 0 Then
                            'Calculate C:N ratio of aboveground dead litter; test to avoid division by zero
                            If PIP(ispp, iind, 7) > 0 Then
                                ShootCNrat = PIP(ispp, iind, 6) / PIP(ispp, iind, 7)
                            Else
                                ShootCNrat = 0
                                'Check to see plant N pool = zero.  If so, record error 7, because plant N
                                ' should not be zero if C is nonzero. 
                                'Check to see if plant N pool was < 0.  If so and it looks like a round-off error in  
                                ' N calcs, zero it out. Record error 6 if it was a relatively large error.   
                                If PIP(ispp, iind, 7) = 0 Then
                                    ErrCount += 1
                                    ErrLast = 7
                                ElseIf PIP(ispp, iind, 7) < 0 Then
                                    If PIP(ispp, iind, 7) < -0.00001 Then
                                        ErrCount += 1
                                        ErrLast = 6
                                    End If
                                    'Note 5/28/14:  this next line said k = 4, but we think it should be k = 7
                                    PIP(ispp, iind, 7) = 0
                                End If
                            End If  'test of whether plant internal aboveground N pool > 0, prior to calculating C:N  

                            'Calculate plant aboveground litter C and N fluxes for the individual plant.
                            'Daily rate is annual proportion divided by 45 for 45-day period
                            CShootLit = PIP(ispp, iind, 6) * PlantLitProp(ispp) / 45    'units g C/plant day
                            NShootLit = CShootLit / ShootCNrat                          'units g N/plant day
                            'Test if there would be too little left in the standing dead litter.  If so, make it all go to downed litter.
                            'This helps in counting number of living plant stems later
                            'This also helps with mass balance checks
                            'In addition, if it is the last day of the litterfall period, all aboveground C and nutrients enter litter flux
                            'and aboveground standing dead litter C and nutrients gets zeroed out
                            If (PIP(ispp, iind, 6) - CShootLit < 0.1) Or iday = (GSstart - 1) Then
                                CShootLit = PIP(ispp, iind, 6)
                                NShootLit = PIP(ispp, iind, 7)
                            End If
                            'Subtract plant aboveground litterfall C and N fluxes from plant aboveground standing dead tissue pools
                            PIP(ispp, iind, 6) = PIP(ispp, iind, 6) - CShootLit
                            PIP(ispp, iind, 7) = PIP(ispp, iind, 7) - NShootLit

                            Call LitterfallSpat(CShootLit, NShootLit, ispp, iind)

                            'Increment the whole-area sums of C and N in plant litterfall flux, convert to per m2
                            CAGltrfallDay = CAGltrfallDay + CShootLit / GridArea               'g C/m2 d
                            NAGltrfallDay = NAGltrfallDay + NShootLit / GridArea               'g N/m2 d

                            'Re-calculate plant height after loss of biomass through litterfall 
                            Call AGHeightCalc(ispp, iind)

                        End If  'End of the test of whether there is a nonzero standing dead C in this indiv location

                    Next  'End of loop for plant individuals

                ElseIf iday = GSstart Then
                    '*********************** Belowground rhizomes die based on max longevity ***************************************
                    'Once per year after aboveground litterfall in spring and before plants begin to grow at start of growing season,
                    'belowground plant parts die if they have passed max longevity 
                    For iind = 1 To iindmax(ispp)
                        'Test to see if max longevity has been reached
                        If pgr(ispp, iind, 4) = maxLongev(ispp) Then BGMortality(ispp, iind)
                    Next

                End If 'Test of litterfall time period prior to growing season

                '************************* Litter produced by management mowing for invasives ****************************************
                'Now calc and distribute litter produced in invasives management activity: BioCut (e.g. mowing) 
                If BioCut Then
                    For iind = 1 To iindmax(ispp)
                        If PIP(ispp, iind, 1) + PIP(ispp, iind, 6) > 0 Then  'Test that there is positive aboveground C for this indiv
                            'Calculate how much litter is produced from this individual due to BioCut treatment
                            Call MgmtBioCut(ispp, iind, BioCutLitAGC, BioCutLitSDC, BioCutLitAGN, BioCutLitSDN)

                            If BioCutRemoval = 0 Then
                                'Biocut live biomass is not removed from site; thus, distribute spatially to grid cell litter pools 
                                Call LitterfallSpat(BioCutLitAGC, BioCutLitAGN, ispp, iind)
                            Else
                                'Biocut live biomass is removed from site. No need to add it anywhere in the model. 
                                'Account for this removal from the system in the mass balance check.  
                                EcosCinout += -BioCutLitAGC
                                EcosNinout += -BioCutLitAGN
                            End If
                            'Update values of ramet C and N for this individual 
                            PIP(ispp, iind, 1) = PIP(ispp, iind, 1) - BioCutLitAGC
                            PIP(ispp, iind, 2) = PIP(ispp, iind, 2) - BioCutLitAGN

                            If BioCutRemoval = 0 Then
                                'Biocut standing dead biomass is not removed from site; thus, distribute spatially to grid cell litter pools 
                                Call LitterfallSpat(BioCutLitSDC, BioCutLitSDN, ispp, iind)
                            Else
                                'Biocut standing dead biomass is removed from site. No need to add it anywhere in the model. 
                                'Account for this removal from the system in the mass balance check.  
                                EcosCinout += -BioCutLitSDC
                                EcosNinout += -BioCutLitSDN
                            End If
                            'Update values of ramet C and N for this individual 
                            PIP(ispp, iind, 6) = PIP(ispp, iind, 6) - BioCutLitSDC
                            PIP(ispp, iind, 7) = PIP(ispp, iind, 7) - BioCutLitSDN

                            'Increment the whole-area sums of C and N in plant litterfall flux, convert to per m2
                            ' But only count it as litterfall fluxes if biocut biomass was not removed from site 
                            If BioCutRemoval = 0 Then
                                CAGltrfallDay = CAGltrfallDay + BioCutLitAGC / GridArea + BioCutLitSDC / GridArea               'g C/m2 d
                                NAGltrfallDay = NAGltrfallDay + BioCutLitAGN / GridArea + BioCutLitSDN / GridArea               'g N/m2 d
                            End If

                            'Calculate revised height for this individual post-biocut 
                            Call AGHeightCalc(ispp, iind)

                        End If  'test if there is positive aboveground C for this individual
                    Next 'individual within the species 
                End If  'test to see if there is a BioCut on current day 

                '*********************** Litter produced by management herbicide for invasives  ******************************
                ' Herbicide penetrates to a certain depth in the canopy and kills all individuals that it contacts. 
                ' At least 10% of an individual's height must be exposed to the herbicide for the indiv to die.  
                If Hbcide Then
                    For iind = 1 To iindmax(ispp)
                        If Phgt(ispp, iind) > (MaxCanopyHgt - HbcidePen) / 0.9 Then
                            'If control gets to this point, we are killing this individual. Its above and belowground parts die.
                            'Move all C and N from living aboveground pools into standing dead aboveground pools for this indiv. 
                            'Thus this individual can no longer grow.  But its standing dead tissue will continue to shade other 
                            ' individuals in light competition subroutine. 
                            PIP(ispp, iind, 6) += PIP(ispp, iind, 1)
                            PIP(ispp, iind, 1) = 0
                            PIP(ispp, iind, 7) += PIP(ispp, iind, 2)
                            PIP(ispp, iind, 2) = 0
                            'Kill the belowground rhizome of this individual also and zero out all arrays that store info
                            ' for this individual. 
                            Call BGMortality(ispp, iind)
                        End If
                    Next
                End If

                '************************ Litter produced by mangement burn for invasives **********************************
                ' No individuals are killed directly. Tissue above the 'burn height' is volatized or scorched.  Scorched 
                ' tissue moves to downed litter following the burn.  Living tissue below the 'burn height' remains alive
                ' and can continue to grow.  
                If BurnTr Then   'If today is a day of a burn treatment 
                    For iind = 1 To iindmax(ispp)
                        'Calculate proportion of living aboveground indiv C that is burned 
                        IndivHgt = Phgt(ispp, iind)
                        IndivStemC = PIP(ispp, iind, 1)
                        IndivStemN = PIP(ispp, iind, 2)

                        ' ************* Here we burn living aboveground biomass ***************
                        'Only do burning calcs for this indiv if it has nonzero stem C
                        If IndivStemC > 0 Then
                            If IndivHgt > BurnTrHeight Then
                                PropCBurn = (IndivHgt - BurnTrHeight) / IndivHgt
                            Else
                                PropCBurn = 0
                            End If
                            'Double-check that this proportion stays between zero and 1 
                            If PropCBurn > 1 Then PropCBurn = 1 : If PropCBurn < 0 Then PropCBurn = 0

                            'Of the living aboveground indiv C that is burned, calculate proportions volatilized and scorched. 
                            ' Check to make sure the user did not enter a burn efficiency that is > 1.  
                            If BurnTrLiveEff <= 1 And BurnTrLiveEff >= 0 Then
                                PropCVol = BurnTrLiveEff * PropCBurn
                                PropCScorch = (1 - BurnTrLiveEff) * PropCBurn
                            Else
                                ErrCount += 1
                                ErrLast = 38
                                PropCVol = 1
                                PropCScorch = 0
                            End If

                            'Now from these proportions volatilized and scorched, calculate the C fluxes 
                            CVolBurn = PropCVol * IndivStemC         'g C / indiv 
                            Cscorched = PropCScorch * IndivStemC     'g C / indiv 

                            'From C fluxes and C/N ratio in stem tissue, calculate N fluxes 
                            NVolBurn = BurnTrNVol * CVolBurn * IndivStemN / IndivStemC              'g N / indiv
                            NMinBurn = (1 - BurnTrNVol) * CVolBurn * IndivStemN / IndivStemC        'g N / indiv 
                            Try
                                Nscorched = Cscorched * IndivStemN / IndivStemC                         'g N / indiv 
                            Catch ex As Exception  'catch error if tried to divide by zero
                                ErrCount += 1
                                ErrLast = 45
                            End Try

                            'Distribute the C and N scorched to downed litter pools post-burn. 
                            If Cscorched > 0 Then
                                If Nscorched > 0 Then
                                    Call LitterfallSpat(Cscorched, Nscorched, ispp, iind)
                                Else
                                    ErrCount += 1
                                    ErrLast = 44
                                End If
                            End If

                            'Update plant internal C and N pools 
                            PIP(ispp, iind, 1) = PIP(ispp, iind, 1) - CVolBurn - Cscorched               'g C / indiv 
                            PIP(ispp, iind, 2) = PIP(ispp, iind, 2) - NVolBurn - NMinBurn - Nscorched    'g N / indiv 

                            'Increment the whole-area sums of C and N in plant litterfall flux, convert to per m2
                            CAGltrfallDay = CAGltrfallDay + Cscorched / GridArea               'g C/m2 d
                            NAGltrfallDay = NAGltrfallDay + Nscorched / GridArea               'g N/m2 d

                        ElseIf PIP(ispp, iind, 6) > 0 Then
                            '************ Here we burn standing dead litter ************************** 
                            'If control gets to here, there is no living aboveground stem, 
                            ' but there is standing dead litter at this indiv location.  Height is saved in same array.  
                            If IndivHgt > BurnTrHeight Then
                                PropCBurn = (IndivHgt - BurnTrHeight) / IndivHgt
                            Else
                                PropCBurn = 0
                            End If
                            'Double-check that this proportion stays between zero and 1 
                            If PropCBurn > 1 Then PropCBurn = 1 : If PropCBurn < 0 Then PropCBurn = 0

                            'For standing dead litter, all C burned is volatilized: nothing is scorched 
                            PropCVol = PropCBurn
                            CVolBurn = PropCVol * PIP(ispp, iind, 6)         'g C / indiv 

                            'From C fluxes and C/N ratio in standing dead litter, calculate N fluxes 
                            NVolBurn = BurnTrNVol * CVolBurn * PIP(ispp, iind, 7) / PIP(ispp, iind, 6)            'g N / indiv
                            NMinBurn = (1 - BurnTrNVol) * CVolBurn * PIP(ispp, iind, 7) / PIP(ispp, iind, 6)      'g N / indiv 

                            'Update plant internal C and N pools 
                            PIP(ispp, iind, 6) = PIP(ispp, iind, 6) - CVolBurn                'g C / indiv 
                            PIP(ispp, iind, 7) = PIP(ispp, iind, 7) - NVolBurn - NMinBurn     'g N / indiv 
                        Else
                            NMinBurn = 0 : CVolBurn = 0 : NVolBurn = 0

                        End If   'Test if IndivStemC > 0 or standing dead tissue C > 0

                        '****** Bookkeeping that applies to burn of either living or dead aboveground tissue 
                        'add mineralized N to regional N pool 
                        RNP(1) = RNP(1) + NMinBurn                                         'g N 
                        LitNminGCPDay = LitNminGCPDay + NMinBurn / GridArea                'g N / m2 day 

                        'Update Ecosystem C and N losses through volatilization
                        EcosCinout = EcosCinout - CVolBurn
                        EcosNinout = EcosNinout - NVolBurn
                    Next  'Next individual within the species 
                End If  'Test if BurnTr = true 

            Else    'PlantType(ispp) not = 1
                'This is the place to add litterfall calculations for other plant types
                'If other plant types need to be included, should probably move litterfall calcs for each type
                'into separate subs and call them from here to maintain code/flow clarity
                'For now, just record an error if PlantType(ispp) not 1
                ErrCount = ErrCount + 1
                ErrLast = 8
            End If  'Test of PlantType(ispp) = 1

        Next   'ispp: loop back for next species

    End Sub
    Private Sub LitterfallSpat(ByVal CShootLit As Single, ByVal NShootLit As Single, ByVal ispp As Integer, ByVal iind As Integer)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
        ' Calculates the spatial distribution of litter as it falls from plant pools to 
        'litter pools. 
        '
        ' Called by Litterfall.  Called during normal litter senescence, and also called
        'for management for invasives (e.g. mowing, burning).  
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim ixGridLoc As Short     'integer representing grid cell x location of individual plant
        Dim iyGridLoc As Short     'integer representing grid cell y location of individual plant
        Dim ix, iy, newx, newy As Short    'integer variables for index substitution in litterfall loop

        'Model Structure Option:  Select user choice of spatial distribution of aboveground litterfall
        Select Case LitSpatDistr
            Case 1
                'In this option, litter is distributed mean-field, spread around all grid cells equally
                For ixGridLoc = 1 To GridixMax
                    For iyGridLoc = 1 To GridiyMax
                        'add litter C and N to grid cell
                        GCP(ixGridLoc, iyGridLoc, 1) += CShootLit / ncells       'g C
                        GCP(ixGridLoc, iyGridLoc, 2) += NShootLit / ncells       'g C
                        'keep track of info needed to calculate weighted k value of current year litter   
                        gckcalcs(ixGridLoc, iyGridLoc, 1) += (CShootLit / ncells)                'g C
                        gckcalcs(ixGridLoc, iyGridLoc, 2) += (CShootLit / ncells) * kLit(ispp)   'g C * k
                    Next
                Next

            Case 2
                'In this option, litter is spatial, spread over 9 cells centered on plant individual

                'Figure out which grid cell the individual plant is in
                'First grid cell is numbered (1,1); see notes
                ixGridLoc = Int(pil(ispp, iind, 1) / s) + 1
                iyGridLoc = Int(pil(ispp, iind, 2) / s) + 1

                'Adding plant litter to litter C and N pools in grid cells
                'Put half of the plant litter C and N in the grid cell the plant individual is in, and
                'Spread the remaining half of the plant litter C and N among the 8 adjacent grid cells

                'First, put 7/16 of the litter in the plant's own cell
                GCP(ixGridLoc, iyGridLoc, 1) = GCP(ixGridLoc, iyGridLoc, 1) + CShootLit * 7 / 16   'units g C
                GCP(ixGridLoc, iyGridLoc, 2) = GCP(ixGridLoc, iyGridLoc, 2) + NShootLit * 7 / 16 'units g N

                'keep track of info needed to calculate weighted k value of current year litter   
                gckcalcs(ixGridLoc, iyGridLoc, 1) += (CShootLit * 7 / 16)               'g C
                gckcalcs(ixGridLoc, iyGridLoc, 2) += (CShootLit * 7 / 16) * kLit(ispp)  'g C * k

                'Next, use a loop to put 1/16 of the litter per cell in the set of 9 cells containing
                '  and surrounding the plant's own cell (7/16 + 1/16 will equal 1/2 in the plant's own cell)
                For ix = (ixGridLoc - 1) To (ixGridLoc + 1)
                    For iy = (iyGridLoc - 1) To (iyGridLoc + 1)

                        'Use an index variable substitution to 'wrap the map'
                        ' if litter would leave the grid cell space, wrap it to the opposite side of the space
                        newx = ix : newy = iy
                        If newx < 1 Then newx = GridixMax
                        If newx > GridixMax Then newx = 1
                        If newy < 1 Then newy = GridiyMax
                        If newy > GridiyMax Then newy = 1

                        GCP(newx, newy, 1) = GCP(newx, newy, 1) + CShootLit / 16  'units g C
                        GCP(newx, newy, 2) = GCP(newx, newy, 2) + NShootLit / 16  'units g N

                        'keep track of info needed to calculate weighted k value of current year litter   
                        gckcalcs(ixGridLoc, iyGridLoc, 1) += (CShootLit / 16)                'g C
                        gckcalcs(ixGridLoc, iyGridLoc, 2) += (CShootLit / 16) * kLit(ispp)   'g C * k

                    Next
                Next
                'End of the loops distributing 1/16 of the plant's litter among the 9 cells containing and surrounding the plant

        End Select   'End of the model structure option, choice of spatial distr of litter

    End Sub

    Private Sub Senescence()
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  Called once per year, after the end of the growing season, from Main.
        '  Here, foliar senescence occurs.  N is resorbed into perennial tissue.  
        '  Different processes will need to be used for different types of plants. 
        '  For plants with annual aboveground tissue and longer-lived belowground tissue,
        ' (including clonal marsh plants), a proportion of N from aboveground tissue is resorbed
        ' into belowground tissue during aboveground senescence, each year. 
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim ispp, iind As Integer
        Dim Nresorb As Single    'N resorption flux for an individual stem, g N/day

        For ispp = 1 To 5
            'PlantType 1 is clonal marsh plants, annual aboveground ramets
            If PlantType(ispp) = 1 Then
                ' Note this code block will only execute for spp included in this run, where iindmax(ispp) > 0
                For iind = 1 To iindmax(ispp)
                    'Test to see if there is nonzero living aboveground biomass
                    If PIP(ispp, iind, 1) > 0 Then
                        'Transfer a proportion of aboveground tissue N to belowground tissue 
                        Nresorb = resorbProp(ispp) * PIP(ispp, iind, 2)
                        PIP(ispp, iind, 2) = PIP(ispp, iind, 2) - Nresorb           'g N
                        PIP(ispp, iind, 4) = PIP(ispp, iind, 4) + Nresorb           'g N
                        'Move aboveground C and remaining aboveground N from living pools to standing dead pools 
                        PIP(ispp, iind, 6) += PIP(ispp, iind, 1)                    'g C
                        PIP(ispp, iind, 1) = 0                                      'g C
                        PIP(ispp, iind, 7) += PIP(ispp, iind, 2)                    'g N
                        PIP(ispp, iind, 2) = 0                                      'g N
                    End If
                Next
            Else
                'Record an error:  there is no code yet for additional plant types
                ErrCount = ErrCount + 1
                ErrLast = 10
            End If
        Next

    End Sub

    Private Sub VertDimCalcs(ByVal ix As Short, ByVal iy As Short)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        ' Called once per day, for each grid cell, from Sub DetritalDynCell. 
        ' Calculates vertical dimensions, i.e. top and bottom vertical positions of each 
        '  pool of litter and detritus within the grid cell. 
        ' "Zero" in the vertical dimension is defined as top of MSOM pool.  Units are meters.  
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        ' For each litter and detritus pool, subscript 1 = top of pool in meters, 
        ' 2 = bottom of pool in meters.  
        'The MSOM pool is defined as having its top edge at "zero" vertical dimension.
        MSOMvertdim(1) = 0
        'MSOM bottom edge vertical dimension is negative, fixed at 0.3 meters 
        MSOMvertdim(2) = MSOMdepth

        'The MUCK pool has its bottom edge at zero, and the pool is all above zero. 
        'Convert grid cell C to gC/m2, divide by bulk density and c conc, and convert units to m 
        MUCKvertdim(1) = GCP(ix, iy, 9) / ((s ^ 2) * bdMUCK * cconcMUCK * 10 ^ 6)
        MUCKvertdim(2) = 0

        'The AGL pool is all above zero, and all above the MUCK pool.  Its bottom edge sits at the 
        ' top edge of the MUCK pool. 
        AGLvertdim(2) = MUCKvertdim(1)
        AGLvertdim(1) = AGLvertdim(2) + GCP(ix, iy, 1) / ((s ^ 2) * bdAGL * cconcTissue * 10 ^ 6)

        'The BGL pool is more complicated.  Its top edge sits at the same height as the top of the MUCK pool.  
        ' Its depth is a dynamic variable determined by its biomass and bulk density.    
        ' If the MUCK is deeper than the BGL depth, none of the BGL pool will extend down into the MSOM. 
        ' If the MUCK is less deep than the BGL depth, a portion of the BGL pool will extend down into the MSOM. 
        ' The assumption for now is that belowground litter is spread over a depth that is a multiple of what
        '  it would be as pure litter; this multiple is the parameter BGLspread.
        BGLvertdim(1) = MUCKvertdim(1)
        BGLvertdim(2) = BGLvertdim(1) - GCP(ix, iy, 7) * BGLspread / ((s ^ 2) * bdBGL * cconcTissue * 10 ^ 6)
        'Bottom of BGL pool is not allowed to go below the bottom of the MSOM pool, currently -0.3 meters
        If BGLvertdim(2) < MSOMdepth Then
            BGLvertdim(2) = MSOMdepth
        End If

        'Identify the vertical top and bottom of the available NH4 pool, for use in calculating nitrification
        'under only aerobic conditions. 
        '  The top of the available NH4 pool is 0.05 m up into the AGL litter layer, or the top of the
        ' AGL litter layer, whichever is less
        If AGLvertdim(1) - AGLvertdim(2) > 0.05 Then
            NH4vertdim(1) = AGLvertdim(2) + 0.05
        Else
            NH4vertdim(1) = AGLvertdim(1)
        End If
        'The bottom of the available NH4 pool is down in the MSOM, the active zone depth 
        NH4vertdim(2) = AZdepth

        'Identify the vertical top and bottom of the available NO3 pool, for use in calculating denitrification
        'under only anaerobic conditions. 
        '  The top of the available NO3 pool is the top of the MUCK. 
        NO3vertdim(1) = MUCKvertdim(1)
        '  The bottome of the available NO3 pool is down in the MSOM, the active zone depth 
        NO3vertdim(2) = AZdepth

    End Sub

    Private Sub DetritalDyn(ByVal iyr As Short, ByVal iday As Short)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  Called once per day, from Sub Main.
        '  Sets up the loop through each grid cell, calls DetritalDynCell to do detritus dynamics
        ' within each grid cell
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim ix As Short              'integer x index of grid cell in grid
        Dim iy As Short              'integer y index of grid cell in grid
        Dim PropLitterBurn As Single   'Proportion of litter layer that burns within a single cell 
        Dim AGLThicknessCell As Single 'Thickness of downed litter layer in a single grid cell 
        Dim CVolBurn As Single       'C flux volatilized from litter burned during management burn 
        Dim NVolBurn As Single       'N flux volatilized from litter burned during management burn 
        Dim NMinBurn As Single       'N flux mineralized from litter burned during management burn
        Dim Wltoday, Wl1dago, Wl2dago, Wl3dago, Wl4dago As Single  'Water level (m) today, 1 day ago, etc. 

        'Call sub to calculate loss of detritus via export
        DetritalExport()

        '**************** Save water levels for the past 5 days ***********************************
        'Based on iday, save values for water level today, 1 day ago, 2 days ago, etc.
        ' If it is in the first 4 days of the year, then we have to use days from end of previous year
        If (iyr = 1) And (iday < 5) Then
            'The first few days of year 1, just use water level = 0
            Wltoday = 0 : Wl1dago = 0 : Wl2dago = 0 : Wl3dago = 0 : Wl4dago = 0
        Else
            If iday = 1 Then
                Wltoday = Waterlvl(iyr, 1)
                Wl1dago = Waterlvl((iyr - 1), 365)
                Wl2dago = Waterlvl((iyr - 1), 364)
                Wl3dago = Waterlvl((iyr - 1), 363)
                Wl4dago = Waterlvl((iyr - 1), 362)
            ElseIf iday = 2 Then
                Wltoday = Waterlvl(iyr, 2)
                Wl1dago = Waterlvl(iyr, 1)
                Wl2dago = Waterlvl((iyr - 1), 365)
                Wl3dago = Waterlvl((iyr - 1), 364)
                Wl4dago = Waterlvl((iyr - 1), 363)
            ElseIf iday = 3 Then
                Wltoday = Waterlvl(iyr, 3)
                Wl1dago = Waterlvl(iyr, 2)
                Wl2dago = Waterlvl(iyr, 1)
                Wl3dago = Waterlvl((iyr - 1), 365)
                Wl4dago = Waterlvl((iyr - 1), 364)
            ElseIf iday = 4 Then
                Wltoday = Waterlvl(iyr, 4)
                Wl1dago = Waterlvl(iyr, 3)
                Wl2dago = Waterlvl(iyr, 2)
                Wl3dago = Waterlvl(iyr, 1)
                Wl4dago = Waterlvl((iyr - 1), 365)
            Else 'iday = 5 or more
                Wltoday = Waterlvl(iyr, iday)
                Wl1dago = Waterlvl(iyr, (iday - 1))
                Wl2dago = Waterlvl(iyr, (iday - 2))
                Wl3dago = Waterlvl(iyr, (iday - 3))
                Wl4dago = Waterlvl(iyr, (iday - 4))
            End If
        End If

        '**************** Decide which value of N2O proportion to use for denitrification ******************************
        'Denitrification will be divided into an N2O proportion and an N2 proportion.  
        ' There are three values for N2O proportion, entered by the user in the parameter input file.
        ' Here, choose one of the three to use throughout all the grid cells.  The choice is based on 
        ' whether the benchmark depth within the active became newly inundated today, or 2 to 4 days ago.
        'If none of these are true, then the long-term value of N2O proportion will be used.  Note that this is 
        ' correct both when the benchmark value was either inundated the last 5 days, or non-inundated the last 5 days. 

        'First, assume the long-term value of N2O proportion is the one to use 
        DntrN2OProp = N2Oplt
        'If the water level was above the benchmark active zone depth today and below it 1 day ago,  
        ' then use the appropriate N2O proportion for the first day of inundation. 
        'If the water level was above the benchmark active zone depth today, also above it 1 day ago, 
        ' but below it either 3, 4, or 5 days ago, then use the N2O proportion for the first 4 days of inundation.
        If Wltoday >= N2Opdepth Then
            If Wl1dago < N2Opdepth Then
                DntrN2OProp = N2Op1d  'one day of inundation
            ElseIf (Wl2dago < N2Opdepth) Or (Wl3dago < N2Opdepth) Or (Wl4dago < N2Opdepth) Then
                DntrN2OProp = N2Op4d  '2 to 4 days of inundation 
            End If
        End If

        '**************** Carry out decomposition and nutrient dynamics in each cell ************************
        For ix = 1 To GridixMax
            For iy = 1 To GridiyMax

                Call DetritalDynCell(ix, iy, iday)

            Next
        Next

        '**************** Burn treatments for invasive species management 

        'Calculate average litter layer and muck thickness over the entire space
        ' (needed for burn management treatment) 
        AGLThicknessAvg = AGLThicknessSum / ncells     'meters
        MUCKThicknessAvg = MUCKThicknessSum / ncells   'meters 

        'If there is a management burn treatment on this day, burn part of the litter layer 
        ' based on burn severity, controlled by a user parameter 
        If BurnTr Then

            'Burn litter layer within each grid cell, always down to the burn height 
            For ix = 1 To GridixMax
                For iy = 1 To GridiyMax
                    'Need to know thickness of the litter layer in this indiv grid cell  
                    Call VertDimCalcs(ix, iy)
                    AGLThicknessCell = AGLvertdim(1) - AGLvertdim(2)
                    'Calculate proportion of litter layer to burn in this indiv grid cell.
                    'Fire starts at the top of the litter layer and burns down to the burn height.
                    'But if burn height is higher than the top of the litter layer (e.g. because of higher
                    ' water level), then proportion of litter layer burned is zero.  
                    If BurnTrHeight < AGLThicknessCell And AGLThicknessCell > 0 Then
                        PropLitterBurn = (AGLThicknessCell - BurnTrHeight) / AGLThicknessCell
                    Else
                        PropLitterBurn = 0
                    End If
                    If PropLitterBurn > 1 Then PropLitterBurn = 1

                    'C volatilized during burning of litter in this grid cell 
                    CVolBurn = PropLitterBurn * GCP(ix, iy, 1)                                   'g C

                    'From C fluxes and C/N ratio in standing dead litter, calculate N fluxes 
                    If GCP(ix, iy, 1) > 0 Then
                        NVolBurn = BurnTrNVol * CVolBurn * GCP(ix, iy, 2) / GCP(ix, iy, 1)            'g N 
                        NMinBurn = (1 - BurnTrNVol) * CVolBurn * GCP(ix, iy, 2) / GCP(ix, iy, 1)      'g N 
                    Else
                        NVolBurn = 0
                        NMinBurn = 0
                    End If

                    'add mineralized N to regional N pool
                    RNP(1) = RNP(1) + NMinBurn                                'g N 

                    'Update grid cell C and N pools 
                    GCP(ix, iy, 1) = GCP(ix, iy, 1) - CVolBurn                'g C  
                    GCP(ix, iy, 2) = GCP(ix, iy, 2) - NVolBurn - NMinBurn     'g N 

                    'Check for a round-off error in GCP inorganic N pool 
                    If GCP(ix, iy, 2) < 0 Then
                        If GCP(ix, iy, 2) > -0.000001 Then
                            GCP(ix, iy, 2) = 0
                        Else
                            ErrCount += 1
                            ErrLast = 40
                        End If
                    End If

                    'Update bookkeeping fluxes of N mineralization
                    LitNminGCPDay = LitNminGCPDay + (NMinBurn / GridArea)       'g N / m2 day

                    'Update Ecosystem C and N losses through volatilization
                    EcosCinout = EcosCinout - CVolBurn
                    EcosNinout = EcosNinout - NVolBurn
                Next
            Next

        End If

        '********************* Bookkeeping code ***********************************************

        'Update annual N fluxes 
        LitNminYr = LitNminYr + LitNminGCPDay                                         'g N/m2 y
        MSOMNminYr = MSOMNminYr + MSOMNminGCPDay                                      'g N/m2 y
        MUCKNminYr = MUCKNminYr + MUCKNminGCPDay                                      'g N/m2 y
        NitrifYr = NitrifYr + NitrifDay                                               'g N/m2 y
        N2DntrYr = N2DntrYr + N2DntrDay                                               'g N/m2 y
        N2ODntrYr = N2ODntrYr + N2ODntrDay                                            'g N/m2 y
        'Update annual C fluxes
        CtoMSOMYr = CtoMSOMYr + CtoMSOMDay                                            'g C/m2 y
        CtoMUCKYr = CtoMUCKYr + CtoMUCKDay                                            'g C/m2 y
        CO2CRespYr = CO2CRespYr + CO2CRespDay                                         'g C/m2 y
        CH4CRespYr = CH4CRespYr + CH4CRespDay                                         'g C/m2 y

        'EcosNinout is for mass-balance check
        EcosNinout = EcosNinout - (N2DntrDay + N2ODntrDay) * GridArea                                 'g N
        'EcosCinout is for mass-balance check
        EcosCinout = EcosCinout - (CO2CRespDay + CH4CRespDay) * GridArea                              'g C

    End Sub
    Private Sub DetritalDynCell(ByVal ix As Short, ByVal iy As Short, ByVal iday As Short)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  Called once for each grid cell, from Sub DetritalDyn.
        '  There are three output fluxes from Detritus: 
        '   Export of detritus out of the system;
        '   Turnover and release of nutrients from detritus to local depletion zone, and to regional nutrient pool
        '  Those fluxes are calculated and executed here. 
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim DelGCPMSOMC, DelGCPMUCKC As Single    'Delta (change in) MSOM-C pool and MUCK-C pool in GCP grid cell due to decomposition
        Dim DelGCPAGltr, DelGCPBGltr As Single    'Delta (change in) litter C pools in GCP grid cell due to decomposition
        Dim AerDelGCPAGltr, AerDelGCPBGltr As Single 'Delta (change in) litter C pools in GCP grid cell due to decomp under aerobic conditions
        Dim GCPhumCNrat As Single  'C/N ratio of MSOM in GPC grid cell  
        Dim NGCPMSOMrelease, NGCPMUCKrelease As Single  'in grid cell, N release from MSOM organic N & MUCK organic N to inorg N due to net N min
        Dim NGCPAGlitRelease, NGCPBGlitRelease As Single  'in grid cell, N release from litter pool organic N to inorganic N due to net N min
        Dim ExtraNAG, ExtraNBG As Single       'temporary variables for calculating N release
        Dim CBGLtoMSOM, CBGLtoMSOMAn As Single    'C flux transfers from BG litter pool to MSOM
        Dim CAGLtoMUCK, CBGLtoMUCK As Single    'C flux transfers from litter pools to MUCK due to decomposition
        Dim CAGLtoMUCKAn, CBGLtoMUCKAn As Single    'C flux transfers from litter pools to MUCK under anaerobic conditions
        Dim BGLtoMUCKProp As Single  'Proportion of aerboic BG litter decomp that goes to MUCK vs MSOM 
        Dim NBGLtoMSOM, NBGLtoMSOMAn As Single    'N flux transfers from litter pools to MSOM
        Dim NAGLtoMUCK, NBGLtoMUCK As Single    'N flux transfers from litter pools to MUCK under aerobic conditions
        Dim NAGLtoMUCKAn, NBGLtoMUCKAn As Single    'N flux transfers from litter pools to MUCK under anaerobic conditions 
        Dim CBioturb, NBioturb As Single   'C and N fluxes transferred from MUCK to MSOM, representing bioturbation and particulate matter
        Dim TdayAGLcell, TdayBGLcell As Single    'Daily average temperature of AGL and BGL in this cell 
        Dim TdayMUCKcell, TdayMSOMcell As Single  'Daily average temperature of MUCK and MSOM in this cell 
        Dim TfactAGL, TfactBGL, TfactMUCK, TfactMSOM As Single  'Factors for temperature effect on decomp rates in detrital pools in this grid cell
        Dim PropAGLAn, PropBGLAn, PropMUCKAn, PropMSOMAn As Single   'Proportions of detrital pools that are anaerobic, based on water level and lag time
        Dim AGLSubmProp, BGLSubmProp, MUCKSubmProp, MSOMSubmProp As Single  'Proportions of detrital pools that are submerged in this cell on this day 
        Dim NitrifDayCell, NDntrDayCell, N2DntrDayCell, N2ODntrDayCell As Single  'Nitrification and denitrification in the grid cell 
        Dim PropNH4Aer As Single                 'Proportion of available NH4-N pool that is aerobic (0 to 1)
        Dim PropNO3Anaer As Single               'Proportion of available NO3-N pool that is anaerobic (0 to 1)
        Dim HetRespFact As Single                'Factor to include the effect of labile C availability on denitrification rate
        Dim CHetRespCell As Single               'C lost via heterotrophic respiration in this cell on this day (includes CO2-C and CH4-C)  
        Dim CH4CRespProp As Single               'Proportion of heterotrophic C respiration that is CH4-C; remainder is CO2-C  
        Dim fWTP, fT As Single                   'Indices (unitless) used in calculation of CH4 production

        'Call VertDimCalcs to calculate vertical dimensions of each litter and detritus pool in current grid cell
        VertDimCalcs(ix, iy)

        'We want to be able to later calculate average top of litter layer over entire space, for burn management treatment.
        ' Save info here that allows that calc of average. 
        If ix = 1 And iy = 1 Then
            AGLThicknessSum = 0
            MUCKThicknessSum = 0
        End If
        AGLThicknessSum += AGLvertdim(1) - AGLvertdim(2)    'meters 
        MUCKThicknessSum += MUCKvertdim(1)                  'meters

        'Belowground litter gets transferred partially to MUCK and MSOM based on depths of detrital pools
        ' BGLtoMUCKProp is proportion that goes to MUCK pool
        If BGLvertdim(2) >= 0 Then
            BGLtoMUCKProp = 1
        Else
            BGLtoMUCKProp = BGLvertdim(1) / (BGLvertdim(1) - BGLvertdim(2))
        End If
        If BGLtoMUCKProp > 1 Or BGLtoMUCKProp < 0 Then
            ErrLast = 20
            ErrCount = ErrCount + 1
        End If

        '******************* Temperature of Litter, MUCK and MSOM and its effects on decomp rates ***********************
        '  NOTE:  this block of code needs to be here in DetritalDynCell becuase litter mass within the grid cell
        'can potentially affect temperature of MUCK and MSOM in the grid cell 
        ' Temperature of detrital pools is either air T or water T or a weighted average, based on where the 
        'water level is relative to the detrital pool vertical dimensions in this grid cell 

        'Calculate proportions of each detrital pool that is submerged in this cell on this day  
        If WaterlvlDay <= AGLvertdim(2) Then  'AGL pool is fully above water 
            AGLsubmProp = 0
        Else   'AGL pool is at least partially submerged 
            AGLsubmProp = (WaterlvlDay - AGLvertdim(2)) / (AGLvertdim(1) - AGLvertdim(2))
            If AGLsubmProp > 1 Then AGLsubmProp = 1   'max proportion submerged is 1 
        End If
        If WaterlvlDay <= BGLvertdim(2) Then  'BGL pool is fully above water 
            BGLsubmProp = 0
        Else   'BGL pool is at least partially submerged 
            BGLsubmProp = (WaterlvlDay - BGLvertdim(2)) / (BGLvertdim(1) - BGLvertdim(2))
            If BGLsubmProp > 1 Then BGLsubmProp = 1   'max proportion submerged is 1 
        End If
        If WaterlvlDay <= MUCKvertdim(2) Then  'MUCK pool is fully above water 
            MUCKsubmProp = 0
        Else   'MUCK pool is at least partially submerged 
            MUCKsubmProp = (WaterlvlDay - MUCKvertdim(2)) / (MUCKvertdim(1) - MUCKvertdim(2))
            If MUCKsubmProp > 1 Then MUCKsubmProp = 1   'max proportion submerged is 1 
        End If
        If WaterlvlDay < MSOMvertdim(2) Then  'MSOM pool is fully above water 
            MSOMsubmProp = 0
        Else   'MSOM pool is at least partially submerged 
            MSOMsubmProp = (WaterlvlDay - MSOMvertdim(2)) / (MSOMvertdim(1) - MSOMvertdim(2))
            If MSOMsubmProp > 1 Then MSOMsubmProp = 1   'max proportion submerged is 1 
        End If

        'Now that we know the proportions of detrital pools submerged, calculate average temperature in each, 
        ' based on proportion in air and proportion under water (submerged)  
        TdayAGLcell = (1 - AGLSubmProp) * TdayAir + AGLSubmProp * TdayWater
        TdayBGLcell = (1 - BGLSubmProp) * TdayAir + BGLSubmProp * TdayWater
        TdayMUCKcell = (1 - MUCKSubmProp) * TdayAir + MUCKSubmProp * TdayWater
        TdayMSOMcell = (1 - MSOMSubmProp) * TdayAir + MSOMSubmProp * TdayWater

        'Model Structure Option: whether MUCK and MSOM temperatures are lowered by litter mass during growing season  
        ' If so, calculate new temperatures based on litter mass in this grid cell
        If TeffLitOM Then
            If GSeas Then
                'This function subtracts 1 degree C for every 300 g C/m2 litter mass, or thereabouts, 
                'depending on value of parameter TeffLitOMparm
                TdayMUCKcell = TdayMUCKcell - TeffLitOMparm * GCP(ix, iy, 1) / (s ^ 2)
                TdayMSOMcell = TdayMSOMcell - TeffLitOMparm * GCP(ix, iy, 1) / (s ^ 2)
            End If
        End If

        '************ Now that we know the temperature in each detrital pool, on this day and in this cell, 
        ' calculate the temperature effect on decomposition rates in that detrital pool.  

        'This Function() Is a Q10 relationship where factor Is 0.5 at 10C, 1 at 20C, 2 at 30C
        TfactAGL = Exp(0.06931 * (TdayAGLcell - 20))
        TfactBGL = Exp(0.06931 * (TdayBGLcell - 20))

        ' Model Structure Option:  whether T affects MUCK and MSOM decay rates
        If TeffOM Then
            'This function is a Q10 relationship where factor is 0.5 at 10C, 1 at 20C, 2 at 30C
            TfactMUCK = Exp(0.06931 * (TdayMUCKcell - 20))
            TfactMSOM = Exp(0.06931 * (TdayMSOMcell - 20))
        Else
            TfactMUCK = 1 : TfactMSOM = 1
        End If

        '*************  Decomp and nutrient release flux from MSOM and MUCK *******

        'Calculate C/N ratio of MSOM (mineral soil organic matter) in GCP grid cell (avoiding division by zero)
        If GCP(ix, iy, 6) > 0 Then
            GCPhumCNrat = GCP(ix, iy, 5) / GCP(ix, iy, 6)
        Else
            GCPhumCNrat = 1000
            ErrCount = ErrCount + 1
            ErrLast = 39
        End If

        'Calculate C loss flux due to decomposition of MSOM and MUCK 
        DelGCPMUCKC = GCP(ix, iy, 9) * TfactMUCK * (1 - Exp(-kMUCK / 365))     'g C/d
        DelGCPMSOMC = GCP(ix, iy, 5) * TfactMSOM * (1 - Exp(-kMSOM / 365))     'g C/d

        'Modify decomp mass losses for MSOM and MUCK if anaerobic conditions 
        If TrailAvgWlvl > 0 Then   'This means that at least some of the muck is below trailing avg water level
            PropMUCKAn = TrailAvgWlvl / MUCKvertdim(1)
            If PropMUCKAn > 1 Then PropMUCKAn = 1
            'MUCK mass loss made less in proportion of MUCK that is anaerobic 
            DelGCPMUCKC = DelGCPMUCKC * (PropMUCKAn * MuckAnmod + (1 - PropMUCKAn))
        End If

        If TrailAvgWlvl >= 0 Then 'this means that all of the MSOM is below water level
            PropMSOMAn = 1
        ElseIf TrailAvgWlvl > MSOMvertdim(2) Then  'This means that at least some of the MSOM is below trailing avg water level
            PropMSOMAn = 1 - TrailAvgWlvl / MSOMvertdim(2)
        Else  'This means that none of the MSOM is below water level
            PropMSOMAn = 0
        End If
        DelGCPMSOMC = DelGCPMSOMC * (PropMSOMAn * MSOMAnmod + (1 - PropMSOMAn))

        'Calculate N release, mineralization flux due to decomposition 
        NGCPMUCKrelease = DelGCPMUCKC * GCP(ix, iy, 10) / GCP(ix, iy, 9)                         'g N/d
        NGCPMSOMrelease = DelGCPMSOMC * GCP(ix, iy, 6) / GCP(ix, iy, 5)                          'g N/d

        '******* Decomposition of litter, litter N mineralization, and transfers to MUCK and MSOM *********

        ' Calculate overall empirical decomposition constant for litter detritus in each grid cell.
        '  Concern is that different species may have different values of decay constant kL.  
        '  This method below is imperfect but it will work for now.  
        ' Once per year at the end of the period of early-year litterfall in the spring, 
        '  calculate a new litter mass-weighted average value of kL
        ' Then just use that for the whole following year
        ' gckcalcs array stores litter C mass in (1), and current-yr litter C mass * k in (2)
        '  divide (2) by (1) to get mass-weighted k to use for the next annual period. 
        ' Note that above + belowground litter are lumped together for this, and assumed to have same kL
        '  within a species.  
        If iday = (GSstart - 1) Then
            If gckcalcs(ix, iy, 1) > 0 Then
                kL(ix, iy) = gckcalcs(ix, iy, 2) / gckcalcs(ix, iy, 1)
            Else
                'if there is no litter mass in the cell, set k value equal to that of spp 1
                kL(ix, iy) = kLit(1)
            End If
        End If

        'Calculate C loss fluxes in above + below ground litter pools due to decomposition under aerobic conditions 
        AerDelGCPAGltr = GCP(ix, iy, 1) * TfactAGL * (1 - Exp(-kL(ix, iy) / 365))      'g C/d
        AerDelGCPBGltr = GCP(ix, iy, 7) * TfactBGL * (1 - Exp(-kL(ix, iy) / 365))      'g C/d

        'Modify decomp mass losses for AGL and BGL if anaerobic conditions 
        If TrailAvgWlvl > AGLvertdim(2) Then  'This means at least part of AGL is below trailing avg water level 
            PropAGLAn = (TrailAvgWlvl - AGLvertdim(2)) / (AGLvertdim(1) - AGLvertdim(2))
            If PropAGLAn > 1 Then PropAGLAn = 1
            DelGCPAGltr = AerDelGCPAGltr * (PropAGLAn * LitAnmod + (1 - PropAGLAn))
            'C from AGL passed to MUCK under anaerobic conditions is equal to the difference in decomp mass
            ' loss from what decomp mass loss would have been under aerobic conditions
            CAGLtoMUCKAn = AerDelGCPAGltr - DelGCPAGltr
            NAGLtoMUCKAn = CAGLtoMUCKAn * GCP(ix, iy, 2) / GCP(ix, iy, 1)
        Else  'Conditions are aerobic for the entire AGL pool
            DelGCPAGltr = AerDelGCPAGltr
        End If

        If TrailAvgWlvl > BGLvertdim(2) Then  'This means that at least part of BGL is below trailing avg water level
            PropBGLAn = (TrailAvgWlvl - BGLvertdim(2)) / (BGLvertdim(1) - BGLvertdim(2))
            If PropBGLAn > 1 Then PropBGLAn = 1
            DelGCPBGltr = AerDelGCPBGltr * (PropBGLAn * LitAnmod + (1 - PropBGLAn))
            'C from BGL passed to MUCK under anaerobic conditions is equal to the difference in decomp mass
            ' loss from what decomp mass loss would have been under aerobic conditions
            CBGLtoMUCKAn = BGLtoMUCKProp * (AerDelGCPBGltr - DelGCPBGltr)
            CBGLtoMSOMAn = (1 - BGLtoMUCKProp) * (AerDelGCPBGltr - DelGCPBGltr)
            NBGLtoMUCKAn = CBGLtoMUCKAn * GCP(ix, iy, 8) / GCP(ix, iy, 7)
            NBGLtoMSOMAn = CBGLtoMSOMAn * GCP(ix, iy, 8) / GCP(ix, iy, 7)
        Else  'Conditions are aerobic for the entire BGL pool
            DelGCPBGltr = AerDelGCPBGltr
        End If

        'Error-checking
        If NBGLtoMSOMAn < 0 Or NBGLtoMUCKAn < 0 Then
            ErrLast = 31
            ErrCount = ErrCount + 1
        End If

        '************ Transfers of C and N from litter pools to MUCK and MSOM.
        'This transfer represents humification in part.  The transfer only occurs at all if the litter in this cell 
        'has an average C:N ratio equal to or less than the C:N ratio of "humification" set by the user. 
        'C transfer to MSOM is calculated as a proportion relative to mass loss from decomp
        'N transfer to MSOM occurs at the C:N ratio of "humification" set by the user. 

        'From aboveground litter pools: aerobic transfers to MUCK  
        If GCP(ix, iy, 1) / GCP(ix, iy, 2) <= HumifCNrat Then
            CAGLtoMUCK = DelGCPAGltr * GCPhumProp                                  'g C/d
            NAGLtoMUCK = CAGLtoMUCK / HumifCNrat                                   'g N/d
        Else
            CAGLtoMUCK = 0
            NAGLtoMUCK = 0
        End If
        'From belowground litter pools: aerobic transfers to MUCK and MSOM
        If GCP(ix, iy, 7) / GCP(ix, iy, 8) <= HumifCNrat Then
            CBGLtoMUCK = DelGCPBGltr * GCPhumProp * BGLtoMUCKProp                  'g C/d
            CBGLtoMSOM = DelGCPBGltr * GCPhumProp * (1 - BGLtoMUCKProp)            'g C/d
            NBGLtoMUCK = CBGLtoMUCK / HumifCNrat                                   'g N/d
            NBGLtoMSOM = CBGLtoMSOM / HumifCNrat                                   'g N/d
        Else
            CBGLtoMUCK = 0
            NBGLtoMUCK = 0
            CBGLtoMSOM = 0
            NBGLtoMSOM = 0
        End If

        '************ Transfers of C and N from MUCK to MSOM *************************************
        ' This is a one-way transfer representing bioturbation and particulate matter eluviation
        ' Occurs only when aerobic
        '  Use structured error handling to catch any exceptions 
        Try
            CBioturb = GCP(ix, iy, 9) * (1 - Exp(-kBioturb / 365))   'g C/d
        Catch ex As Exception
            ErrCount += 1
            ErrLast = 41
        End Try

        If GCP(ix, iy, 9) > 0 Then
            Try
                NBioturb = CBioturb * GCP(ix, iy, 10) / GCP(ix, iy, 9)
            Catch ex As Exception
                ErrCount += 1
                ErrLast = 42
            End Try
            'g N/d
        Else
            NBioturb = 0
        End If

        '******* N mineralization from litter ***************************************
        'Calculate N mineralization from litter based on C loss from decomp and critical C:N ratio,
        ' while taking into account the C and N that will be transferred from litter to MSOM
        ExtraNAG = GCP(ix, iy, 2) - NAGLtoMUCK - (GCP(ix, iy, 1) - DelGCPAGltr - CAGLtoMUCK) / LitCritCNRat     'g N/d
        If ExtraNAG > 0 Then NGCPAGlitRelease = ExtraNAG
        ExtraNBG = GCP(ix, iy, 8) - NBGLtoMSOM - NBGLtoMUCK - (GCP(ix, iy, 7) - DelGCPBGltr - CBGLtoMSOM - CBGLtoMUCK) / LitCritCNRat     'g N/d
        If ExtraNBG > 0 Then NGCPBGlitRelease = ExtraNBG

        '******* Updates of pools: mass loss, transfers, N mineraliz ************************************
        ' C loss from AG litter pool is mass loss via decay and transfer to MUCK C pool. 
        GCP(ix, iy, 1) = GCP(ix, iy, 1) - DelGCPAGltr - CAGLtoMUCK - CAGLtoMUCKAn                'g C
        ' C loss from BG litter pool is mass loss via decay and transfer to MSOM C pool. 
        GCP(ix, iy, 7) = GCP(ix, iy, 7) - DelGCPBGltr - CBGLtoMUCK - CBGLtoMSOM - CBGLtoMUCKAn - CBGLtoMSOMAn    'g C
        ' N loss from litter pools is from N transferrred to MSOM and N mineralized from litter. 
        ' The algorithm for litter transfer to MSOM and N min from litter can show small round-off errors;
        ' scale back N min from litter if needed so as not to draw available N pool in litter below zero.  
        ' Aboveground litter pool: 
        GCP(ix, iy, 2) = GCP(ix, iy, 2) - NAGLtoMUCK - NAGLtoMUCKAn                             'g N
        If NGCPAGlitRelease > GCP(ix, iy, 2) Then
            If GCP(ix, iy, 2) > 0.0001 Then
                NGCPAGlitRelease = GCP(ix, iy, 2) - 0.0001 'g N/day
            Else
                NGCPAGlitRelease = 0 'g N/day
            End If
        End If
        ' Belowground litter pool: 
        GCP(ix, iy, 8) = GCP(ix, iy, 8) - NBGLtoMSOM - NBGLtoMUCK - NBGLtoMSOMAn - NBGLtoMUCKAn                'g N
        If NGCPBGlitRelease > GCP(ix, iy, 8) Then
            If GCP(ix, iy, 8) > 0.0001 Then
                NGCPBGlitRelease = GCP(ix, iy, 8) - 0.0001 'g N/day
            Else
                NGCPBGlitRelease = 0 'g N/day
            End If
        End If

        GCP(ix, iy, 2) = GCP(ix, iy, 2) - NGCPAGlitRelease    'g N
        GCP(ix, iy, 8) = GCP(ix, iy, 8) - NGCPBGlitRelease    'g N
        ' Use structured error handling
        Try
            ' N change in MSOM:  lost via MSOM N mineralization, gained via transfer from BG litter. 
            GCP(ix, iy, 6) = GCP(ix, iy, 6) - NGCPMSOMrelease + NBGLtoMSOM + NBGLtoMSOMAn + NBioturb  'g N
        Catch ex As Exception
            ErrCount += 1
            ErrLast = 42
        End Try
        'Available NH4-N pool:  gains N via mineralization from AG litter, BG litter, Muck, and MSOM 
        GCP(ix, iy, 3) = GCP(ix, iy, 3) + NGCPMUCKrelease + NGCPMSOMrelease + NGCPAGlitRelease + NGCPBGlitRelease  'g N
        'Update C in MSOM pool due to decomp and transfer from BG litter
        GCP(ix, iy, 5) = GCP(ix, iy, 5) - DelGCPMSOMC + CBGLtoMSOM + CBGLtoMSOMAn + CBioturb                'g C
        'Update C in MUCK pool due to decomp and transfer from AG litter 
        GCP(ix, iy, 9) = GCP(ix, iy, 9) - DelGCPMUCKC + CAGLtoMUCK + CBGLtoMUCK + CAGLtoMUCKAn + CBGLtoMUCKAn - CBioturb               'g C
        Try
            ' N change in MUCK:  lost via MUCK N mineralization, gained via transfer from AG and BG litter 
            GCP(ix, iy, 10) = GCP(ix, iy, 10) - NGCPMUCKrelease + NAGLtoMUCK + NAGLtoMUCKAn + NBGLtoMUCK + NBGLtoMUCKAn - NBioturb         'g N
        Catch ex As Exception
            ErrCount += 1
            ErrLast = 43
        End Try

        '************* Nitrification and denitrification *****************************
        'Now that N mineralization has added NH4-N to grid cell pools, we can calculate 
        ' rates of nitrification and denitrification in this grid cell.
        ' Note that because these happen here, before plants are able to compete for the 
        ' mineralized N, essentially nitrifiers and denitrifiers out-compete plants for available N. 
        'Upper and lower vertical limits of NH4-N pool were calculated in Sub VertDimCalcs

        If TrailAvgWlvl >= NH4vertdim(1) Then      'water level is above top of avail NH4 pool
            PropNH4Aer = 0
        ElseIf TrailAvgWlvl <= NH4vertdim(2) Then  'water level is below bottom of avail NH4 pool
            PropNH4Aer = 1
        Else                                       'water level is intermediate 
            PropNH4Aer = (NH4vertdim(1) - TrailAvgWlvl) / (NH4vertdim(1) - NH4vertdim(2))
        End If

        'Nitrification occurs to that proportion of avail NH4 pool that is aerobic
        'TfactMUCK is used as the temperature effect, subject to model structural option: 
        ' if user included T effect on OM 
        'Also: if MUCK temperature is below zero, all nitrification is shut down, 
        ' no matter whether user selected the T effect on OM or not. 
        If TdayMUCKcell < 0 Then
            NitrifDayCell = 0
        Else
            NitrifDayCell = TfactMUCK * PropNH4Aer * NitrifParm * GCP(ix, iy, 3)    'g N / day
        End If

        'Update Grid cell pools for NH4-N and NO3-N
        'First, ensure that grid cell pool can not go below zero 
        If NitrifDayCell > GCP(ix, iy, 3) Then NitrifDayCell = GCP(ix, iy, 3)

        GCP(ix, iy, 3) = GCP(ix, iy, 3) - NitrifDayCell             'g N
        GCP(ix, iy, 4) = GCP(ix, iy, 4) + NitrifDayCell             'g N   

        'Denitrification follows nitrification 
        'Calculate proportion of vertical NO3 pool that is anaerobic 
        If TrailAvgWlvl >= NO3vertdim(1) Then
            PropNO3Anaer = 1
        ElseIf TrailAvgWlvl <= NO3vertdim(2) Then
            PropNO3Anaer = 0
        Else
            PropNO3Anaer = (TrailAvgWlvl - NO3vertdim(2)) / (NO3vertdim(1) - NO3vertdim(2))
        End If

        'Heterotrophic Respiration Factor: Models the effect of labile C availability on denitrification rate.  
        'Sums the C loss from MUCK, MSOM, and BGL, converts it to g C / m2 day, and compares it to 
        ' a benchmark value entered by user, also converted to g C / m2 day.
        'Note that this also already includes a temperature effect becuase the C losses depend directly on temperature
        HetRespFact = ((DelGCPMUCKC + DelGCPMSOMC + DelGCPBGltr) / s ^ 2) / (HetRespBench / 365)       'unitless factor

        'Denitrification occurs to that proportion of avail NO3 pool that is anaerobic
        'If MUCK temperature is below 4 C, all denitrification is shut down, 
        ' no matter whether user turned on TeffOM or not. 
        If TdayMUCKcell < 4 Then
            N2DntrDayCell = 0 : N2ODntrDayCell = 0
        Else
            'For now, dentrification N flux is separated into N2-N and N2O-N fluxes using a simple proportion, 
            ' supplied in parameter file.  In future, this separation could be more mechanistic.
            ' This is the only place NDntrDayCell is used; after this it is always separated int N2-N and N2O-N fluxes 
            NDntrDayCell = HetRespFact * PropNO3Anaer * DntrParm * GCP(ix, iy, 4)    'g N / day
            N2ODntrDayCell = DntrN2OProp * NDntrDayCell
            N2DntrDayCell = (1 - DntrN2OProp) * NDntrDayCell
        End If

        'Update grid cell pool for NO3-N
        'first, ensure that grid cell pool can not go below zero.  Subtract N2 and N2O fluxes one at a time.  
        If N2DntrDayCell > GCP(ix, iy, 4) Then N2DntrDayCell = GCP(ix, iy, 4)
        GCP(ix, iy, 4) = GCP(ix, iy, 4) - N2DntrDayCell             'g N
        If N2ODntrDayCell > GCP(ix, iy, 4) Then N2ODntrDayCell = GCP(ix, iy, 4)
        GCP(ix, iy, 4) = GCP(ix, iy, 4) - N2ODntrDayCell             'g N

        '**************** Error checking ************************************************
        'Call sub to check all grid cell pools for below zero error and round-off error
        GCPEcheck(ix, iy)

        'Error checking:  check to make sure that all anaerobic proportions and submerged proportions are between 0 and 1
        If PropAGLAn < 0 Or PropAGLAn > 1 Then
            ErrCount = +1 : ErrLast = 19
        End If
        If PropBGLAn < 0 Or PropBGLAn > 1 Then
            ErrCount = +1 : ErrLast = 19
        End If
        If PropMUCKAn < 0 Or PropMUCKAn > 1 Then
            ErrCount = +1 : ErrLast = 19
        End If
        If PropMSOMAn < 0 Or PropMSOMAn > 1 Then
            ErrCount = +1 : ErrLast = 19
        End If
        If DelGCPAGltr < 0 Or DelGCPBGltr < 0 Or DelGCPMSOMC < 0 Or DelGCPMUCKC < 0 Then
            ErrCount = +1 : ErrLast = 19
        End If
        If (PropNH4Aer < 0) Or (PropNH4Aer > 1) Or (PropNO3Anaer < 0) Or (PropNO3Anaer > 1) Then
            ErrCount += 1 : ErrLast = 46
        End If
        If (AGLSubmProp < 0) Or (AGLSubmProp > 1) Or (BGLSubmProp < 0) Or (BGLSubmProp > 1) Then
            ErrCount += 1 : ErrLast = 47
        End If
        If (MUCKSubmProp < 0) Or (MUCKSubmProp > 1) Or (MSOMSubmProp < 0) Or (MSOMSubmProp > 1) Then
            ErrCount += 1 : ErrLast = 47
        End If

        '************* Bookkeeping *******************************************************
        'Increment sum of N min from GCP pools, convert to per m2
        LitNminGCPDay = LitNminGCPDay + (NGCPAGlitRelease + NGCPBGlitRelease) / GridArea      'g N/m2 d
        MSOMNminGCPDay = MSOMNminGCPDay + NGCPMSOMrelease / GridArea                          'g N/m2 d
        MUCKNminGCPDay = MUCKNminGCPDay + NGCPMUCKrelease / GridArea                          'g N/m2 d 
        NitrifDay = NitrifDay + NitrifDayCell / GridArea                                      'g N/m2 d 
        N2DntrDay = N2DntrDay + N2DntrDayCell / GridArea                                         'g N/m2 d
        N2ODntrDay = N2ODntrDay + N2ODntrDayCell / GridArea                                         'g N/m2 d

        'Increment daily C fluxes, convert to per m2
        CtoMSOMDay = CtoMSOMDay + (CBGLtoMSOM) / GridArea                        'g C/m2 d
        CtoMUCKDay = CtoMUCKDay + (CAGLtoMUCK + CBGLtoMUCK) / GridArea                        'g C/m2 d
        'Heterotrophic respiration, C loss via C mineralization during decomp, 
        ' is equal to the change in C pools from decomposition
        ' Convert units to per m2
        CHetRespCell = (DelGCPMUCKC + DelGCPMSOMC + DelGCPAGltr + DelGCPBGltr) / GridArea             'g C/m2 d

        '***** CH4 and CO2 emissions:  Divide heterotrophic C respiration into CH4-C and CO2-C fluxes.  
        ' This submodel of CH4 production is based on that of Cao et al. 1996, J Geophys Research: Atm 101(D9):14399-14414.

        'fWTP is an index from zero to 1 that lowers CH4 production based on non-ideal conditions related to water level (wtp = 'water table position')
        'Using trailing average water level here, and subtract the thickness of the Muck layer so that it is water level with respect to the 
        'top of the Muck.  Also convert water level units from m to cm. 
        fWTP = 0.383 * Exp(0.096 * 100 * (TrailAvgWlvl - MUCKvertdim(1)))
        'Ensure that this index can not go greater than 1 or less than zero
        If fWTP > 1 Then fWTP = 1
        If fWTP < 0 Then fWTP = 0

        'fT is an index from zero to 1 that lowers CH4 production based on water temperature 
        'fT has max value of 1 at 30C, has a Q10 form, and a value of 0.12 at 0 degrees C
        'If water temperature is zero or below, shut down CH4 production by setting this index to zero
        If TdayWater > 0 Then
            fT = Exp(0.0693 * TdayWater) / 8
        Else
            fT = 0
        End If
        'Ensure that this index can not go greater than 1 or less than zero
        If fT > 1 Then fT = 1
        If fT < 0 Then fT = 0

        'The proportion of heterotrophic C respiration that is CH4-C production is a max value under ideal conditions, 
        ' CH4P0 (set by the user in the parameter input file), lowered by the two indices based on 
        ' trailing average water level and water temperature 
        CH4CRespProp = CH4P0 * fWTP * fT

        'Now that we have production of CH4 we need to reduce this based on CH4 oxidation in the wetland.  
        'There are two constant proportions for CH4 oxidation from the user in the Parameter input file. 
        'Use the appropriate one depending on the water level. In so doing, assume any CH4 oxidized is 
        'simply converted back to CO2, so it just lowers the proportion of heterotrophic respiration 
        'allocated to CH4-C vs CO2-C 

        'If trailing average water level is above the top of the Muck, consider it inundated
        If TrailAvgWlvl > MUCKvertdim(1) Then
            CH4CRespProp = CH4CRespProp * (1 - CH4oxIn)
            'If trailing avg water level is below the top of the Muck but above the bottom of the Muck, 
            'consider it partially aerobic
        ElseIf TrailAvgWlvl > 0 Then
            CH4CRespProp = CH4CRespProp * (1 - CH4oxAer * (1 - PropMUCKAn))
            'If trealing avg water level is below zero, consider Muck fully aerobic 
        Else
            CH4CRespProp = CH4CRespProp * (1 - CH4oxAer)
        End If

        'Divide C flux from heterotrophic respiration in this cell, this day, into CO2-C and CH4-C fluxes                     
        CO2CRespDay = CO2CRespDay + (1 - CH4CRespProp) * CHetRespCell
        CH4CRespDay = CH4CRespDay + CH4CRespProp * CHetRespCell

    End Sub
    Private Sub GCPEcheck(ByVal ix As Short, ByVal iy As Short)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  Zero out any small round-off errors that caused Gric cell C or N pools to go below zero
        'At same time, record an error if the pool went below zero by more than a round-off error
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        If GCP(ix, iy, 1) < 0 Then
            If GCP(ix, iy, 1) < -0.00001 Then
                ErrCount = ErrCount + 1
                ErrLast = 21
            End If
            GCP(ix, iy, 1) = 0
        End If
        If GCP(ix, iy, 2) < 0 Then
            If GCP(ix, iy, 2) < -0.00001 Then
                ErrCount = ErrCount + 1
                ErrLast = 22
            End If
            GCP(ix, iy, 2) = 0
        End If
        If GCP(ix, iy, 3) < 0 Then
            If GCP(ix, iy, 3) < -0.00001 Then
                ErrCount = ErrCount + 1
                ErrLast = 23
            End If
            GCP(ix, iy, 3) = 0
        End If
        If GCP(ix, iy, 4) < 0 Then
            If GCP(ix, iy, 4) < -0.00001 Then
                ErrCount = ErrCount + 1
                ErrLast = 24
            End If
            GCP(ix, iy, 4) = 0
        End If
        If GCP(ix, iy, 5) < 0 Then
            If GCP(ix, iy, 5) < -0.00001 Then
                ErrCount = ErrCount + 1
                ErrLast = 25
            End If
            GCP(ix, iy, 5) = 0
        End If
        If GCP(ix, iy, 6) < 0 Then
            If GCP(ix, iy, 6) < -0.00001 Then
                ErrCount = ErrCount + 1
                ErrLast = 26
            End If
            GCP(ix, iy, 6) = 0
        End If
        If GCP(ix, iy, 7) < 0 Then
            If GCP(ix, iy, 7) < -0.00001 Then
                ErrCount = ErrCount + 1
                ErrLast = 27
            End If
            GCP(ix, iy, 7) = 0
        End If
        If GCP(ix, iy, 8) < 0 Then
            If GCP(ix, iy, 8) < -0.00001 Then
                ErrCount = ErrCount + 1
                ErrLast = 28
            End If
            GCP(ix, iy, 8) = 0
        End If
        If GCP(ix, iy, 9) < 0 Then
            If GCP(ix, iy, 9) < -0.00001 Then
                ErrCount = ErrCount + 1
                ErrLast = 29
            End If
            GCP(ix, iy, 9) = 0
        End If
        If GCP(ix, iy, 10) < 0 Then
            If GCP(ix, iy, 10) < -0.00001 Then
                ErrCount = ErrCount + 1
                ErrLast = 30
            End If
            GCP(ix, iy, 10) = 0
        End If

    End Sub
    Private Sub WaterLvlCalc(ByVal iyr As Short, ByVal iday As Short)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        'Calculate trailing average in water level
        ' For now, the days over which this is averaged is hard-wired in the code here 
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        Dim index As Short                   'integer used for loop
        Dim Waterlvlsum As Single            'Water level sum used in calculation of average

        ' Array WaterlvlLast5days stores water level for today plus the last 4 days, with subscript 1 = today, 2 = yesterday, etc. 

        For index = 4 To 1 Step -1
            WaterlvlLast5days(index + 1) = WaterlvlLast5days(index)
        Next
        WaterlvlLast5days(1) = Waterlvl(iyr, iday)
        If iyr = 1 And iday < 5 Then
            TrailAvgWlvl = 0
        Else
            Waterlvlsum = 0
            For index = 1 To 5
                Waterlvlsum = Waterlvlsum + WaterlvlLast5days(index)
            Next
            TrailAvgWlvl = Waterlvlsum / 5
        End If

    End Sub
    Private Sub FloodingMortality(ByVal iyr As Short, ByVal iday As Short)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  There are two types of mortality due to flooding. This sub does the first, which occurs only during 
        ' the growing season.  
        '  Track how long each individual aboveground plant stem has been continuously submerged via flooding. 
        '  If this period continuously submerged is greater than the species-specific flooding mortality
        '  parameter, kill the aboveground stem.  
        '
        '  Called by MondrianMain, each day during the growing season.    
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim ispp As Short
        Dim iind As Integer
        Dim ixGridLoc, iyGridLoc As Short        'Grid cell location of individual plant

        'Flooding mortality #1 : individual submerged for certain number of continuous days 
        'For each species, for each individual, test to see if there is an aboveground stem.  
        For ispp = 1 To 5
            For iind = 1 To iindmax(ispp)
                'Test to see if there is an aboveground stem
                If PIP(ispp, iind, 1) > 0 Then
                    'If there is an aboveground stem, figure out which grid cell the individual plant is in 
                    'First grid cell is numbered (1,1); see notes
                    ixGridLoc = Int(pil(ispp, iind, 1) / s) + 1
                    iyGridLoc = Int(pil(ispp, iind, 2) / s) + 1

                    'Now that we know what grid cell it is in, call VertDimCalcs to calculate the 
                    ' thickness of the MUCK pool in that grid cell 
                    VertDimCalcs(ixGridLoc, iyGridLoc)

                    'Now we want to know if the water level on this date is above the current plant height. 
                    'To compare against water level, we have to add the height of the aboveground stem to the 
                    ' MUCK thickness, because the base of the aboveground stem is at the top of the MUCK layer
                    ' and this is how plant height is defined in our vertical dimension definitions.  
                    If Phgt(ispp, iind) + MUCKvertdim(1) <= Waterlvl(iyr, iday) Then
                        'This individual plant is submerged on this date. 
                        ' increment the days submerged by 1.  
                        Pdsubmerged(ispp, iind) += 1
                        ' increment LFA counter #7
                        LFASpp(ispp, 7) += 1
                    Else
                        'This individual plant is not fully submerged on this date.  
                        ' Re-set the days submerged to zero.  
                        Pdsubmerged(ispp, iind) = 0
                    End If

                    'If the days submerged is greater than the species-specific parameter for that species, then 
                    ' kill the aboveground plant stem due to flooding. The belowground rhizome can continue to live. 
                    If Pdsubmerged(ispp, iind) >= FloodMortDays(ispp) Then
                        'Kill the aboveground stem. 
                        AGMortality(ispp, iind)
                        ' increment LFA counter #8:  records numbers of stems killed in this species due to flooding
                        LFASpp(ispp, 8) += 1
                    End If

                End If  'test of whether there is an aboveground stem

            Next
        Next

    End Sub

    Private Sub FloodingMortality2(ByVal iyr As Short, ByVal iday As Short)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  There are two types of mortality due to flooding. This sub does Flooding mortality #2, which occurs  
        ' from long-term flooding above a threshold depth.  For tracking that, this sub needs to be called 
        ' every day even outside of the growing season.  
        '  Track whether the water level has been above a certain long-term threshold for 2 years.  
        '  If it stays above that long-term flooding theshold for 2 years, kill all plants, both stems and rhizomes. 
        '
        '  Called by MondrianMain.    
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        'For simplicity, MUCK depth is not taken into consideration here.  
        If Waterlvl(iyr, iday) >= 1.2 Then
            LTfloodtally += 1  'increment the tally if water level stays above long-term threshold 
        Else
            LTfloodtally = 0   're-set to zero if water level goes below threshold 
        End If
        If LTfloodtally >= 730 Then
            'Water level has been deeper than the long-term threshold for 2 years.  Kill all plants, stems and rhizomes. 
            For ispp = 1 To 5
                For iind = 1 To iindmax(ispp)
                    'Kill the aboveground stem. 
                    'Check to see if there is a living stem first, to speed model operation. 
                    If PIP(ispp, iind, 1) > 0 Then
                        AGMortality(ispp, iind)
                        ' increment LFA counter #8:  records numbers of stems killed in this species due to flooding
                        LFASpp(ispp, 8) += 1
                    End If
                    'Kill the belowground rhizome.  
                    'Check to see if there is a living individual there first, to speed model operation. 
                    If PIP(ispp, iind, 3) > 0 Then BGMortality(ispp, iind)
                Next
            Next
        End If

    End Sub
    Private Sub DetritalExport()
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  Calculate losses of detritus from grid cell pools via hydrologic export from wetlands
        '  assumed to be during storms.  Comes from aboveground litter pools.   
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim ix, iy As Short
        Dim CDetExp As Single      'C exported from a grid cell, g C/day
        Dim NDetONExp As Single    'N exported from a grid cell as organic N, g N/day
        Dim CDetExpDay As Single   'C export summed over all grid cells, g C/day
        Dim NDetExpDay As Single   'N export, organic plus inorganic, summed over all grid cells, g C/day

        For ix = 1 To GridixMax
            For iy = 1 To GridiyMax

                'Calculate quantities to export, as proportions of grid cell pool sizes
                CDetExp = GCP(ix, iy, 1) * DetExpProp / 365      'g C
                NDetONExp = GCP(ix, iy, 2) * DetExpProp / 365    'g N

                'Subtract quantities from grid cell pools
                GCP(ix, iy, 1) = GCP(ix, iy, 1) - CDetExp        'g C/cell day
                GCP(ix, iy, 2) = GCP(ix, iy, 2) - NDetONExp      'g N/cell day

                'Bookkeeping:  keep track of how much was exported
                CDetExpDay = CDetExpDay + CDetExp                'g C/day
                NDetExpDay = NDetExpDay + NDetONExp              'g N/day

            Next
        Next

        'Bookkeeping:  keep track of how much is exported annually
        CdetExpYr = CdetExpYr + CDetExpDay / GridArea             'g C/m2 y
        NdetexpYr = NdetexpYr + NDetExpDay / GridArea             'g N/m2 y

        'EcosCinout, EcosNinout for mass balance checks
        EcosCinout = EcosCinout - CDetExpDay                      'g C
        EcosNinout = EcosNinout - NDetExpDay                      'g N

    End Sub

    Private Sub Printout(ByVal iinsubset As Short, ByVal irun As Short, ByVal iyr As Short, ByVal iday As Short)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        '  Print results to text output files 
        '  This Sub is called each day of every year; results can be printed daily, yearly, or
        ' every 5 years, or just at the end of the model run, or whatever is desired   
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim TabLoc(150) As Short    'tab locations for printing results
        Dim iloc, ispp As Short
        Dim iind, iparent As Integer
        Dim PBGCNN(5) As Single     'temporary variable for printing
        Dim RGRLgtModAvg(5) As Single  'temporary variable for printing
        Dim LRParaChangeHeadings As String
        Dim Nreten, Nretenland As Single  'temporary variables for printing

        ' Assemble header to be used on multiple output files, including comments
        ' transferred from input files 
        If ((irun = 1) And (iyr = 1) And (iday = 1) And (iinsubset = 1)) Then

            '************ Organize user comments, run date/time, and file identifiers ******************
            ' Stored in array of strings: OutputComments()
            ' User comments were previously saved when text files were read in 
            OutputComments(1) = "Mondrian model results: date " & DateString & " time " & TimeString
            OutputComments(2) = "*** User comments from the seed input file:"
            '  OutputComments(3) = 1st line of comments from seed input file
            '  OutputComments(4) = 2nd line of comments from seed input file 
            OutputComments(5) = "*** User comments from the parameter input file:"
            '  OutputComments(6) = 1st comment line from parameter input file
            '  OutputComments(7) = 2nd comment line from parameter input file
            OutputComments(8) = "*** User comments from the batch input file (large sets of runs only):"
            '  OutputComments(9) = 1st comment line from batch text input file
            '  OutputComments(10) = 2nd comment line from batch text input file
            OutputComments(11) = "*** User comments from the scenario input file:"
            '  OutputComments(12) = 1st comment line from scenario text input file
            '  OutputComments(13) = 2nd comment line from scenario text input file

            If LargeRunSet = False Then
                OutputComments(9) = "n/a"
                OutputComments(10) = "n/a"
            End If
            OutputComments(14) = "" 'blank, for separation 
            '  OutputComments(15) = description of which output file this is (varies by output file, below)  
            OutputComments(16) = "" 'blank, for separation from the rest of the header 

        End If  'test If ((irun = 1) And (iyr = 1) And (iday = 1) And (iinsubset = 1)) 

        '**************************************************************************************************************
        ' Print results to outpout file 11, main master text output file
        ' This works both for single model runs and large sets of model runs. 
        ' For large sets of model runs, the header and column headings are printed only for the first model run, 
        '  then with each subsequent model run, results are just appended to the end of the file.  
        '**************************************************************************************************************
        TabLoc(1) = 1
        For iloc = 2 To 110
            TabLoc(iloc) = 10 * (iloc - 1)
        Next
        ' For names of parameters varied in large sets of runs, outlined in batch text file, consolidate first into one string. 
        ' This needs to be done once in every model run, because these change in each model run when in batch mode. 
        If LargeRunSet Then
            LRParaChangeHeadings = LRParaChangeName(1) & ", " & LRParaChangeName(2) & ", " & LRParaChangeName(3) & ", " & LRParaChangeName(4) & ", " & LRParaChangeName(5) & ", " & LRParaChangeName(6) & ", " & LRParaChangeName(7) & ", " & LRParaChangeName(8) & ", " &
                LRParaChangeName(9) & ", " & LRParaChangeName(10) & ", " & LRParaChangeName(11) & ", " & LRParaChangeName(12) & ", " & LRParaChangeName(13) & ", " & LRParaChangeName(14) & ", " & LRParaChangeName(15)
        Else
            LRParaChangeHeadings = "n/a-1,    n/a-2,    n/a-3,    n/a-4,    n/a-5,    n/a-6,    n/a-7,    n/a-8,    n/a-9,   n/a-10,   n/a-11,   n/a-12,   n/a-13,   n/a-14,   n/a-15"
        End If

        If ((irun = 1) And (iyr = 1) And (iday = 365) And (iinsubset = 1)) Then
            OutputComments(15) = "This is the master main output file."
            'Print the full header including comments transferred from input files 
            For iloc = 1 To 16
                PrintLine(11, OutputComments(iloc))
            Next iloc
            'Print definitions of column headings
            PrintLine(11, "*** Definitions of column headings in order: (for more info. see User Guide)")
            PrintLine(11, "year:  year of model run")
            PrintLine(11, "")
            PrintLine(11, "*** Species specific results:  ")
            PrintLine(11, "wrdens-1: winter rhizome density of species 1 [n/m2]")
            PrintLine(11, "wrdens-2: winter rhizome density of species 2 [n/m2]")
            PrintLine(11, "wrdens-3: winter rhizome density of species 3 [n/m2]")
            PrintLine(11, "wrdens-4: winter rhizome density of species 4 [n/m2]")
            PrintLine(11, "wrdens-5: winter rhizome density of species 5 [n/m2]")
            PrintLine(11, "gsdens-1: growing season stem density (living) of species 1 on date MGSPrintDay [n/m2]")
            PrintLine(11, "gsdens-2: growing season stem density (living) of species 2 on date MGSPrintDay [n/m2]")
            PrintLine(11, "gsdens-3: growing season stem density (living) of species 3 on date MGSPrintDay [n/m2]")
            PrintLine(11, "gsdens-4: growing season stem density (living) of species 4 on date MGSPrintDay [n/m2]")
            PrintLine(11, "gsdens-5: growing season stem density (living) of species 5 on date MGSPrintDay [n/m2]")
            PrintLine(11, "ABNPP-1:  above + belowground NPP for species 1 [gC/m2y]")
            PrintLine(11, "ABNPP-2:  above + belowground NPP for species 2 [gC/m2y]")
            PrintLine(11, "ABNPP-3:  above + belowground NPP for species 3 [gC/m2y]")
            PrintLine(11, "ABNPP-4:  above + belowground NPP for species 4 [gC/m2y]")
            PrintLine(11, "ABNPP-5:  above + belowground NPP for species 5 [gC/m2y]")
            PrintLine(11, "ANPP-1:   aboveground NPP for species 1 [gC/m2y]")
            PrintLine(11, "ANPP-2:   aboveground NPP for species 2 [gC/m2y]")
            PrintLine(11, "ANPP-3:   aboveground NPP for species 3 [gC/m2y]")
            PrintLine(11, "ANPP-4:   aboveground NPP for species 4 [gC/m2y]")
            PrintLine(11, "ANPP-5:   aboveground NPP for species 5 [gC/m2y]")
            PrintLine(11, "AGC/ind1: average aboveground (stem) C per individual with nonzero aboveground (stem) C on date MGSPrintDay: species 1 [gC]")
            PrintLine(11, "AGC/ind2: average aboveground (stem) C per individual with nonzero aboveground (stem) C on date MGSPrintDay: species 2 [gC]")
            PrintLine(11, "AGC/ind3: average aboveground (stem) C per individual with nonzero aboveground (stem) C on date MGSPrintDay: species 3 [gC]")
            PrintLine(11, "AGC/ind4: average aboveground (stem) C per individual with nonzero aboveground (stem) C on date MGSPrintDay: species 4 [gC]")
            PrintLine(11, "AGC/ind5: average aboveground (stem) C per individual with nonzero aboveground (stem) C on date MGSPrintDay: species 5 [gC]")
            PrintLine(11, "PBGCNN-1: spp 1 PBGCNN (proportion of belowground C and N needed to make a stem)")
            PrintLine(11, "RGRLM-1:  spp 1 RGRLM (Relative Growth Rate light modifier, average across this species")
            PrintLine(11, "PBGCNN-2: spp 2 PBGCNN")
            PrintLine(11, "RGRLM-2:  spp 2 RGRLM")
            PrintLine(11, "PBGCNN-3: spp 3 PBGCNN")
            PrintLine(11, "RGRLM-3:  spp 3 RGRLM")
            PrintLine(11, "PBGCNN-4: spp 4 PBGCNN")
            PrintLine(11, "RGRLM-4:  spp 4 RGRLM")
            PrintLine(11, "PBGCNN-5: spp 5 PBGCNN")
            PrintLine(11, "RGRLM-5:  spp 5 RGRLM")
            PrintLine(11, "")
            PrintLine(11, "*** C and N pools:  ")
            PrintLine(11, "AGlit-C:  Aboveground litter C [gC/m2] pool averaged over all grid cells")
            PrintLine(11, "AGlit-N:  Aboveground litter N [gN/m2] pool averaged over all grid cells")
            PrintLine(11, "AGlit-t:  Aboveground litter layer thickness [meters], averaged over all grid cells")
            PrintLine(11, "BGlit-C:  Belowground litter C [gC/m2] pool averaged over all grid cells")
            PrintLine(11, "BGlit-N:  Belowground litter N [gN/m2] pool averaged over all grid cells")
            PrintLine(11, "BGlit-t:  Belowground litter layer thickness [meters], averaged over all grid cells")
            PrintLine(11, "MUCK-C:   MUCK C [gC/m2] pool averaged over all grid cells")
            PrintLine(11, "MUCK-N:   MUCK N [gN/m2] pool averaged over all grid cells")
            PrintLine(11, "MUCK-t:   MUCK thickness [meters] averaged over all grid cells")
            PrintLine(11, "MSOM-C:   Mineral soil organic matter C [gC/m2] pool averaged over all grid cells")
            PrintLine(11, "MSOM-N:   Mineral soil organic matter N [gN/m2] pool averaged over all grid cells")
            PrintLine(11, "GCP-N:    Sum of available inorganic N: NH4-N + NO3-N [gN/m2] pools, averaged over all grid cells ")
            PrintLine(11, "RNP-N:    Sum of available inorganic N: NH4-N + NO3-N [gN/m2] in the regional nutrient pool")
            PrintLine(11, "Ecos-C:   Sum of all C mass in the ecosystem: living + detritus [gC/m2]")
            PrintLine(11, "")
            PrintLine(11, "*** N fluxes:  ")
            PrintLine(11, "NH4-in:   NH4 flux into the ecosystem from all sources expressed as NH4-N [gN/m2y]")
            PrintLine(11, "NO3-in:   NO3 flux into the ecosystem from all sources expressed as NO3-N [gN/m2y]")
            PrintLine(11, "NH4-out:  NH4 in hydrologic flux leaving the ecosystem expressed as NH4-N [gN/m2y]")
            PrintLine(11, "NO3-out:  NO3 in hydrologic flux leaving the ecosystem expressed as NO3-N [gN/m2y]")
            PrintLine(11, "Ndexp:    Export of N in detritus leaving ecosystem in organic form [gN/m2y]")
            PrintLine(11, "Nmin-l:   N mineralization from litter pools [gN/m2y]")
            PrintLine(11, "Nmin-M:   N mineralization from MUCK pool [gN/m2y]")
            PrintLine(11, "Nmin-S:   N mineralization from MSOM pool [gN/m2y]")
            PrintLine(11, "Nitrif:   Nitrification flux [gN/m2y]")
            PrintLine(11, "N2-dntr:  N2-N Denitrification flux [gN/m2y]")
            PrintLine(11, "N2O-dntr: N2O-N Denitrification flux [gN/m2y]")
            PrintLine(11, "Ntrans:   NH4-N + NO3-N transfers (diffusion) between grid cells and regional pools [gN/m2y]")
            PrintLine(11, "NH4upt:   NH4 uptake by plants expressed as NH4-N [gN/m2y] averaged over all grid cells")
            PrintLine(11, "NO3upt:   NO3 uptake by plants expressed as NO3-N [gN/m2y] averaged over all grid cells")
            PrintLine(11, "NAGlitf:  organic N flux in production of aboveground litter [gN/m2y] averaged over all grid cells")
            PrintLine(11, "NBGlitf:  organic N flux in production of belowground litter [gN/m2y] averaged over all grid cells")
            PrintLine(11, "Nret-l:   ecosystem N retention of land N influx (precip not included as an influx); unitless")
            PrintLine(11, "Nret:     ecosystem N retention of all N influxes including precipitation; unitless")
            PrintLine(11, "")
            PrintLine(11, "*** C fluxes:  ")
            PrintLine(11, "ABNPP:    Above + below ground NPP of entire plant community [gC/m2y]")
            PrintLine(11, "NEP:      Net Ecosystem Productivity [gC/m2y]")
            PrintLine(11, "NEE:      Net Ecosystem Exchange of CO2 only [gC/m2y]")
            PrintLine(11, "CO2rh:    CO2-C flux from heterotrophic respiration [gC/m2y]")
            PrintLine(11, "CH4rh:    CH4-C emission flux from heterotrophic respiration (CH4 production minus oxidation) [gC/m2y]")
            PrintLine(11, "AGgrow:   C flux in aboveground growth of entire plant community [gC/m2y]")
            PrintLine(11, "BGalloc:  C flux in belowground allocation in entire plant community [gC/m2y]")
            PrintLine(11, "CAGlitf:  C flux in production of aboveground litter [gC/m2y]")
            PrintLine(11, "CBGlitf:  C flux in production of belowground litter [gC/m2y]")
            PrintLine(11, "Cdexp:    Export of C in detritus leaving ecosystem in organic form [gC/m2y]")
            PrintLine(11, "CtoMSOM:  C transferred to Mineral Soil Organic matter from litter and MUCK [gC/m2y]")
            PrintLine(11, "CtoMUCK:  C transferred to MUCK from litter pools [gC/m2y]")
            PrintLine(11, "")
            PrintLine(11, "*** Error checking:  ")
            PrintLine(11, "Cbal:     C mass balance check [g C]")
            PrintLine(11, "Nbal:     N mass balance check [g N]")
            PrintLine(11, "Errct:    Error counter (if 0 then no errors counted)")
            PrintLine(11, "Errlast:  Identifier for last error that was logged (if 0 then no errors logged)")
            PrintLine(11, "")
            PrintLine(11, "*** Run indentifier indices")
            PrintLine(11, "sto.rep:  run number in a set of stochastic replicates")
            PrintLine(11, "sppcomb:  species combination identifier number; used to chunk large run sets")
            PrintLine(11, "iinsset:  i in subset:  run number identifier used to chunk large run sets")
            PrintLine(11, "After these indices: up to 15 variables may be printed that represent parameter changes in batch sets")
            PrintLine(11, "")

            ' Print column headings.  Note that these are hand-aligned with the tabs for the printing of results in columns. 
            PrintLine(11, "year,    wrdens-1, wrdens-2, wrdens-3, wrdens-4, wrdens-5, gsdens-1, gsdens-2, gsdens-3, gsdens-4, gsdens-5, ABNPP-1,  ABNPP-2,  ABNPP-3,  ABNPP-4,  ABNPP-5,  ANPP-1,   ANPP-2,   ANPP-3,   ANPP-4,   ANPP-5,   AGC/ind1, AGC/ind2, AGC/ind3, AGC/ind4, AGC/ind5, PBGCNN-1, RGRLM-1,  PBGCNN-2, RGRLM-2,  PBGCNN-3, RGRLM-3,  PBGCNN-4, RGRLM-4,  PBGCNN-5, RGRLM-5,  AGlit-C,  AGlit-N,  AGlit-t,  BGlit-C,  BGlit-N,  BGlit-t,  MUCK-C,   MUCK-N,   MUCK-t,   MSOM-C,   MSOM-N,   GCP-N,    RNP-N,    Ecos-C,   NH4-in,   NO3-in,   NH4-out,  NO3-out,  Ndexp,    Nmin-l,   Nmin-M,   Nmin-S,   Nitrif,   N2-dntr,  N2O-dntr, Ntrans,   NH4upt,   NO3upt,   NAGlitf,  NBGlitf,  Nret-l,   Nret,     ABNPP,    NEP,      NEE,      CO2rh,    CH4rh,    AGgrow,   BGalloc,  CAGlitf,  CBGlitf,  Cdexp,    CtoMSOM,  CtoMUCK,  CBal,     NBal,     ErrCt,    ErrLast,  sto.rep,  sppcomb,  iinsset,  " & LRParaChangeHeadings)
        End If
        ' Once per year, at end of each year, print all model results in columns 
        If iday = 365 Then
            'Calculate PGGCNN and RGRLgtModAvg by species, avoiding division by zero
            For ispp = 1 To 5
                If PropBGCNneedAnnNum(ispp) > 0 Then
                    PBGCNN(ispp) = PropBGCNneedAnnTot(ispp) / PropBGCNneedAnnNum(ispp)
                Else
                    PBGCNN(ispp) = 0
                End If
                If RGRLgtModNum(ispp) > 0 Then
                    RGRLgtModAvg(ispp) = RGRLgtModSum(ispp) / RGRLgtModNum(ispp)
                Else
                    RGRLgtModAvg(ispp) = 0
                End If
            Next
            Nretenland = 1 - (NH4NhydexpYr + NO3NhydexpYr + NdetexpYr + N2DntrYr + N2ODntrYr) / NinfluxLandYr
            If Nretenland < -999 Then Nretenland = -999
            Nreten = 1 - (NH4NhydexpYr + NO3NhydexpYr + NdetexpYr + N2DntrYr + N2ODntrYr) / (NH4NinfluxYr + NO3NinfluxYr)

            Print(11, TAB(TabLoc(1)), Format(iyr, "0") & ",")
            Print(11, TAB(TabLoc(2)), Format(SppRhiz(1) / GridArea, "0.0") & ",")
            Print(11, TAB(TabLoc(3)), Format(SppRhiz(2) / GridArea, "0.0") & ",")
            Print(11, TAB(TabLoc(4)), Format(SppRhiz(3) / GridArea, "0.0") & ",")
            Print(11, TAB(TabLoc(5)), Format(SppRhiz(4) / GridArea, "0.0") & ",")
            Print(11, TAB(TabLoc(6)), Format(SppRhiz(5) / GridArea, "0.0") & ",")
            Print(11, TAB(TabLoc(7)), Format(SppStems(1) / GridArea, "0.0") & ",")
            Print(11, TAB(TabLoc(8)), Format(SppStems(2) / GridArea, "0.0") & ",")
            Print(11, TAB(TabLoc(9)), Format(SppStems(3) / GridArea, "0.0") & ",")
            Print(11, TAB(TabLoc(10)), Format(SppStems(4) / GridArea, "0.0") & ",")
            Print(11, TAB(TabLoc(11)), Format(SppStems(5) / GridArea, "0.0") & ",")
            Print(11, TAB(TabLoc(12)), Format(CNPPYrSpp(1), "0.0") & ",")
            Print(11, TAB(TabLoc(13)), Format(CNPPYrSpp(2), "0.0") & ",")
            Print(11, TAB(TabLoc(14)), Format(CNPPYrSpp(3), "0.0") & ",")
            Print(11, TAB(TabLoc(15)), Format(CNPPYrSpp(4), "0.0") & ",")
            Print(11, TAB(TabLoc(16)), Format(CNPPYrSpp(5), "0.0") & ",")
            Print(11, TAB(TabLoc(17)), Format(CANPPYrSpp(1), "0.0") & ",")
            Print(11, TAB(TabLoc(18)), Format(CANPPYrSpp(2), "0.0") & ",")
            Print(11, TAB(TabLoc(19)), Format(CANPPYrSpp(3), "0.0") & ",")
            Print(11, TAB(TabLoc(20)), Format(CANPPYrSpp(4), "0.0") & ",")
            Print(11, TAB(TabLoc(21)), Format(CANPPYrSpp(5), "0.0") & ",")
            Print(11, TAB(TabLoc(22)), Format(SppAGCsum(1) / SppStems(1), "0.0000") & ",")
            Print(11, TAB(TabLoc(23)), Format(SppAGCsum(2) / SppStems(2), "0.0000") & ",")
            Print(11, TAB(TabLoc(24)), Format(SppAGCsum(3) / SppStems(3), "0.0000") & ",")
            Print(11, TAB(TabLoc(25)), Format(SppAGCsum(4) / SppStems(4), "0.0000") & ",")
            Print(11, TAB(TabLoc(26)), Format(SppAGCsum(5) / SppStems(5), "0.0000") & ",")
            Print(11, TAB(TabLoc(27)), Format(PBGCNN(1), "0.00") & ",")
            Print(11, TAB(TabLoc(28)), Format(RGRLgtModAvg(1), "0.00") & ",")
            Print(11, TAB(TabLoc(29)), Format(PBGCNN(2), "0.00") & ",")
            Print(11, TAB(TabLoc(30)), Format(RGRLgtModAvg(2), "0.00") & ",")
            Print(11, TAB(TabLoc(31)), Format(PBGCNN(3), "0.00") & ",")
            Print(11, TAB(TabLoc(32)), Format(RGRLgtModAvg(3), "0.00") & ",")
            Print(11, TAB(TabLoc(33)), Format(PBGCNN(4), "0.00") & ",")
            Print(11, TAB(TabLoc(34)), Format(RGRLgtModAvg(4), "0.00") & ",")
            Print(11, TAB(TabLoc(35)), Format(PBGCNN(5), "0.00") & ",")
            Print(11, TAB(TabLoc(36)), Format(RGRLgtModAvg(5), "0.00") & ",")
            Print(11, TAB(TabLoc(37)), Format(GCPAGlitCsum / GridArea, "0.0") & ",")
            Print(11, TAB(TabLoc(38)), Format(GCPAGlitNsum / GridArea, "0.0") & ",")
            Print(11, TAB(TabLoc(39)), Format((AGLvertdim(1) - AGLvertdim(2)), "0.000") & ",")
            Print(11, TAB(TabLoc(40)), Format(GCPBGlitCsum / GridArea, "0.0") & ",")
            Print(11, TAB(TabLoc(41)), Format(GCPBGlitNsum / GridArea, "0.0") & ",")
            Print(11, TAB(TabLoc(42)), Format((BGLvertdim(1) - BGLvertdim(2)), "0.000") & ",")
            Print(11, TAB(TabLoc(43)), Format(GCPMUCKCsum / GridArea, "0.0") & ",")
            Print(11, TAB(TabLoc(44)), Format(GCPMUCKNsum / GridArea, "0.0") & ",")
            Print(11, TAB(TabLoc(45)), Format((MUCKvertdim(1) - MUCKvertdim(2)), "0.000") & ",")
            Print(11, TAB(TabLoc(46)), Format(GCPMSOMCsum / GridArea, "0.0") & ",")
            Print(11, TAB(TabLoc(47)), Format(GCPMSOMNsum / GridArea, "0.0") & ",")
            Print(11, TAB(TabLoc(48)), Format(GCPINsum / GridArea, "0.0") & ",")
            Print(11, TAB(TabLoc(49)), Format((RNP(1) + RNP(2)) / GridArea, "0.0") & ",")
            Print(11, TAB(TabLoc(50)), Format(EcosystemC / GridArea, "0.0") & ",")
            Print(11, TAB(TabLoc(51)), Format(NH4NinfluxYr, "0.00") & ",")
            Print(11, TAB(TabLoc(52)), Format(NO3NinfluxYr, "0.00") & ",")
            Print(11, TAB(TabLoc(53)), Format(NH4NhydexpYr, "0.00") & ",")
            Print(11, TAB(TabLoc(54)), Format(NO3NhydexpYr, "0.00") & ",")
            Print(11, TAB(TabLoc(55)), Format(NdetexpYr, "0.00") & ",")
            Print(11, TAB(TabLoc(56)), Format(LitNminYr, "0.00") & ",")
            Print(11, TAB(TabLoc(57)), Format(MUCKNminYr, "0.00") & ",")
            Print(11, TAB(TabLoc(58)), Format(MSOMNminYr, "0.00") & ",")
            Print(11, TAB(TabLoc(59)), Format(NitrifYr, "0.00") & ",")
            Print(11, TAB(TabLoc(60)), Format(N2DntrYr, "0.00") & ",")
            Print(11, TAB(TabLoc(61)), Format(N2ODntrYr, "0.000") & ",")
            Print(11, TAB(TabLoc(62)), Format(NtransYr, "0.00") & ",")
            Print(11, TAB(TabLoc(63)), Format(NH4NUptakeYr, "0.00") & ",")
            Print(11, TAB(TabLoc(64)), Format(NO3NUptakeYr, "0.00") & ",")
            Print(11, TAB(TabLoc(65)), Format(NAGltrfallYr, "0.00") & ",")
            Print(11, TAB(TabLoc(66)), Format(NBGltrfallYr, "0.00") & ",")
            Print(11, TAB(TabLoc(67)), Format(Nretenland, "0.00") & ",")
            Print(11, TAB(TabLoc(68)), Format(Nreten, "0.00") & ",")
            Print(11, TAB(TabLoc(69)), Format(CNPPYr, "0.0") & ",")
            Print(11, TAB(TabLoc(70)), Format(CNEPYr, "0.0") & ",")
            Print(11, TAB(TabLoc(71)), Format(CNEEYr, "0.0") & ",")
            Print(11, TAB(TabLoc(72)), Format(CO2CRespYr, "0.0") & ",")
            Print(11, TAB(TabLoc(73)), Format(CH4CRespYr, "0.0") & ",")
            Print(11, TAB(TabLoc(74)), Format(CAGgrowthYr, "0.0") & ",")
            Print(11, TAB(TabLoc(75)), Format(CBGallocYr, "0.0") & ",")
            Print(11, TAB(TabLoc(76)), Format(CAGltrfallYr, "0.0") & ",")
            Print(11, TAB(TabLoc(77)), Format(CBGltrfallYr, "0.0") & ",")
            Print(11, TAB(TabLoc(78)), Format(CdetExpYr, "0.0") & ",")
            Print(11, TAB(TabLoc(79)), Format(CtoMSOMYr, "0.0") & ",")
            Print(11, TAB(TabLoc(80)), Format(CtoMUCKYr, "0.0") & ",")
            'Mass balance checks and error-checking 
            Print(11, TAB(TabLoc(81)), Format(Cmassbal, "0") & ",")
            Print(11, TAB(TabLoc(82)), Format(Nmassbal, "0.0") & ",")
            Print(11, TAB(TabLoc(83)), Format(ErrCount, "0") & ",")
            Print(11, TAB(TabLoc(84)), Format(ErrLast, "0") & ",")
            'Indices that refer to sets of runs
            Print(11, TAB(TabLoc(85)), Format(irun, "0") & ",")
            Print(11, TAB(TabLoc(86)), Format(sppcomb, "0") & ",")
            Print(11, TAB(TabLoc(87)), Format(iinsubset, "0") & ",")
            'Listing of parameter values that are changed in batch set of runs 
            For iloc = 1 To 15
                Print(11, TAB(TabLoc(87 + iloc)), LRParaChangeVals(iinsubset, iloc))
                If iloc < 15 Then Print(11, ", ")
            Next
            PrintLine(11, TAB(TabLoc(103)), "")
        End If

        '******************** Output files 12 to 19 are only printed if PrintSuppress = false *******************

        If PrintSuppress = False Then

            '********************************************************************************************************
            ' Print to output file 15, plant genet and ramet info. 
            ' This information about all ramets and genets prints only once, at end of model run.   
            ' This could be expanded to print every year, or multiple times per year, to follow genet and ramet info 
            ' more closely if desired. 
            '********************************************************************************************************
            TabLoc(1) = 1
            For iloc = 2 To 99
                TabLoc(iloc) = 10 * (iloc - 1)
            Next

            ' Print header and column headings only on last year, last day.  
            ' Do this only for the last run in the set, when there are multiple runs. 
            If ((irun = nStochasticRuns) And (iyr = Runyears) And (iday = 365)) Then
                'Print the full header including commments transferred from input files 
                OutputComments(15) = "Genet and ramet info for all individuals with belowground tissue, at end of model run. "
                For iloc = 1 To 16
                    PrintLine(15, OutputComments(iloc))
                Next iloc
                'Print explanations of column headings
                PrintLine(15, "Explanations of column headings:")
                PrintLine(15, "irun:  number of model run in set of stochastic reps")
                PrintLine(15, "year:  year of model run")
                PrintLine(15, "day:   day of year")
                PrintLine(15, "spp:   species number")
                PrintLine(15, "indiv: identifier index of individual; used in model code")
                PrintLine(15, "genet: genet number within this species")
                PrintLine(15, "repro: has this individual reproduced vegetatively yet?  (0 = no; 1 = yes)")
                PrintLine(15, "age [y]:  age of this individual in years (0 = less than 1 year)")
                PrintLine(15, "x loc: x coordinate; meters")
                PrintLine(15, "y loc: y coordinate; meters")
                PrintLine(15, "x gloc:  grid cell number in x dimension")
                PrintLine(15, "y gloc:  grid cell number in y dimension")
                PrintLine(15, "C BG:  C mass belowground [grams]")
                PrintLine(15, "N BG:  N mass belowground [grams]")
                PrintLine(15, "C AG:  C mass aboveground [grams]")
                PrintLine(15, "N AG:  N mass aboveground [grams]")
                PrintLine(15, "")

                'Print column headings 
                PrintLine(15, "irun,    year,     day,      spp,      indiv,    genet,    repro,    age,      x loc,    y loc,    x gloc,   y gloc,   C BG,     N BG,     C AG,     NAG")
            End If

            'print plant genet and ramet info only on last year, last day
            If ((irun = nStochasticRuns) And (iyr = Runyears) And (iday = 365)) Then
                For ispp = 1 To 5
                    ' Note this code block will only execute for spp included in this run, where iindmax(ispp) > 0
                    For iind = 1 To iindmax(ispp)
                        'test to see if there is an individual, by belowground C or by nonzero location
                        '(also serves as error check on agreement of zero belowground C and zero location)
                        If PIP(ispp, iind, 3) > 0 Or pil(ispp, iind, 1) > 0 Or pil(ispp, iind, 2) > 0 Then
                            Print(15, TAB(TabLoc(1)), Format(irun, "0") & ",")
                            Print(15, TAB(TabLoc(2)), Format(iyr, "0") & ",")
                            Print(15, TAB(TabLoc(3)), Format(iday, "0") & ",")
                            Print(15, TAB(TabLoc(4)), Format(ispp, "0") & ",")
                            Print(15, TAB(TabLoc(5)), Format(iind, "0") & ",")
                            Print(15, TAB(TabLoc(6)), Format(pgr(ispp, iind, 1), "0") & ",")
                            Print(15, TAB(TabLoc(7)), Format(pgr(ispp, iind, 3), "0") & ",")
                            Print(15, TAB(TabLoc(8)), Format(pgr(ispp, iind, 4), "0") & ",")
                            Print(15, TAB(TabLoc(9)), Format(pil(ispp, iind, 1), "0.00") & ",")
                            Print(15, TAB(TabLoc(10)), Format(pil(ispp, iind, 2), "0.00") & ",")
                            Print(15, TAB(TabLoc(11)), Format((Int(pil(ispp, iind, 1) / s) + 1), "0") & ",")
                            Print(15, TAB(TabLoc(12)), Format((Int(pil(ispp, iind, 2) / s) + 1), "0") & ",")
                            Print(15, TAB(TabLoc(13)), Format(PIP(ispp, iind, 3) + PIP(ispp, iind, 5), "0.000") & ",")
                            Print(15, TAB(TabLoc(14)), Format(PIP(ispp, iind, 4), "0.0000") & ",")
                            Print(15, TAB(TabLoc(15)), Format(PIP(ispp, iind, 1), "0.000") & ",")
                            PrintLine(15, TAB(TabLoc(16)), Format(PIP(ispp, iind, 2), "0.0000"))
                        End If
                    Next
                Next
            End If

            '******************************************************************************************************
            ' Print to output file 16, year-end values of counters in Logical Flow Analysis by Species array 
            '******************************************************************************************************
            TabLoc(1) = 1
            For iloc = 2 To 99
                TabLoc(iloc) = 10 * (iloc - 1)
            Next

            'Print header and column headings just once, at end of model run
            If ((irun = 1) And (iyr = 1) And (iday = 365)) Then
                'Print full header, including comments transferred from input files
                OutputComments(15) = "Results of Logical Flow Analysis by Species, each year. "
                For iloc = 1 To 16
                    PrintLine(16, OutputComments(iloc))
                Next iloc

                'Print detailed information about what information is contained in this printout 
                PrintLine(16, " These are counters that represent various aspects of logical flow that can be used")
                PrintLine(16, "to understand the plant physiology, phenology, and competition in the model run.")
                PrintLine(16, " Each counter is incremented every time a condition occurs, for each individual plant")
                PrintLine(16, "each day.  Each counter is zeroed out and restarted at the start of each new year.")
                PrintLine(16, "")
                PrintLine(16, " These counters are defined as follows:")
                PrintLine(16, "c1 = uptake N demand was scaled back due to limitation of available inorg N in grid cell.")
                PrintLine(16, "c1/ind = counter 1 divided by current number of individuals (winter rhizomes) in the species.")
                PrintLine(16, "")
                PrintLine(16, "c2 = there is enough rhiz C, but not enough rhiz N, to est an AG stem without parental subsidy.")
                PrintLine(16, "c2/ind = counter 2 divided by current number of individuals (winter rhizomes) in the species.")
                PrintLine(16, "")
                PrintLine(16, "c3 = there is enough rhiz N, but not enough rhiz C, to est an AG stem without parental subsidy.")
                PrintLine(16, "c3/ind = counter 3 divided by current number of individuals (winter rhizomes) in the species.")
                PrintLine(16, "")
                PrintLine(16, "c4 = there is both enough rhiz C and N at same time to estab AG stem, with or without parental subsidy.")
                PrintLine(16, "      (i.e. total number of new aboveground stems created.)")
                PrintLine(16, "c4/ind = counter 4 divided by current number of individuals (winter rhizomes) in the species.")
                PrintLine(16, "")
                PrintLine(16, "c5 = aboveground growth limited becuase not enough N is in belowground part to send up.")
                PrintLine(16, "c5/ind = counter 5 divided by current number of individuals (winter rhizomes) in the species.")
                PrintLine(16, "")
                PrintLine(16, "c6 = (not currently used).")
                PrintLine(16, "c6/ind = (not currently used).")
                PrintLine(16, "")
                PrintLine(16, "c7 = number of individual-days that a living aboveground stem was completely submerged.")
                PrintLine(16, "c7/ind = counter 7 divided by current number of individuals (winter rhizomes) in the species.")
                PrintLine(16, "")
                PrintLine(16, "c8 = number of times an individual living stem was killed because it was submerged too long.")
                PrintLine(16, "c8/ind = counter 8 divided by current number of individuals (winter rhizomes) in the species.")
                PrintLine(16, "")
                PrintLine(16, "c9 = number of times a first new daughter rhizome node was created by a terminal node.")
                PrintLine(16, "c9/ind = counter 9 divided by current number of individuals (winter rhizomes) in the species.")
                PrintLine(16, "")
                PrintLine(16, "c10 = number of times a new terminal branch and daughter rhizome node were created by a terminal node.")
                PrintLine(16, "c10/ind = counter 10 divided by current number of individuals (winter rhizomes) in the species.")
                PrintLine(16, "")
                PrintLine(16, "c11 = number of times a new lateral branch and daughter rhizome node were created by a non-terminal node.")
                PrintLine(16, "c11/ind = counter 11 divided by current number of individuals (winter rhizomes) in the species.")
                PrintLine(16, "")
                'Print column headings 
                PrintLine(16, "irun,    year,     species,  c1,       c1/ind,    c2,      c2/ind,   c3,       c3/ind,   c4,       c4/ind,   c5,       c5/ind,   c6,       c6/ind,   c7,       c7/ind,   c8,       c8/ind,   c9,       c9/ind,   c10,      c10/ind,  c11,      c11/ind")
            End If  'end of block that prints just once 

            'Start of block that prints on the last day of the year, every year 
            If iday = 365 Then
                For ispp = 1 To 5
                    Print(16, TAB(TabLoc(1)), Format(irun, "0") & ",")
                    Print(16, TAB(TabLoc(2)), Format(iyr, "0") & ",")
                    Print(16, TAB(TabLoc(3)), Format(ispp, "0") & ",")
                    If SppRhiz(ispp) > 0 Then
                        Print(16, TAB(TabLoc(4)), Format(LFASpp(ispp, 1), "0") & ",")
                        Print(16, TAB(TabLoc(5)), Format(LFASpp(ispp, 1) / SppRhiz(ispp), "0.0") & ",")
                        Print(16, TAB(TabLoc(6)), Format(LFASpp(ispp, 2), "0") & ",")
                        Print(16, TAB(TabLoc(7)), Format(LFASpp(ispp, 2) / SppRhiz(ispp), "0.0") & ",")
                        Print(16, TAB(TabLoc(8)), Format(LFASpp(ispp, 3), "0") & ",")
                        Print(16, TAB(TabLoc(9)), Format(LFASpp(ispp, 3) / SppRhiz(ispp), "0.0") & ",")
                        Print(16, TAB(TabLoc(10)), Format(LFASpp(ispp, 4), "0") & ",")
                        Print(16, TAB(TabLoc(11)), Format(LFASpp(ispp, 4) / SppRhiz(ispp), "0.0") & ",")
                        Print(16, TAB(TabLoc(12)), Format(LFASpp(ispp, 5), "0") & ",")
                        Print(16, TAB(TabLoc(13)), Format(LFASpp(ispp, 5) / SppRhiz(ispp), "0.0") & ",")
                        Print(16, TAB(TabLoc(14)), Format(LFASpp(ispp, 6), "0") & ",")
                        Print(16, TAB(TabLoc(15)), Format(LFASpp(ispp, 6) / SppRhiz(ispp), "0.00") & ",")
                        Print(16, TAB(TabLoc(16)), Format(LFASpp(ispp, 7), "0") & ",")
                        Print(16, TAB(TabLoc(17)), Format(LFASpp(ispp, 7) / SppRhiz(ispp), "0.00") & ",")
                        Print(16, TAB(TabLoc(18)), Format(LFASpp(ispp, 8), "0") & ",")
                        Print(16, TAB(TabLoc(19)), Format(LFASpp(ispp, 8) / SppRhiz(ispp), "0.00") & ",")
                        Print(16, TAB(TabLoc(20)), Format(LFASpp(ispp, 9), "0") & ",")
                        Print(16, TAB(TabLoc(21)), Format(LFASpp(ispp, 9) / SppRhiz(ispp), "0.00") & ",")
                        Print(16, TAB(TabLoc(22)), Format(LFASpp(ispp, 10), "0") & ",")
                        Print(16, TAB(TabLoc(23)), Format(LFASpp(ispp, 10) / SppRhiz(ispp), "0.00") & ",")
                        Print(16, TAB(TabLoc(24)), Format(LFASpp(ispp, 11), "0") & ",")
                        PrintLine(16, TAB(TabLoc(25)), Format(LFASpp(ispp, 11) / SppRhiz(ispp), "0.00"))
                    Else
                        PrintLine(16, "")
                    End If
                Next
            End If  'test if iday = 365

            '******************************************************************************************************
            'Print to output file #17, branching information for creating branching visualization 
            '******************************************************************************************************

            TabLoc(1) = 1
            For iloc = 2 To 99
                TabLoc(iloc) = 10 * (iloc - 1)
            Next

            If ((irun = 1) And (iyr = 1) And (iday = 1)) Then
                'Print full header, including comments transferred from input files
                OutputComments(15) = "Mondrian results needed for visualization of clonal branching patterns. "
                For iloc = 1 To 16
                    PrintLine(17, OutputComments(iloc))
                Next iloc
                'Print detailed information about what is contained in columns 
                PrintLine(17, "Width of the space; meters: " & Format((Sqrt(ncells) * s), "0.000"))
                PrintLine(17, "Information is printed for day of year: " & Format(MGSPrintDay, "0"))
                PrintLine(17, "")
                PrintLine(17, "Definitions of column labels:")
                PrintLine(17, "x coord:  x coordinate [m] of present individual (living rhizome)")
                PrintLine(17, "y coord:  y coordinate [m] of present individual (living rhizome)")
                PrintLine(17, "wrap:  Did this indiv wrap around the torus when it was created? (0 = no; 1 = yes)")
                PrintLine(17, "genet:  genet number of present individual")
                PrintLine(17, "age:  age of present individual (0 means < 1 year)")
                PrintLine(17, "AGC[g]:  aboveground C as grams C.  This is stem C.  Value of zero means no aboveground stem")
                PrintLine(17, "pxcoord:  x coordinate [m] of immediate parent (rhizome) if still living (0 = not living)")
                PrintLine(17, "pycoord:  y coordinate [m] of immediate parent (rhizome) if still living (0 = not living)")
                PrintLine(17, "")
                PrintLine(17, "irun,    year,     species,  xcoord,   ycoord,   wrap,     genet,    age [y],  AGC[g],   pxcoord,  pycoord")
            End If  'end of block to print header only once 

            'Print branching information every 5 years, beginning in year 5
            If iyr Mod BrVisFreq = 0 Then
                'Is this the day in the growing season to print branching information.  
                'Parameter MGSPrintDay is used to set this date. 
                If iday = MGSPrintDay Then
                    'For every living rhizome of every species, print info needed to reconstruct branching 
                    For ispp = 1 To 5
                        'Only execute the printing code block below if this species was included in model run
                        If SppInclude(ispp) Then
                            For iind = 1 To iindmax(ispp)
                                'Test to see if this is a living individual
                                If PIP(ispp, iind, 3) > 0 Then
                                    Print(17, TAB(TabLoc(1)), Format(irun, "0") & ",")
                                    Print(17, TAB(TabLoc(2)), Format(iyr, "0") & ",")
                                    Print(17, TAB(TabLoc(3)), Format(ispp, "0") & ",")
                                    Print(17, TAB(TabLoc(4)), Format(pil(ispp, iind, 1), "0.000") & ",")    'x coord location of the present individual
                                    Print(17, TAB(TabLoc(5)), Format(pil(ispp, iind, 2), "0.000") & ",")    'y coord location of the present individual 
                                    Print(17, TAB(TabLoc(6)), Format(pgr(ispp, iind, 2), "0") & ",")        'info about whether this indiv wrapped around the torus 
                                    Print(17, TAB(TabLoc(7)), Format(pgr(ispp, iind, 1), "0") & ",")        'genet number of the present individual
                                    Print(17, TAB(TabLoc(8)), Format(pgr(ispp, iind, 4), "0") & ",")        'age (years) of the present individual
                                    Print(17, TAB(TabLoc(9)), Format(PIP(ispp, iind, 1), "0.000") & ",")    'aboveground (stem) C of the present individual
                                    'Retrieve the present individual's immediate parent identifier, store in a temporary variable
                                    iparent = pgr(ispp, iind, 6)
                                    If iparent > 0 Then  'Test to see if this individual ever had an immediate parent
                                        'Test to see if the immediate parent is still alive 
                                        If PIP(ispp, iparent, 3) > 0 Then
                                            Print(17, TAB(TabLoc(10)), Format(pil(ispp, iparent, 1), "0.000") & ",")    'x coord location of the immediate parent
                                            PrintLine(17, TAB(TabLoc(11)), Format(pil(ispp, iparent, 2), "0.000"))    'y coord location of the immediate parent 
                                        Else
                                            Print(17, TAB(TabLoc(10)), 0.0 & ",")
                                            PrintLine(17, TAB(TabLoc(11)), 0.0)
                                        End If
                                    Else
                                        Print(17, TAB(TabLoc(10)), 0.0 & ",")
                                        PrintLine(17, TAB(TabLoc(11)), 0.0)
                                    End If

                                End If   'Test if this is a living individual 
                            Next iind
                        End If   'Test whether this species was included in the model run
                    Next ispp
                End If
            End If  'Test if this is a year to print branching information 

        End If   'test of PrintSuppress = false

    End Sub
    Private Sub Sums(ByVal iday As Short)
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        ' This sub is called every day, from Main, prior to Printout
        ' this sub calculates sums for Printout of aggregated pools
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        Dim ispp, ix, iy As Short
        Dim iind As Integer

        'Sum the C and N pools for all plant internal pools in absolute g
        'Count the number of ramets by species
        PIPCsum = 0 : PIPNsum = 0

        For ispp = 1 To 5
            SppRhiz(ispp) = 0 : SppCsum(ispp) = 0 : SppNsum(ispp) = 0
            'SppAGCsum(ispp) and SppStems(ispp) are summed on one particular day; don't re-initialize to zero 
            ' on days after that 
            If iday = MGSPrintDay Then
                SppAGCsum(ispp) = 0 : SppStems(ispp) = 0
            End If
            ' Note this code block will only execute for spp included in this run, where iindmax(ispp) > 0
            For iind = 1 To iindmax(ispp)
                'Calculate sums over individuals within a species
                'Use individual belowground C mass > 0 as a test for existence of an individual rhizome
                If PIP(ispp, iind, 3) > 0 Then
                    SppRhiz(ispp) = SppRhiz(ispp) + 1                  'number
                    'Note that Spp C and N sums include only living tissue:  Any standing dead litter C and N is excluded 
                    SppCsum(ispp) = SppCsum(ispp) + PIP(ispp, iind, 1) + PIP(ispp, iind, 3) + PIP(ispp, iind, 5)   'g C
                    SppNsum(ispp) = SppNsum(ispp) + PIP(ispp, iind, 2) + PIP(ispp, iind, 4)                        'g N
                End If
                'Count the number of living stems on one day in mid growing season 
                If iday = MGSPrintDay Then
                    If PIP(ispp, iind, 1) > 0 Then
                        SppStems(ispp) = SppStems(ispp) + 1
                        'Also sum the aboveground C, in living stems, within a species
                        SppAGCsum(ispp) = SppAGCsum(ispp) + PIP(ispp, iind, 1)            'g C
                    End If
                End If
            Next
            'Calculate total plant C and N sums by summing over species
            PIPCsum = PIPCsum + SppCsum(ispp)                              'g C
            PIPNsum = PIPNsum + SppNsum(ispp)                              'g N
        Next

        'Additional detailed bookkeeping for species 1, for model testing
        ispp = 1
        Sp1AGCsum = 0 : Sp1AGNsum = 0 : Sp1BGCsum = 0 : Sp1BGNsum = 0 : Sp1AGnumber = 0 : Sp1BGnumber = 0
        For iind = 1 To iindmax(ispp)
            'Aboveground C sum, number, and N sum
            Sp1AGCsum = Sp1AGCsum + PIP(ispp, iind, 1)
            If PIP(ispp, iind, 1) > 0 Then Sp1AGnumber = Sp1AGnumber + 1
            Sp1AGNsum = Sp1AGNsum + PIP(ispp, iind, 2)
            'Belowground C sum, number, and N sum
            Sp1BGCsum = Sp1BGCsum + PIP(ispp, iind, 3) + PIP(ispp, iind, 5)
            If PIP(ispp, iind, 3) > 0 Then Sp1BGnumber = Sp1BGnumber + 1
            Sp1BGNsum = Sp1BGNsum + PIP(ispp, iind, 4)
        Next

        'Sum grid cell C and N pools in grid cells in absolute g
        GCPAGlitCsum = 0 : GCPAGlitNsum = 0 : GCPBGlitCsum = 0 : GCPBGlitNsum = 0 : GCPINsum = 0
        GCPMUCKCsum = 0 : GCPMUCKNsum = 0 : GCPMSOMCsum = 0 : GCPMSOMNsum = 0
        For ix = 1 To GridixMax
            For iy = 1 To GridiyMax
                GCPAGlitCsum = GCPAGlitCsum + GCP(ix, iy, 1)             'g C
                GCPAGlitNsum = GCPAGlitNsum + GCP(ix, iy, 2)             'g N
                GCPBGlitCsum = GCPBGlitCsum + GCP(ix, iy, 7)             'g C
                GCPBGlitNsum = GCPBGlitNsum + GCP(ix, iy, 8)             'g N
                GCPINsum = GCPINsum + GCP(ix, iy, 3) + GCP(ix, iy, 4)    'g N
                GCPMUCKCsum = GCPMUCKCsum + GCP(ix, iy, 9)               'g C
                GCPMUCKNsum = GCPMUCKNsum + GCP(ix, iy, 10)              'g N
                GCPMSOMCsum = GCPMSOMCsum + GCP(ix, iy, 5)               'g C
                GCPMSOMNsum = GCPMSOMNsum + GCP(ix, iy, 6)               'g N
            Next
        Next

    End Sub
End Module